@@ -1,4 +1,4 @@
-use crate::config::{BindingForEvent, ColorTermusic};
+use crate::config::v2::tui::{keys::KeyBinding, theme::styles::ColorTermusic};
 use crate::invidious::{Instance, YoutubeVideo};
 use crate::podcast::{EpData, PodcastFeed, PodcastNoId};
 use crate::songtag::SongTag;
@@ -51,7 +51,7 @@ pub enum Msg {
     None,
 }
 
-#[derive(Clone, PartialEq, Eq, Debug)]
+#[derive(Clone, Copy, PartialEq, Eq, Debug)]
 pub enum XYWHMsg {
     Hide,
     MoveLeft,
@@ -77,7 +77,7 @@ pub enum DLMsg {
     FetchPhotoErr(String),
 }
 
-#[derive(Clone, PartialEq, Eq, Debug)]
+#[derive(Clone, Copy, PartialEq, Eq, Debug)]
 pub enum LyricMsg {
     LyricTextAreaBlurUp,
     LyricTextAreaBlurDown,
@@ -91,6 +91,7 @@ pub enum ConfigEditorMsg {
     PodcastSimulDownloadBlurUp,
     PodcastMaxRetriesBlurDown,
     PodcastMaxRetriesBlurUp,
+
     AlbumPhotoAlignBlurDown,
     AlbumPhotoAlignBlurUp,
     ChangeLayout,
@@ -107,12 +108,14 @@ pub enum ConfigEditorMsg {
     KeyFocus(KFMsg),
     MusicDirBlurDown,
     MusicDirBlurUp,
+
     PlaylistDisplaySymbolBlurDown,
     PlaylistDisplaySymbolBlurUp,
     PlaylistRandomTrackBlurDown,
     PlaylistRandomTrackBlurUp,
     PlaylistRandomAlbumBlurDown,
     PlaylistRandomAlbumBlurUp,
+
     LibraryForegroundBlurDown,
     LibraryForegroundBlurUp,
     LibraryBackgroundBlurDown,
@@ -123,6 +126,7 @@ pub enum ConfigEditorMsg {
     LibraryHighlightBlurUp,
     LibraryHighlightSymbolBlurDown,
     LibraryHighlightSymbolBlurUp,
+
     PlaylistForegroundBlurDown,
     PlaylistForegroundBlurUp,
     PlaylistBackgroundBlurDown,
@@ -133,42 +137,66 @@ pub enum ConfigEditorMsg {
     PlaylistHighlightBlurUp,
     PlaylistHighlightSymbolBlurDown,
     PlaylistHighlightSymbolBlurUp,
+
     ProgressForegroundBlurDown,
     ProgressForegroundBlurUp,
     ProgressBackgroundBlurDown,
     ProgressBackgroundBlurUp,
     ProgressBorderBlurDown,
     ProgressBorderBlurUp,
+
     LyricForegroundBlurDown,
     LyricForegroundBlurUp,
     LyricBackgroundBlurDown,
     LyricBackgroundBlurUp,
     LyricBorderBlurDown,
     LyricBorderBlurUp,
+
     ThemeSelectBlurDown,
     ThemeSelectBlurUp,
     ThemeSelectLoad(usize),
-    KeyChange(IdKey, BindingForEvent),
+
+    KeyChange(IdKey, KeyBinding),
     SaveLastPositionBlurDown,
     SaveLastPosotionBlurUp,
     SeekStepBlurDown,
     SeekStepBlurUp,
     KillDaemonBlurDown,
     KillDaemonBlurUp,
+
     PlayerUseMprisBlurDown,
     PlayerUseMprisBlurUp,
     PlayerUseDiscordBlurDown,
     PlayerUseDiscordBlurUp,
     PlayerPortBlurDown,
     PlayerPortBlurUp,
+
     CurrentlyPlayingTrackSymbolBlurDown,
     CurrentlyPlayingTrackSymbolBlurUp,
+
+    ImportantPopupForegroundBlurDown,
+    ImportantPopupForegroundBlurUp,
+    ImportantPopupBackgroundBlurDown,
+    ImportantPopupBackgroundBlurUp,
+    ImportantPopupBorderBlurDown,
+    ImportantPopupBorderBlurUp,
+
+    FallbackForegroundBlurDown,
+    FallbackForegroundBlurUp,
+    FallbackBackgroundBlurDown,
+    FallbackBackgroundBlurUp,
+    FallbackBorderBlurDown,
+    FallbackBorderBlurUp,
+    FallbackHighlightBlurDown,
+    FallbackHighlightBlurUp,
 }
 
-#[derive(Clone, Debug, PartialEq, Eq)]
+#[derive(Clone, Copy, Debug, PartialEq, Eq)]
 pub enum KFMsg {
     DatabaseAddAllBlurDown,
     DatabaseAddAllBlurUp,
+    DatabaseAddSelectedBlurDown,
+    DatabaseAddSelectedBlurUp,
     GlobalConfigBlurDown,
     GlobalConfigBlurUp,
     GlobalDownBlurDown,
@@ -249,10 +277,10 @@ pub enum KFMsg {
     PlaylistSwapDownBlurUp,
     PlaylistSwapUpBlurDown,
     PlaylistSwapUpBlurUp,
-    PlaylistLqueueBlurDown,
-    PlaylistLqueueBlurUp,
-    PlaylistTqueueBlurDown,
-    PlaylistTqueueBlurUp,
+    PlaylistAddRandomAlbumBlurDown,
+    PlaylistAddRandomAlbumBlurUp,
+    PlaylistAddRandomTracksBlurDown,
+    PlaylistAddRandomTracksBlurUp,
     LibrarySwitchRootBlurDown,
     LibrarySwitchRootBlurUp,
     LibraryAddRootBlurDown,
@@ -297,8 +325,8 @@ pub enum KFMsg {
 
 #[derive(Clone, Debug, PartialEq, Eq)]
 pub enum LIMsg {
-    TreeExtendDir(String),
-    TreeGoToUpperDir,
+    TreeStepInto(String),
+    TreeStepOut,
     TreeBlur,
     Yank,
     Paste,
@@ -307,7 +335,7 @@ pub enum LIMsg {
     RemoveRoot,
 }
 
-#[derive(Clone, Debug, PartialEq, Eq)]
+#[derive(Clone, Copy, Debug, PartialEq, Eq)]
 pub enum DBMsg {
     AddAllToPlaylist,
     AddPlaylist(usize),
@@ -388,11 +416,9 @@ pub enum PLMsg {
     /// Swap a entry at INDEX with -1 (up)
     SwapUp(usize),
     /// Start choosing random albums to be added to the playlist
-    // TODO: the shortform "CmusLQueue" should also be explained
-    CmusLQueue,
+    AddRandomAlbum,
     /// Start choosing random tracks to be added to the playlist
-    // TODO: the shortform "CmusTQueue" should also be explained
-    CmusTQueue,
+    AddRandomTracks,
 }
 #[derive(Clone, Debug, PartialEq, Eq)]
 pub enum GSMsg {
@@ -442,7 +468,7 @@ pub enum TEMsg {
     TESelectLyricOk(usize),
 }
 
-#[derive(Clone, Debug, PartialEq, Eq)]
+#[derive(Clone, Copy, Debug, PartialEq, Eq)]
 pub enum TFMsg {
     CounterDeleteBlurDown,
     CounterDeleteBlurUp,
@@ -462,7 +488,7 @@ pub enum TFMsg {
     TextareaLyricBlurUp,
 }
 
-#[derive(Debug, Eq, PartialEq, Clone, Hash)]
+#[derive(Debug, Eq, PartialEq, Clone, Copy, Hash)]
 pub enum Id {
     ConfigEditor(IdConfigEditor),
     DBListCriteria,
@@ -497,7 +523,7 @@ pub enum Id {
     YoutubeSearchTablePopup,
 }
 
-#[derive(Debug, Eq, PartialEq, Clone, Hash)]
+#[derive(Debug, Eq, PartialEq, Clone, Copy, Hash)]
 pub enum IdTagEditor {
     CounterDelete,
     LabelHint,
@@ -510,7 +536,7 @@ pub enum IdTagEditor {
     TextareaLyric,
 }
 
-#[derive(Debug, Eq, PartialEq, Clone, Hash)]
+#[derive(Debug, Eq, PartialEq, Clone, Copy, Hash)]
 pub enum IdConfigEditor {
     AlbumPhotoAlign,
     CEThemeSelect,
@@ -520,20 +546,24 @@ pub enum IdConfigEditor {
     Header,
     Key(IdKey),
     KillDamon,
+
     LibraryBackground,
     LibraryBorder,
     LibraryForeground,
     LibraryHighlight,
     LibraryHighlightSymbol,
     LibraryLabel,
+
     LyricBackground,
     LyricBorder,
     LyricForeground,
     LyricLabel,
+
     MusicDir,
     PlayerPort,
     PlayerUseDiscord,
     PlayerUseMpris,
+
     PlaylistBackground,
     PlaylistBorder,
     PlaylistDisplaySymbol,
@@ -543,21 +573,37 @@ pub enum IdConfigEditor {
     PlaylistLabel,
     PlaylistRandomAlbum,
     PlaylistRandomTrack,
+
     CurrentlyPlayingTrackSymbol,
+
     PodcastDir,
     PodcastMaxRetries,
     PodcastSimulDownload,
+
     ProgressBackground,
     ProgressBorder,
     ProgressForeground,
     ProgressLabel,
+
     SaveLastPosition,
     SeekStep,
+
+    ImportantPopupLabel,
+    ImportantPopupBackground,
+    ImportantPopupBorder,
+    ImportantPopupForeground,
+
+    FallbackBackground,
+    FallbackBorder,
+    FallbackForeground,
+    FallbackHighlight,
+    FallbackLabel,
 }
 
-#[derive(Debug, Eq, PartialEq, Clone, Hash)]
+#[derive(Debug, Eq, PartialEq, Clone, Copy, Hash)]
 pub enum IdKey {
     DatabaseAddAll,
+    DatabaseAddSelected,
     GlobalConfig,
     GlobalDown,
     GlobalGotoBottom,
@@ -598,8 +644,8 @@ pub enum IdKey {
     PlaylistSearch,
     PlaylistSwapDown,
     PlaylistSwapUp,
-    PlaylistLqueue,
-    PlaylistTqueue,
+    PlaylistAddRandomAlbum,
+    PlaylistAddRandomTracks,
     LibrarySwitchRoot,
     LibraryAddRoot,
     LibraryRemoveRoot,
@@ -656,17 +702,17 @@ impl YoutubeOptions {
         Err(anyhow!("index not found"))
     }
 
-    pub fn prev_page(&mut self) -> Result<()> {
+    pub async fn prev_page(&mut self) -> Result<()> {
         if self.page > 1 {
             self.page -= 1;
-            self.items = self.invidious_instance.get_search_query(self.page)?;
+            self.items = self.invidious_instance.get_search_query(self.page).await?;
         }
         Ok(())
     }
 
-    pub fn next_page(&mut self) -> Result<()> {
+    pub async fn next_page(&mut self) -> Result<()> {
         self.page += 1;
-        self.items = self.invidious_instance.get_search_query(self.page)?;
+        self.items = self.invidious_instance.get_search_query(self.page).await?;
         Ok(())
     }
 
