@@ -61,14 +61,13 @@ pub mod playlist;
 
 use anyhow::{Context, Result};
 use async_trait::async_trait;
-use parking_lot::RwLock;
 pub use playlist::{Playlist, Status};
 use serde::{Deserialize, Serialize};
-use std::sync::Arc;
 use std::time::Duration;
-use termusiclib::config::{LastPosition, SeekStep, Settings};
+use termusiclib::config::v2::server::config_extra::ServerConfigVersionedDefaulted;
+use termusiclib::config::{new_shared_server_settings, ServerOverlay, SharedServerSettings};
+use termusiclib::library_db::DataBase;
 use termusiclib::podcast::db::Database as DBPod;
-use termusiclib::sqlite::DataBase;
 use termusiclib::track::{MediaType, Track};
 use termusiclib::utils::get_app_config_path;
 use tokio::runtime::Handle;
@@ -112,7 +111,7 @@ pub type PlayerCmdSender = UnboundedSender<PlayerCmd>;
 
 impl Backend {
     /// Create a new Backend based on `backend`([`BackendSelect`])
-    fn new_select(backend: BackendSelect, config: &Settings, cmd_tx: PlayerCmdSender) -> Self {
+    fn new_select(backend: BackendSelect, config: &ServerOverlay, cmd_tx: PlayerCmdSender) -> Self {
         match backend {
             #[cfg(feature = "mpv")]
             BackendSelect::Mpv => Self::new_mpv(config, cmd_tx),
@@ -128,7 +127,7 @@ impl Backend {
     ///
     /// For the order see [`BackendSelect::Default`]
     #[allow(unreachable_code)]
-    fn new_default(config: &Settings, cmd_tx: PlayerCmdSender) -> Self {
+    fn new_default(config: &ServerOverlay, cmd_tx: PlayerCmdSender) -> Self {
         #[cfg(feature = "rusty")]
         return Self::new_rusty(config, cmd_tx);
         #[cfg(feature = "gst")]
@@ -137,26 +136,26 @@ impl Backend {
         return Self::new_mpv(config, cmd_tx);
 
         #[cfg(not(any(feature = "rusty", feature = "gst", feature = "mpv")))]
-        compile_error!("No useable backend feature!");
+        unimplemented!("No compiled backend!");
     }
 
     /// Explicitly choose Backend [`RustyBackend`](rusty_backend::RustyBackend)
     #[cfg(feature = "rusty")]
-    fn new_rusty(config: &Settings, cmd_tx: PlayerCmdSender) -> Self {
+    fn new_rusty(config: &ServerOverlay, cmd_tx: PlayerCmdSender) -> Self {
         info!("Using Backend \"rusty\"");
         Self::Rusty(rusty_backend::RustyBackend::new(config, cmd_tx))
     }
 
     /// Explicitly choose Backend [`GstreamerBackend`](gstreamer_backend::GStreamerBackend)
     #[cfg(feature = "gst")]
-    fn new_gstreamer(config: &Settings, cmd_tx: PlayerCmdSender) -> Self {
+    fn new_gstreamer(config: &ServerOverlay, cmd_tx: PlayerCmdSender) -> Self {
         info!("Using Backend \"GStreamer\"");
         Self::GStreamer(gstreamer_backend::GStreamerBackend::new(config, cmd_tx))
     }
 
     /// Explicitly choose Backend [`MpvBackend`](mpv_backend::MpvBackend)
     #[cfg(feature = "mpv")]
-    fn new_mpv(config: &Settings, cmd_tx: PlayerCmdSender) -> Self {
+    fn new_mpv(config: &ServerOverlay, cmd_tx: PlayerCmdSender) -> Self {
         info!("Using Backend \"mpv\"");
         Self::Mpv(mpv_backend::MpvBackend::new(config, cmd_tx))
     }
@@ -170,6 +169,8 @@ impl Backend {
             Backend::Rusty(v) => v,
             #[cfg(feature = "gst")]
             Backend::GStreamer(v) => v,
+            #[cfg(not(any(feature = "rusty", feature = "gst", feature = "mpv")))]
+            _ => unimplemented!("No compiled backend!"),
         }
     }
 
@@ -182,6 +183,8 @@ impl Backend {
             Backend::Rusty(v) => v,
             #[cfg(feature = "gst")]
             Backend::GStreamer(v) => v,
+            #[cfg(not(any(feature = "rusty", feature = "gst", feature = "mpv")))]
+            _ => unimplemented!("No compiled backend!"),
         }
     }
 }
@@ -212,16 +215,14 @@ pub enum PlayerCmd {
     VolumeUp,
 }
 
-pub type SharedSettings = Arc<RwLock<Settings>>;
-
 #[allow(clippy::module_name_repetitions)]
 pub struct GeneralPlayer {
     pub backend: Backend,
     pub playlist: Playlist,
-    pub config: SharedSettings,
+    pub config: SharedServerSettings,
     pub current_track_updated: bool,
-    pub mpris: mpris::Mpris,
-    pub discord: discord::Rpc,
+    pub mpris: Option<mpris::Mpris>,
+    pub discord: Option<discord::Rpc>,
     pub db: DataBase,
     pub db_podcast: DBPod,
     pub cmd_tx: PlayerCmdSender,
@@ -236,26 +237,35 @@ impl GeneralPlayer {
     /// - if config path creation fails
     pub fn new_backend(
         backend: BackendSelect,
-        config: Settings,
+        config: ServerOverlay,
         cmd_tx: PlayerCmdSender,
     ) -> Result<Self> {
         let backend = Backend::new_select(backend, &config, cmd_tx.clone());
 
         let db_path = get_app_config_path().with_context(|| "failed to get podcast db path.")?;
 
-        let db_podcast =
-            DBPod::connect(&db_path).with_context(|| "error connecting to podcast db.")?;
-        let db = DataBase::new(&config);
+        let db_podcast = DBPod::new(&db_path).with_context(|| "error connecting to podcast db.")?;
+        let db = DataBase::new(&config)?;
 
-        let config = Arc::new(RwLock::new(config));
+        let config = new_shared_server_settings(config);
         let playlist = Playlist::new(config.clone()).unwrap_or_default();
+        let mpris = if config.read().settings.player.use_mediacontrols {
+            Some(mpris::Mpris::new(cmd_tx.clone()))
+        } else {
+            None
+        };
+        let discord = if config.read().get_discord_status_enable() {
+            Some(discord::Rpc::default())
+        } else {
+            None
+        };
 
         Ok(Self {
             backend,
             playlist,
             config,
-            mpris: mpris::Mpris::new(cmd_tx.clone()),
-            discord: discord::Rpc::default(),
+            mpris,
+            discord,
             db,
             db_podcast,
             cmd_tx,
@@ -269,10 +279,56 @@ impl GeneralPlayer {
     ///
     /// - if connecting to the database fails
     /// - if config path creation fails
-    pub fn new(config: Settings, cmd_tx: PlayerCmdSender) -> Result<Self> {
+    pub fn new(config: ServerOverlay, cmd_tx: PlayerCmdSender) -> Result<Self> {
         Self::new_backend(BackendSelect::Default, config, cmd_tx)
     }
 
+    /// Reload the config from file, on fail continue to use the old
+    ///
+    /// # Errors
+    ///
+    /// - if Config could not be parsed
+    pub fn reload_config(&mut self) -> Result<()> {
+        info!("Reloading config");
+        let mut config = self.config.write();
+        let parsed = ServerConfigVersionedDefaulted::from_config_path()?.into_settings();
+        config.settings = parsed;
+
+        if config.settings.player.use_mediacontrols && self.mpris.is_none() {
+            // start mpris if new config has it enabled, but is not active yet
+            let mut mpris = mpris::Mpris::new(self.cmd_tx.clone());
+            // actually set the metadata of the currently playing track, otherwise the controls will work but no title or coverart will be set until next track
+            if let Some(track) = self.playlist.current_track() {
+                mpris.add_and_play(track);
+            }
+            // the same for volume
+            mpris.update_volume(self.volume());
+            self.mpris.replace(mpris);
+        } else if !config.settings.player.use_mediacontrols && self.mpris.is_some() {
+            // stop mpris if new config does not have it enabled, but is currently active
+            self.mpris.take();
+        }
+
+        if config.get_discord_status_enable() && self.discord.is_none() {
+            // start discord ipc if new config has it enabled, but is not active yet
+            let mut discord = discord::Rpc::default();
+
+            // actually set the metadata of the currently playing track, otherwise the controls will work but no title or coverart will be set until next track
+            if let Some(track) = self.playlist.current_track() {
+                discord.update(track);
+            }
+
+            self.discord.replace(discord);
+        } else if !config.get_discord_status_enable() && self.discord.is_some() {
+            // stop discord ipc if new config does not have it enabled, but is currently active
+            self.discord.take();
+        }
+
+        info!("Config Reloaded");
+
+        Ok(())
+    }
+
     fn get_player(&self) -> &dyn PlayerTrait {
         self.backend.as_player()
     }
@@ -284,7 +340,7 @@ impl GeneralPlayer {
     pub fn toggle_gapless(&mut self) -> bool {
         let new_gapless = !self.backend.as_player().gapless();
         self.backend.as_player_mut().set_gapless(new_gapless);
-        self.config.write().player_gapless = new_gapless;
+        self.config.write().settings.player.gapless = new_gapless;
         new_gapless
     }
 
@@ -332,13 +388,12 @@ impl GeneralPlayer {
 
     fn add_and_play_mpris_discord(&mut self) {
         if let Some(track) = self.playlist.current_track() {
-            let config = self.config.read();
-            if config.player_use_mpris {
-                self.mpris.add_and_play(track);
+            if let Some(ref mut mpris) = self.mpris {
+                mpris.add_and_play(track);
             }
 
-            if config.player_use_discord {
-                self.discord.update(track);
+            if let Some(ref mut discord) = self.discord {
+                discord.update(track);
             }
         }
     }
@@ -380,25 +435,23 @@ impl GeneralPlayer {
         match self.playlist.status() {
             Status::Running => {
                 self.get_player_mut().pause();
-                let config = self.config.read();
-                if config.player_use_mpris {
-                    self.mpris.pause();
+                if let Some(ref mut mpris) = self.mpris {
+                    mpris.pause();
                 }
-                if config.player_use_discord {
-                    self.discord.pause();
+                if let Some(ref mut discord) = self.discord {
+                    discord.pause();
                 }
                 self.playlist.set_status(Status::Paused);
             }
             Status::Stopped => {}
             Status::Paused => {
                 self.get_player_mut().resume();
-                let config = self.config.read();
-                if config.player_use_mpris {
-                    self.mpris.resume();
+                if let Some(ref mut mpris) = self.mpris {
+                    mpris.resume();
                 }
-                if config.player_use_discord {
-                    let time_pos = self.get_player().position();
-                    self.discord.resume(time_pos);
+                let time_pos = self.get_player().position();
+                if let Some(ref mut discord) = self.discord {
+                    discord.resume(time_pos);
                 }
                 self.playlist.set_status(Status::Running);
             }
@@ -409,12 +462,11 @@ impl GeneralPlayer {
         match self.playlist.status() {
             Status::Running => {
                 self.get_player_mut().pause();
-                let config = self.config.read();
-                if config.player_use_mpris {
-                    self.mpris.pause();
+                if let Some(ref mut mpris) = self.mpris {
+                    mpris.pause();
                 }
-                if config.player_use_discord {
-                    self.discord.pause();
+                if let Some(ref mut discord) = self.discord {
+                    discord.pause();
                 }
                 self.playlist.set_status(Status::Paused);
             }
@@ -427,13 +479,12 @@ impl GeneralPlayer {
             Status::Running | Status::Stopped => {}
             Status::Paused => {
                 self.get_player_mut().resume();
-                let config = self.config.read();
-                if config.player_use_mpris {
-                    self.mpris.resume();
+                if let Some(ref mut mpris) = self.mpris {
+                    mpris.resume();
                 }
-                if config.player_use_discord {
-                    let time_pos = self.get_player().position();
-                    self.discord.resume(time_pos);
+                let time_pos = self.get_player().position();
+                if let Some(ref mut discord) = self.discord {
+                    discord.resume(time_pos);
                 }
                 self.playlist.set_status(Status::Running);
             }
@@ -443,22 +494,22 @@ impl GeneralPlayer {
     ///
     /// if the underlying "seek" returns a error (which current never happens)
     pub fn seek_relative(&mut self, forward: bool) {
-        let mut offset = match self.config.read().player_seek_step {
-            SeekStep::Short => -5_i64,
-            SeekStep::Long => -30,
-            SeekStep::Auto => {
-                if let Some(track) = self.playlist.current_track() {
-                    if track.duration().as_secs() >= 600 {
-                        -30
-                    } else {
-                        -5
-                    }
-                } else {
-                    -5
-                }
-            }
+        let track_len = if let Some(track) = self.playlist.current_track() {
+            track.duration().as_secs()
+        } else {
+            // fallback to 5 instead of not seeking at all
+            5
         };
-        if forward {
+
+        let mut offset = self
+            .config
+            .read()
+            .settings
+            .player
+            .seek_step
+            .get_step(track_len);
+
+        if !forward {
             offset = -offset;
         }
         self.get_player_mut()
@@ -468,99 +519,92 @@ impl GeneralPlayer {
 
     #[allow(clippy::cast_sign_loss)]
     pub fn player_save_last_position(&mut self) {
-        match self.config.read().player_remember_last_played_position {
-            LastPosition::Yes => {
-                if let Some(track) = self.playlist.current_track() {
-                    // let time_pos = self.player.position.lock().unwrap();
-                    let time_pos = self.get_player().position();
-                    match track.media_type {
-                        MediaType::Music => self
-                            .db
-                            .set_last_position(track, time_pos.unwrap_or_default()),
-                        MediaType::Podcast => {
-                            self.db_podcast
-                                .set_last_position(track, time_pos.unwrap_or_default());
-                        }
-                        MediaType::LiveRadio => {}
+        let Some(track) = self.playlist.current_track() else {
+            info!("Not saving Last position as there is no current track");
+            return;
+        };
+        let Some(position) = self.get_player().position() else {
+            info!("Not saving Last position as there is no position");
+            return;
+        };
+
+        let Some(time_before_save) = self
+            .config
+            .read()
+            .settings
+            .player
+            .remember_position
+            .get_time(track.media_type)
+        else {
+            info!(
+                "Not saving Last position as \"Remember last position\" is not enabled for {:#?}",
+                track.media_type
+            );
+            return;
+        };
+
+        if time_before_save < position.as_secs() {
+            match track.media_type {
+                MediaType::Music => {
+                    if let Err(err) = self.db.set_last_position(track, position) {
+                        error!("Saving last_position for music failed, Error: {:#?}", err);
                     }
                 }
-            }
-            LastPosition::No => {}
-            LastPosition::Auto => {
-                if let Some(track) = self.playlist.current_track() {
-                    // 10 minutes
-                    if track.duration().as_secs() >= 600 {
-                        // let time_pos = self.player.position.lock().unwrap();
-                        let time_pos = self.get_player().position();
-                        match track.media_type {
-                            MediaType::Music => self
-                                .db
-                                .set_last_position(track, time_pos.unwrap_or_default()),
-                            MediaType::Podcast => {
-                                self.db_podcast
-                                    .set_last_position(track, time_pos.unwrap_or_default());
-                            }
-                            MediaType::LiveRadio => {}
-                        }
+                MediaType::Podcast => {
+                    if let Err(err) = self.db_podcast.set_last_position(track, position) {
+                        error!("Saving last_position for podcast failed, Error: {:#?}", err);
                     }
                 }
+                MediaType::LiveRadio => (),
             }
+        } else {
+            info!("Not saving Last position as the position is lower than time_before_save");
         }
     }
 
     pub fn player_restore_last_position(&mut self) {
+        let Some(track) = self.playlist.current_track() else {
+            info!("Not restoring Last position as there is no current track");
+            return;
+        };
+
         let mut restored = false;
-        let last_pos = self.config.read().player_remember_last_played_position;
-        match last_pos {
-            LastPosition::Yes => {
-                if let Some(track) = self.playlist.current_track() {
-                    match track.media_type {
-                        MediaType::Music => {
-                            if let Ok(last_pos) = self.db.get_last_position(track) {
-                                self.get_player_mut().seek_to(last_pos);
-                                restored = true;
-                            }
-                        }
 
-                        MediaType::Podcast => {
-                            if let Ok(last_pos) = self.db_podcast.get_last_position(track) {
-                                self.get_player_mut().seek_to(last_pos);
-                                restored = true;
-                            }
-                        }
-                        MediaType::LiveRadio => {}
+        if self
+            .config
+            .read()
+            .settings
+            .player
+            .remember_position
+            .is_enabled_for(track.media_type)
+        {
+            match track.media_type {
+                MediaType::Music => {
+                    if let Ok(last_pos) = self.db.get_last_position(track) {
+                        self.get_player_mut().seek_to(last_pos);
+                        restored = true;
                     }
                 }
-            }
-            LastPosition::No => {}
-            LastPosition::Auto => {
-                if let Some(track) = self.playlist.current_track() {
-                    // 10 minutes
-                    if track.duration().as_secs() >= 600 {
-                        match track.media_type {
-                            MediaType::Music => {
-                                if let Ok(last_pos) = self.db.get_last_position(track) {
-                                    self.get_player_mut().seek_to(last_pos);
-                                    restored = true;
-                                }
-                            }
-
-                            MediaType::Podcast => {
-                                if let Ok(last_pos) = self.db_podcast.get_last_position(track) {
-                                    self.get_player_mut().seek_to(last_pos);
-                                    restored = true;
-                                }
-                            }
-                            MediaType::LiveRadio => {}
-                        }
+                MediaType::Podcast => {
+                    if let Ok(last_pos) = self.db_podcast.get_last_position(track) {
+                        self.get_player_mut().seek_to(last_pos);
+                        restored = true;
                     }
                 }
+                MediaType::LiveRadio => (),
             }
+        } else {
+            info!(
+                "Not restoring Last position as it is not enabled for {:#?}",
+                track.media_type
+            );
         }
 
         if restored {
             if let Some(track) = self.playlist.current_track() {
-                self.db.set_last_position(track, Duration::from_secs(0));
+                if let Err(err) = self.db.set_last_position(track, Duration::from_secs(0)) {
+                    error!("Resetting last_position failed, Error: {:#?}", err);
+                }
             }
         }
     }
@@ -574,11 +618,8 @@ impl PlayerTrait for GeneralPlayer {
     fn volume(&self) -> Volume {
         self.get_player().volume()
     }
-    fn volume_up(&mut self) -> Volume {
-        self.get_player_mut().volume_up()
-    }
-    fn volume_down(&mut self) -> Volume {
-        self.get_player_mut().volume_down()
+    fn add_volume(&mut self, volume: VolumeSigned) -> Volume {
+        self.get_player_mut().add_volume(volume)
     }
     fn set_volume(&mut self, volume: Volume) -> Volume {
         self.get_player_mut().set_volume(volume)
@@ -605,12 +646,8 @@ impl PlayerTrait for GeneralPlayer {
         self.get_player_mut().set_speed(speed)
     }
 
-    fn speed_up(&mut self) -> Speed {
-        self.get_player_mut().speed_up()
-    }
-
-    fn speed_down(&mut self) -> Speed {
-        self.get_player_mut().speed_down()
+    fn add_speed(&mut self, speed: SpeedSigned) -> Speed {
+        self.get_player_mut().add_speed(speed)
     }
 
     fn speed(&self) -> Speed {
@@ -692,7 +729,14 @@ pub struct MediaInfo {
 }
 
 pub type Volume = u16;
+/// The type of [`Volume::saturating_add_signed`]
+pub type VolumeSigned = i16;
 pub type Speed = i32;
+// yes this is currently the same as speed, but for consistentcy with VolumeSigned (and maybe other types)
+pub type SpeedSigned = Speed;
+
+pub const MIN_SPEED: Speed = 1;
+pub const MAX_SPEED: Speed = 30;
 
 #[allow(clippy::module_name_repetitions)]
 #[async_trait]
@@ -701,14 +745,13 @@ pub trait PlayerTrait {
     async fn add_and_play(&mut self, track: &Track);
     /// Get the currently set volume
     fn volume(&self) -> Volume;
-    /// Step the volume up by a backend-set step amount
+    /// Add a relative amount to the current volume
     ///
     /// Returns the new volume
-    fn volume_up(&mut self) -> Volume;
-    /// Step the volume down by a backend-set step amount
-    ///
-    /// Returns the new volume
-    fn volume_down(&mut self) -> Volume;
+    fn add_volume(&mut self, volume: VolumeSigned) -> Volume {
+        let volume = self.volume().saturating_add_signed(volume);
+        self.set_volume(volume)
+    }
     /// Set the volume to a specific amount.
     ///
     /// Returns the new volume
@@ -731,14 +774,14 @@ pub trait PlayerTrait {
     ///
     /// Returns the new speed
     fn set_speed(&mut self, speed: Speed) -> Speed;
-    /// Step the speed up by a backend-set step amount
+    /// Add a relative amount to the current speed
     ///
     /// Returns the new speed
-    fn speed_up(&mut self) -> Speed;
-    /// Step the speed down by a backend-set step amount
-    ///
-    /// Returns the new speed
-    fn speed_down(&mut self) -> Speed;
+    fn add_speed(&mut self, speed: SpeedSigned) -> Speed {
+        // NOTE: the clamping should likely be done in `set_speed` instead of here
+        let speed = (self.speed() + speed).clamp(MIN_SPEED, MAX_SPEED);
+        self.set_speed(speed)
+    }
     /// Get the currently set speed
     fn speed(&self) -> Speed;
     fn stop(&mut self);
