@@ -3,7 +3,9 @@ use souvlaki::{MediaControlEvent, MediaControls, MediaMetadata, MediaPlayback, P
 use std::sync::mpsc::{self, Receiver};
 use termusiclib::track::Track;
 
-use crate::{GeneralPlayer, PlayerCmd, PlayerTimeUnit, PlayerTrait, Status};
+use crate::{
+    GeneralPlayer, PlayerCmd, PlayerProgress, PlayerTimeUnit, PlayerTrait, Status, Volume,
+};
 
 pub struct Mpris {
     controls: MediaControls,
@@ -29,7 +31,7 @@ impl Mpris {
         #[cfg(target_os = "windows")]
         let (hwnd, _dummy_window) = {
             let dummy_window = windows::DummyWindow::new().unwrap();
-            let handle = Some(dummy_window.handle.0 as _);
+            let handle = Some(dummy_window.handle.0);
             (handle, dummy_window)
         };
 
@@ -122,6 +124,20 @@ impl Mpris {
             };
         }
     }
+
+    /// Update the Volume reported by Media-Controls
+    ///
+    /// currently only does something on linux (mpris)
+    #[allow(unused_variables, clippy::unused_self)] // non-linux targets will complain about unused parameters
+    pub fn update_volume(&mut self, volume: Volume) {
+        // currently "set_volume" only exists for "linux"(mpris)
+        #[cfg(target_os = "linux")]
+        {
+            // update the reported volume in mpris
+            let vol = f64::from(volume) / 100.0;
+            let _ = self.controls.set_volume(vol);
+        }
+    }
 }
 
 impl GeneralPlayer {
@@ -209,22 +225,29 @@ impl GeneralPlayer {
         }
     }
 
-    pub fn update_mpris(&mut self) {
-        if let Ok(m) = self.mpris.rx.try_recv() {
-            self.mpris_handler(m);
+    /// Handle Media-Controls events, if enabled to be used
+    pub fn mpris_handle_events(&mut self) {
+        if let Some(ref mut mpris) = self.mpris {
+            if let Ok(m) = mpris.rx.try_recv() {
+                self.mpris_handler(m);
+            }
         }
+    }
 
-        // currently "set_volume" only exists for "linux"(mpris)
-        #[cfg(target_os = "linux")]
-        {
-            // update the reported volume in mpris
-            let vol = f64::from(self.volume()) / 100.0;
-            self.mpris.controls.set_volume(vol).ok();
+    /// Update Media-Controls reported Position & Status, if enabled to be reporting
+    #[inline]
+    pub fn mpris_update_progress(&mut self, progress: &PlayerProgress) {
+        if let Some(ref mut mpris) = self.mpris {
+            mpris.update_progress(progress.position, self.playlist.status());
         }
+    }
 
-        if let Some(progress) = self.get_progress() {
-            self.mpris
-                .update_progress(progress.position, self.playlist.status());
+    /// Update Media-Controls reported volume, if enabled to be reporting
+    #[inline]
+    pub fn mpris_volume_update(&mut self) {
+        let volume = self.volume();
+        if let Some(ref mut mpris) = self.mpris {
+            mpris.update_volume(volume);
         }
     }
 }
@@ -242,9 +265,8 @@ mod windows {
     use windows::Win32::Foundation::{HWND, LPARAM, LRESULT, WPARAM};
     use windows::Win32::System::LibraryLoader::GetModuleHandleW;
     use windows::Win32::UI::WindowsAndMessaging::{
-        CreateWindowExW, DefWindowProcW, DestroyWindow, DispatchMessageW, GetAncestor,
-        IsDialogMessageW, PeekMessageW, RegisterClassExW, TranslateMessage, GA_ROOT, MSG,
-        PM_REMOVE, WINDOW_EX_STYLE, WINDOW_STYLE, WM_QUIT, WNDCLASSEXW,
+        CreateWindowExW, DefWindowProcW, DestroyWindow, RegisterClassExW, WINDOW_EX_STYLE,
+        WINDOW_STYLE, WNDCLASSEXW,
     };
 
     pub struct DummyWindow {
@@ -274,7 +296,7 @@ mod windows {
                     ));
                 }
 
-                let handle = CreateWindowExW(
+                let handle = match CreateWindowExW(
                     WINDOW_EX_STYLE::default(),
                     class_name,
                     w!(""),
@@ -287,9 +309,14 @@ mod windows {
                     None,
                     instance,
                     None,
-                );
+                ) {
+                    Ok(v) => v,
+                    Err(err) => {
+                        return Err(format!("{err}"));
+                    }
+                };
 
-                if handle.0 == 0 {
+                if handle.is_invalid() {
                     Err(format!(
                         "Message only window creation failed: {}",
                         Error::last_os_error()
@@ -319,21 +346,21 @@ mod windows {
         }
     }
 
-    #[allow(dead_code)]
-    pub fn pump_event_queue() -> bool {
-        unsafe {
-            let mut msg: MSG = std::mem::zeroed();
-            let mut has_message = PeekMessageW(&mut msg, None, 0, 0, PM_REMOVE).as_bool();
-            while msg.message != WM_QUIT && has_message {
-                if !IsDialogMessageW(GetAncestor(msg.hwnd, GA_ROOT), &msg).as_bool() {
-                    TranslateMessage(&msg);
-                    DispatchMessageW(&msg);
-                }
+    // #[allow(dead_code)]
+    // pub fn pump_event_queue() -> bool {
+    //     unsafe {
+    //         let mut msg: MSG = std::mem::zeroed();
+    //         let mut has_message = PeekMessageW(&mut msg, None, 0, 0, PM_REMOVE).as_bool();
+    //         while msg.message != WM_QUIT && has_message {
+    //             if !IsDialogMessageW(GetAncestor(msg.hwnd, GA_ROOT), &msg).as_bool() {
+    //                 TranslateMessage(&msg);
+    //                 DispatchMessageW(&msg);
+    //             }
 
-                has_message = PeekMessageW(&mut msg, None, 0, 0, PM_REMOVE).as_bool();
-            }
+    //             has_message = PeekMessageW(&mut msg, None, 0, 0, PM_REMOVE).as_bool();
+    //         }
 
-            msg.message == WM_QUIT
-        }
-    }
+    //         msg.message == WM_QUIT
+    //     }
+    // }
 }
