@@ -16,17 +16,18 @@ use async_trait::async_trait;
 pub use conversions::Sample;
 pub use cpal::{traits::StreamTrait, ChannelCount, SampleRate};
 pub use decoder::Symphonia;
-use reqwest::header::{HeaderMap, HeaderValue};
 pub use sink::Sink;
 pub use source::Source;
 use std::num::{NonZeroU16, NonZeroUsize};
 pub use stream::OutputStream;
+use termusiclib::config::ServerOverlay;
 use tokio::runtime::Handle;
 
 use crate::{MediaInfo, Speed, Volume};
 
 use self::decoder::buffered_source::BufferedSource;
 use self::decoder::read_seek_source::ReadSeekSource;
+use self::decoder::{MediaTitleRx, MediaTitleType};
 
 use super::{PlayerCmd, PlayerProgress, PlayerTrait};
 use anyhow::{anyhow, Context, Result};
@@ -34,13 +35,17 @@ use parking_lot::Mutex;
 use std::fs::File;
 use std::path::Path;
 use std::sync::atomic::{AtomicU16, Ordering};
-use std::sync::mpsc::RecvTimeoutError;
 use std::sync::mpsc::{self, Receiver, Sender};
 use std::sync::Arc;
 use std::time::Duration;
-use stream_download::http::{reqwest::Client, HttpStream};
+use stream_download::http::{
+    reqwest::{
+        header::{HeaderMap, HeaderValue},
+        Client,
+    },
+    HttpStream,
+};
 use stream_download::source::SourceStream;
-use stream_download::storage::adaptive::AdaptiveStorageProvider;
 use stream_download::storage::bounded::BoundedStorageProvider;
 use stream_download::storage::memory::MemoryStorageProvider;
 use stream_download::storage::temp::TempStorageProvider;
@@ -48,11 +53,8 @@ use stream_download::{Settings as StreamSettings, StreamDownload};
 use symphonia::core::io::{
     MediaSource, MediaSourceStream, MediaSourceStreamOptions, ReadOnlySource,
 };
-use termusiclib::config::Settings;
 use termusiclib::track::{MediaType, Track};
 
-static VOLUME_STEP: u16 = 5;
-
 pub type TotalDuration = Option<Duration>;
 pub type ArcTotalDuration = Arc<Mutex<TotalDuration>>;
 
@@ -84,7 +86,7 @@ pub struct RustyBackend {
     command_tx: Sender<PlayerInternalCmd>,
     position: Arc<Mutex<Duration>>,
     total_duration: ArcTotalDuration,
-    radio_title: Arc<Mutex<String>>,
+    media_title: Arc<Mutex<String>>,
     pub radio_downloaded: Arc<Mutex<u64>>,
     // cmd_tx_outside: crate::PlayerCmdSender,
 }
@@ -97,21 +99,21 @@ pub struct RustyBackend {
 impl RustyBackend {
     #[allow(clippy::similar_names)]
     #[allow(clippy::too_many_lines)]
-    pub fn new(config: &Settings, cmd_tx: crate::PlayerCmdSender) -> Self {
+    pub fn new(config: &ServerOverlay, cmd_tx: crate::PlayerCmdSender) -> Self {
         let (picmd_tx, picmd_rx): (Sender<PlayerInternalCmd>, Receiver<PlayerInternalCmd>) =
             mpsc::channel();
         let picmd_tx_local = picmd_tx.clone();
-        let volume = Arc::new(AtomicU16::from(config.player_volume));
+        let volume = Arc::new(AtomicU16::from(config.settings.player.volume));
         let volume_local = volume.clone();
-        let speed = config.player_speed;
-        let gapless = config.player_gapless;
+        let speed = config.settings.player.speed;
+        let gapless = config.settings.player.gapless;
         let position = Arc::new(Mutex::new(Duration::default()));
         let total_duration = Arc::new(Mutex::new(None));
         let total_duration_local = total_duration.clone();
         let position_local = position.clone();
         let pcmd_tx_local = cmd_tx;
-        let radio_title = Arc::new(Mutex::new(String::new()));
-        let radio_title_local = radio_title.clone();
+        let media_title = Arc::new(Mutex::new(String::new()));
+        let media_title_local = media_title.clone();
         let radio_downloaded = Arc::new(Mutex::new(100_u64));
         // let radio_downloaded_local = radio_downloaded.clone();
         // this should likely be a parameter, but works for now
@@ -125,7 +127,7 @@ impl RustyBackend {
                     pcmd_tx_local,
                     picmd_tx_local,
                     picmd_rx,
-                    radio_title_local,
+                    media_title_local,
                     // radio_downloaded_local,
                     position_local,
                     volume_local,
@@ -141,7 +143,7 @@ impl RustyBackend {
             gapless,
             command_tx: picmd_tx,
             position,
-            radio_title,
+            media_title,
             radio_downloaded,
             // cmd_tx_outside: cmd_tx,
         }
@@ -173,22 +175,6 @@ impl PlayerTrait for RustyBackend {
         self.volume.load(Ordering::SeqCst)
     }
 
-    fn volume_up(&mut self) -> Volume {
-        let volume = self
-            .volume
-            .load(Ordering::SeqCst)
-            .saturating_add(VOLUME_STEP);
-        self.set_volume(volume)
-    }
-
-    fn volume_down(&mut self) -> Volume {
-        let volume = self
-            .volume
-            .load(Ordering::SeqCst)
-            .saturating_sub(VOLUME_STEP);
-        self.set_volume(volume)
-    }
-
     fn set_volume(&mut self, volume: Volume) -> Volume {
         let volume = volume.min(100);
         self.volume.store(volume, Ordering::SeqCst);
@@ -220,22 +206,6 @@ impl PlayerTrait for RustyBackend {
         self.command(PlayerInternalCmd::SeekAbsolute(position));
     }
 
-    fn speed_up(&mut self) -> Speed {
-        let mut speed = self.speed + 1;
-        if speed > 30 {
-            speed = 30;
-        }
-        self.set_speed(speed)
-    }
-
-    fn speed_down(&mut self) -> Speed {
-        let mut speed = self.speed - 1;
-        if speed < 1 {
-            speed = 1;
-        }
-        self.set_speed(speed)
-    }
-
     fn set_speed(&mut self, speed: Speed) -> Speed {
         self.speed = speed;
         self.command(PlayerInternalCmd::Speed(speed));
@@ -280,19 +250,37 @@ impl PlayerTrait for RustyBackend {
     }
 
     fn media_info(&self) -> MediaInfo {
-        let radio_title_r = self.radio_title.lock();
-        if radio_title_r.is_empty() {
+        let media_title_r = self.media_title.lock();
+        if media_title_r.is_empty() {
             MediaInfo::default()
         } else {
             MediaInfo {
-                media_title: Some(radio_title_r.clone()),
+                media_title: Some(media_title_r.clone()),
             }
         }
     }
 }
 
+/// Append the `media_source` to the `sink`, while allowing different functions to run with `func` with a [`MediaTitleRx`]
+fn append_to_sink_inner_media_title<F: FnOnce(&mut Symphonia, MediaTitleRx)>(
+    media_source: Box<dyn MediaSource>,
+    trace: &str,
+    sink: &Sink,
+    gapless: bool,
+    func: F,
+) {
+    let mss = MediaSourceStream::new(media_source, MediaSourceStreamOptions::default());
+    match Symphonia::new_with_media_title(mss, gapless) {
+        Ok((mut decoder, rx)) => {
+            func(&mut decoder, rx);
+            sink.append(decoder);
+        }
+        Err(e) => error!("error decoding '{trace}' is: {e:?}"),
+    }
+}
+
 /// Append the `media_source` to the `sink`, while allowing different functions to run with `func`
-fn append_to_sink_inner<F: FnOnce(&Symphonia)>(
+fn append_to_sink_inner<F: FnOnce(&mut Symphonia)>(
     media_source: Box<dyn MediaSource>,
     trace: &str,
     sink: &Sink,
@@ -301,8 +289,8 @@ fn append_to_sink_inner<F: FnOnce(&Symphonia)>(
 ) {
     let mss = MediaSourceStream::new(media_source, MediaSourceStreamOptions::default());
     match Symphonia::new(mss, gapless) {
-        Ok(decoder) => {
-            func(&decoder);
+        Ok(mut decoder) => {
+            func(&mut decoder);
             sink.append(decoder);
         }
         Err(e) => error!("error decoding '{trace}' is: {e:?}"),
@@ -310,19 +298,36 @@ fn append_to_sink_inner<F: FnOnce(&Symphonia)>(
 }
 
 /// Append the `media_source` to the `sink`, while also setting `total_duration*`
-fn append_to_sink(
+///
+/// Expects current thread to have a tokio handle
+fn append_to_sink<MT: Fn(MediaTitleType) + Send + 'static>(
     media_source: Box<dyn MediaSource>,
     trace: &str,
     sink: &Sink,
     gapless: bool,
     total_duration_local: &ArcTotalDuration,
+    media_title_fn: MT,
 ) {
-    append_to_sink_inner(media_source, trace, sink, gapless, |decoder| {
-        std::mem::swap(
-            &mut *total_duration_local.lock(),
-            &mut decoder.total_duration(),
-        );
-    });
+    append_to_sink_inner_media_title(
+        media_source,
+        trace,
+        sink,
+        gapless,
+        |decoder, mut media_title_rx| {
+            std::mem::swap(
+                &mut *total_duration_local.lock(),
+                &mut decoder.total_duration(),
+            );
+
+            let handle = Handle::current();
+
+            handle.spawn(async move {
+                while let Some(cmd) = media_title_rx.recv().await {
+                    media_title_fn(cmd);
+                }
+            });
+        },
+    );
 }
 
 /// Append the `media_source` to the `sink`, while setting duration to be unknown (to [`None`])
@@ -342,19 +347,36 @@ fn append_to_sink_no_duration(
 /// Append the `media_source` to the `sink`, while also setting `next_duration_opt`
 ///
 /// This is used for enqueued entries which do not start immediately
-fn append_to_sink_queue(
+///
+/// Expects current thread to have a tokio handle
+fn append_to_sink_queue<MT: Fn(MediaTitleType) + Send + 'static>(
     media_source: Box<dyn MediaSource>,
     trace: &str,
     sink: &Sink,
     gapless: bool,
     // total_duration_local: &ArcTotalDuration,
     next_duration_opt: &mut Option<Duration>,
+    media_title_fn: MT,
 ) {
-    append_to_sink_inner(media_source, trace, sink, gapless, |decoder| {
-        std::mem::swap(next_duration_opt, &mut decoder.total_duration());
-        // rely on EOS message to set next duration
-        sink.message_on_end();
-    });
+    append_to_sink_inner_media_title(
+        media_source,
+        trace,
+        sink,
+        gapless,
+        |decoder, mut media_title_rx| {
+            std::mem::swap(next_duration_opt, &mut decoder.total_duration());
+            // rely on EOS message to set next duration
+            sink.message_on_end();
+
+            let handle = Handle::current();
+
+            handle.spawn(async move {
+                while let Some(cmd) = media_title_rx.recv().await {
+                    media_title_fn(cmd);
+                }
+            });
+        },
+    );
 }
 
 /// Append the `media_source` to the `sink`, while also setting `next_duration_opt` to be unknown (to [`None`])
@@ -376,6 +398,14 @@ fn append_to_sink_queue_no_duration(
     });
 }
 
+/// Common handling of a `media_title` for a [`append_to_sink`] function
+fn common_media_title_cb(media_title: Arc<Mutex<String>>) -> impl Fn(MediaTitleType) {
+    move |cmd| match cmd {
+        MediaTitleType::Reset => media_title.lock().clear(),
+        MediaTitleType::Value(v) => *media_title.lock() = v,
+    }
+}
+
 /// Player thread loop
 #[allow(
     clippy::cast_precision_loss,
@@ -390,7 +420,7 @@ async fn player_thread(
     pcmd_tx: crate::PlayerCmdSender,
     picmd_tx: Sender<PlayerInternalCmd>,
     picmd_rx: Receiver<PlayerInternalCmd>,
-    radio_title: Arc<Mutex<String>>,
+    media_title: Arc<Mutex<String>>,
     // radio_downloaded: Arc<Mutex<u64>>,
     position: Arc<Mutex<Duration>>,
     volume_inside: Arc<AtomicU16>,
@@ -406,10 +436,9 @@ async fn player_thread(
     sink.set_speed(speed_inside as f32 / 10.0);
     sink.set_volume(f32::from(volume_inside.load(Ordering::SeqCst)) / 100.0);
     loop {
-        let cmd = match picmd_rx.recv_timeout(Duration::from_micros(100)) {
-            Ok(v) => v,
-            Err(RecvTimeoutError::Disconnected) => break,
-            Err(_) => continue,
+        let Ok(cmd) = picmd_rx.recv() else {
+            // only error can be a disconnect (no more senders)
+            break;
         };
 
         match cmd {
@@ -421,7 +450,7 @@ async fn player_thread(
                     &mut is_radio,
                     &total_duration,
                     &mut next_duration_opt,
-                    &radio_title,
+                    &media_title,
                     // &radio_downloaded,
                     false,
                 )
@@ -441,7 +470,7 @@ async fn player_thread(
                     &mut is_radio,
                     &total_duration,
                     &mut next_duration_opt,
-                    &radio_title,
+                    &media_title,
                     // &radio_downloaded,
                     true,
                 )
@@ -467,7 +496,13 @@ async fn player_thread(
                 volume_inside.store(volume, Ordering::SeqCst);
             }
             PlayerInternalCmd::Skip => {
-                sink.skip_one();
+                // the sink can be empty, if for example nothing could be enqueued, so a "skip_one" would be a no-op and never send EOS, which is required to go to the next track
+                if sink.empty() {
+                    let _ = picmd_tx.send(PlayerInternalCmd::Eos);
+                    let _ = pcmd_tx.send(PlayerCmd::Eos);
+                } else {
+                    sink.skip_one();
+                }
                 if sink.is_paused() {
                     sink.play();
                 }
@@ -545,8 +580,7 @@ async fn queue_next(
     is_radio: &mut bool,
     total_duration: &ArcTotalDuration,
     next_duration_opt: &mut Option<Duration>,
-    radio_title: &Arc<Mutex<String>>,
-    // _radio_downloaded: &Arc<Mutex<u64>>,
+    media_title: &Arc<Mutex<String>>,
     enqueue: bool,
 ) -> Result<()> {
     let media_type = &track.media_type;
@@ -566,6 +600,7 @@ async fn queue_next(
                     sink,
                     gapless,
                     next_duration_opt,
+                    common_media_title_cb(media_title.clone()),
                 );
             } else {
                 append_to_sink(
@@ -574,6 +609,7 @@ async fn queue_next(
                     sink,
                     gapless,
                     total_duration,
+                    common_media_title_cb(media_title.clone()),
                 );
             }
 
@@ -585,7 +621,6 @@ async fn queue_next(
             if let Some(file_path) = track.podcast_localfile.clone() {
                 let file = File::open(Path::new(&file_path))
                     .context("Failed to open local podcast file")?;
-
                 if enqueue {
                     append_to_sink_queue(
                         Box::new(BufferedSource::new_default_size(file)),
@@ -593,6 +628,7 @@ async fn queue_next(
                         sink,
                         gapless,
                         next_duration_opt,
+                        common_media_title_cb(media_title.clone()),
                     );
                 } else {
                     append_to_sink(
@@ -601,9 +637,9 @@ async fn queue_next(
                         sink,
                         gapless,
                         total_duration,
+                        common_media_title_cb(media_title.clone()),
                     );
                 }
-
                 return Ok(());
             }
 
@@ -616,26 +652,11 @@ async fn queue_next(
 
             let reader = StreamDownload::from_stream(
                 stream,
-                AdaptiveStorageProvider::new(
-                    TempStorageProvider::with_prefix(".termusic-stream-cache-"),
-                    // ensure we have enough buffer space to store the prefetch data
-                    NonZeroUsize::new(usize::try_from(settings.get_prefetch_bytes() * 2)?).unwrap(),
-                ),
+                TempStorageProvider::with_prefix(".termusic-stream-cache-"),
                 settings,
             )
             .await?;
 
-            // let reader = StreamDownload::from_stream(
-            //     stream,
-            //     BoundedStorageProvider::new(
-            //         MemoryStorageProvider,
-            //         // ensure we have enough buffer space to store the prefetch data
-            //         NonZeroUsize::new(usize::try_from(settings.get_prefetch_bytes() * 10)?)
-            //             .unwrap(),
-            //     ),
-            //     settings,
-            // )
-            // .await?;
             if enqueue {
                 append_to_sink_queue(
                     Box::new(ReadSeekSource::new(reader, file_len)),
@@ -643,6 +664,7 @@ async fn queue_next(
                     sink,
                     gapless,
                     next_duration_opt,
+                    common_media_title_cb(media_title.clone()),
                 );
             } else {
                 append_to_sink(
@@ -651,9 +673,9 @@ async fn queue_next(
                     sink,
                     gapless,
                     total_duration,
+                    common_media_title_cb(media_title.clone()),
                 );
             }
-
             Ok(())
         }
 
@@ -692,7 +714,7 @@ async fn queue_next(
             // // Modify this to what the actual headers said
             // let meta_interval = 8192;
 
-            let radio_title_clone = radio_title.clone();
+            let media_title_clone = media_title.clone();
 
             let cb = move |title: &str| {
                 let new_title = if title.is_empty() {
@@ -701,7 +723,7 @@ async fn queue_next(
                     title.to_string()
                 };
 
-                *radio_title_clone.lock() = new_title;
+                *media_title_clone.lock() = new_title;
             };
 
             // set initial title to what the header says
