@@ -2,22 +2,25 @@ mod cli;
 mod logger;
 mod music_player_service;
 
-use std::path::Path;
+use std::path::{Path, PathBuf};
 use std::sync::Arc;
 #[cfg(any(feature = "gst", feature = "rusty"))]
 use std::time::Duration;
 
-use anyhow::{Context, Result};
+use anyhow::{bail, Context, Result};
 use clap::Parser;
 use music_player_service::MusicPlayerService;
 use parking_lot::Mutex;
-use termusiclib::config::Settings;
+use termusiclib::config::v2::server::config_extra::ServerConfigVersionedDefaulted;
+use termusiclib::config::v2::server::ScanDepth;
+use termusiclib::config::ServerOverlay;
 use termusiclib::track::MediaType;
+use termusiclib::{podcast, utils};
 use termusicplayback::player::music_player_server::MusicPlayerServer;
 use termusicplayback::player::{GetProgressResponse, PlayerTime};
 use termusicplayback::{
     Backend, BackendSelect, GeneralPlayer, PlayerCmd, PlayerCmdReciever, PlayerCmdSender,
-    PlayerProgress, PlayerTrait, Status,
+    PlayerProgress, PlayerTrait, SpeedSigned, Status, VolumeSigned,
 };
 use tokio::runtime::Handle;
 use tokio::sync::oneshot;
@@ -28,6 +31,8 @@ use tonic::transport::Server;
 extern crate log;
 
 pub const MAX_DEPTH: usize = 4;
+pub const VOLUME_STEP: VolumeSigned = 5;
+pub const SPEED_STEP: SpeedSigned = 1;
 
 /// Stats for the music player responses
 #[derive(Debug, Clone, PartialEq)]
@@ -91,12 +96,19 @@ fn main() -> Result<()> {
 async fn actual_main() -> Result<()> {
     let args = cli::Args::parse();
     let _ = logger::setup(&args);
-    info!("background thread start");
+    let config = get_config(&args)?;
 
+    if let Some(action) = args.action {
+        return execute_action(action, &config);
+    }
+
+    #[cfg(not(any(feature = "rusty", feature = "gst", feature = "mpv")))]
+    compile_error!("No useable backend feature!");
+
+    info!("Server starting...");
     let (cmd_tx, cmd_rx) = tokio::sync::mpsc::unbounded_channel();
 
     let music_player_service: MusicPlayerService = MusicPlayerService::new(cmd_tx.clone());
-    let config = get_config(&args)?;
     let playerstats = music_player_service.player_stats.clone();
 
     let cmd_tx_ctrlc = cmd_tx.clone();
@@ -109,7 +121,7 @@ async fn actual_main() -> Result<()> {
     })
     .expect("Error setting Ctrl-C handler");
 
-    let addr = std::net::SocketAddr::new(config.player_interface, config.player_port);
+    let addr = std::net::SocketAddr::from(config.settings.com);
 
     // workaround to print address once sever "actually" is started and address is known
     // see https://github.com/hyperium/tonic/issues/351
@@ -138,6 +150,8 @@ async fn actual_main() -> Result<()> {
             .serve_with_incoming(tcp_stream),
     );
 
+    info!("Server started and listening on {}", addr);
+
     // await the oneshot completing in a async fashion
     player_handle_os_rx.await??;
     // do this *after* the oneshot, because this is a blocking operation
@@ -152,7 +166,7 @@ fn player_loop(
     backend: BackendSelect,
     cmd_tx: PlayerCmdSender,
     mut cmd_rx: PlayerCmdReciever,
-    config: Settings,
+    config: ServerOverlay,
     playerstats: Arc<Mutex<PlayerStats>>,
 ) -> Result<()> {
     let mut player = GeneralPlayer::new_backend(backend, config, cmd_tx)?;
@@ -163,7 +177,7 @@ fn player_loop(
                 info!("about to finish signal received");
                 if !player.playlist.is_empty()
                     && !player.playlist.has_next_track()
-                    && player.config.read().player_gapless
+                    && player.config.read().settings.player.gapless
                 {
                     player.enqueue_next_from_playlist();
                 }
@@ -174,13 +188,15 @@ fn player_loop(
                 if let Err(e) = player.playlist.save() {
                     error!("error when saving playlist: {e}");
                 };
-                if let Err(e) = player.config.read().save() {
+                if let Err(e) =
+                    ServerConfigVersionedDefaulted::save_config_path(&player.config.read().settings)
+                {
                     error!("error when saving config: {e}");
                 };
                 std::process::exit(0);
             }
             PlayerCmd::CycleLoop => {
-                player.config.write().player_loop_mode = player.playlist.cycle_loop_mode();
+                player.config.write().settings.player.loop_mode = player.playlist.cycle_loop_mode();
             }
             PlayerCmd::Eos => {
                 info!("Eos received");
@@ -212,8 +228,9 @@ fn player_loop(
                 player.previous();
             }
             PlayerCmd::ReloadConfig => {
-                player.config.write().load()?;
-                info!("config reloaded");
+                if let Err(err) = player.reload_config() {
+                    error!("Reloading config failed, using old: {:#?}", err);
+                }
             }
             PlayerCmd::ReloadPlaylist => {
                 player.playlist.reload_tracks().ok();
@@ -238,25 +255,23 @@ fn player_loop(
                 player.next();
             }
             PlayerCmd::SpeedDown => {
-                let new_speed = player.speed_down();
+                let new_speed = player.add_speed(-SPEED_STEP);
                 info!("after speed down: {}", new_speed);
-                player.config.write().player_speed = new_speed;
+                player.config.write().settings.player.speed = new_speed;
                 let mut p_tick = playerstats.lock();
                 p_tick.speed = new_speed;
             }
 
             PlayerCmd::SpeedUp => {
-                let new_speed = player.speed_up();
+                let new_speed = player.add_speed(SPEED_STEP);
                 info!("after speed up: {}", new_speed);
-                player.config.write().player_speed = new_speed;
+                player.config.write().settings.player.speed = new_speed;
                 let mut p_tick = playerstats.lock();
                 p_tick.speed = new_speed;
             }
             PlayerCmd::Tick => {
                 // info!("tick received");
-                if player.config.read().player_use_mpris {
-                    player.update_mpris();
-                }
+                player.mpris_handle_events();
                 let mut p_tick = playerstats.lock();
                 p_tick.status = player.playlist.status().as_u32();
                 // branch to auto-start playing if status is "stopped"(not paused) and playlist is not empty anymore
@@ -274,7 +289,8 @@ fn player_loop(
                     continue;
                 }
                 if let Some(progress) = player.get_progress() {
-                    p_tick.progress = progress
+                    p_tick.progress = progress;
+                    player.mpris_update_progress(&p_tick.progress);
                 }
                 if player.current_track_updated {
                     p_tick.current_track_index = player.playlist.get_current_track_index() as u32;
@@ -324,19 +340,21 @@ fn player_loop(
             }
             PlayerCmd::VolumeDown => {
                 info!("before volumedown: {}", player.volume());
-                let new_volume = player.volume_down();
-                player.config.write().player_volume = new_volume;
+                let new_volume = player.add_volume(-VOLUME_STEP);
+                player.config.write().settings.player.volume = new_volume;
                 info!("after volumedown: {}", new_volume);
                 let mut p_tick = playerstats.lock();
                 p_tick.volume = new_volume;
+                player.mpris_volume_update();
             }
             PlayerCmd::VolumeUp => {
                 info!("before volumeup: {}", player.volume());
-                let new_volume = player.volume_up();
-                player.config.write().player_volume = new_volume;
+                let new_volume = player.add_volume(VOLUME_STEP);
+                player.config.write().settings.player.volume = new_volume;
                 info!("after volumeup: {}", new_volume);
                 let mut p_tick = playerstats.lock();
                 p_tick.volume = new_volume;
+                player.mpris_volume_update();
             }
             PlayerCmd::Pause => {
                 player.pause();
@@ -363,28 +381,29 @@ fn ticker_thread(cmd_tx: PlayerCmdSender) -> Result<()> {
     Ok(())
 }
 
-fn get_config(args: &cli::Args) -> Result<Settings> {
-    let mut config = Settings::default();
-    config.load()?;
+fn get_config(args: &cli::Args) -> Result<ServerOverlay> {
+    let config = ServerConfigVersionedDefaulted::from_config_path()?.into_settings();
 
-    config.disable_album_art_from_cli = args.disable_cover;
-    config.disable_discord_rpc_from_cli = args.disable_discord;
+    let max_depth = args.max_depth.map(ScanDepth::Limited);
 
-    if let Some(dir) = &args.music_directory {
-        config.music_dir_from_cli = get_path(dir);
-    }
-
-    config.max_depth_cli = match args.max_depth {
-        Some(d) => d,
-        None => MAX_DEPTH,
+    let music_dir = if let Some(ref dir) = args.music_directory {
+        Some(get_path(dir).context("cli provided music-dir")?)
+    } else {
+        None
     };
 
-    Ok(config)
+    let overlay = ServerOverlay {
+        settings: config,
+        music_dir_overwrite: music_dir,
+        disable_discord_status: args.disable_discord,
+        library_scan_depth: max_depth,
+    };
+
+    Ok(overlay)
 }
 
-fn get_path(dir: &str) -> Option<String> {
-    let music_dir: Option<String>;
-    let mut path = Path::new(&dir).to_path_buf();
+fn get_path(dir: &Path) -> Result<PathBuf> {
+    let mut path = dir.to_path_buf();
 
     if path.exists() {
         if !path.has_root() {
@@ -397,10 +416,32 @@ fn get_path(dir: &str) -> Option<String> {
             path = p_canonical;
         }
 
-        music_dir = Some(path.to_string_lossy().to_string());
-    } else {
-        error!("Error: unknown directory '{dir}'");
-        std::process::exit(0);
+        return Ok(path);
     }
-    music_dir
+
+    bail!("Error: non-existing directory '{}'", dir.display());
+}
+
+fn execute_action(action: cli::Action, config: &ServerOverlay) -> Result<()> {
+    match action {
+        cli::Action::Import { file } => {
+            println!("need to import from file {}", file.display());
+
+            let path = get_path(&file).context("import cli file-path")?;
+            let config_dir_path =
+                utils::get_app_config_path().context("getting app-config-path")?;
+
+            podcast::import_from_opml(&config_dir_path, &config.settings.podcast, &path)
+                .context("import opml")?;
+        }
+        cli::Action::Export { file } => {
+            println!("need to export to file {}", file.display());
+            let path = utils::absolute_path(&file)?;
+            let config_dir_path =
+                utils::get_app_config_path().context("getting app-config-path")?;
+            podcast::export_to_opml(&config_dir_path, &path).context("export opml")?;
+        }
+    };
+
+    Ok(())
 }
