@@ -30,24 +30,40 @@ mod cli;
 mod logger;
 mod ui;
 
-use anyhow::Result;
+use anyhow::{bail, Context, Result};
 use clap::Parser;
-use config::Settings;
 use flexi_logger::LogSpecification;
 use std::net::SocketAddr;
-use std::process;
+use std::path::PathBuf;
 use std::time::{Duration, Instant};
 use std::{error::Error, path::Path};
+use termusiclib::config::v2::server::config_extra::ServerConfigVersionedDefaulted;
+use termusiclib::config::v2::server::ScanDepth;
+use termusiclib::config::v2::tui::config_extra::TuiConfigVersionedDefaulted;
+use termusiclib::config::{
+    new_shared_server_settings, new_shared_tui_settings, ServerOverlay, SharedServerSettings,
+    SharedTuiSettings, TuiOverlay,
+};
 use termusicplayback::player::music_player_client::MusicPlayerClient;
 
 use sysinfo::{Pid, ProcessStatus, System};
-use termusiclib::{config, podcast, utils};
+use termusiclib::{podcast, utils};
 use ui::UI;
+
 #[macro_use]
 extern crate log;
 
 pub const MAX_DEPTH: usize = 4;
 
+/// Combined Shared Settings, because the TUI needs access to both configs
+///
+/// also this exists instead of a unnamed tuple
+#[derive(Debug, Clone)]
+pub struct CombinedSettings {
+    pub server: SharedServerSettings,
+    pub tui: SharedTuiSettings,
+}
+
 fn main() -> Result<()> {
     // print error to the log and then throw it
     if let Err(err) = actual_main() {
@@ -66,9 +82,7 @@ async fn actual_main() -> Result<()> {
     let config = get_config(&args)?;
 
     if let Some(action) = args.action {
-        execute_action(action, &config);
-
-        return Ok(());
+        return execute_action(action, &config);
     }
 
     // launch the daemon if it isn't already
@@ -102,9 +116,13 @@ async fn actual_main() -> Result<()> {
     if launch_daemon {
         let mut server_args = vec![];
 
-        if args.log_options.log_to_file {
-            server_args.push("--log-to-file");
-        }
+        // dont clone over "log-to-file", because default is "true" now, and otherwise can be controlled via TMS_LOGTOFILE or TMS_LOGFILE
+        // server_args.push("--log-to-file");
+        // if args.log_options.log_to_file {
+        //     server_args.push("true");
+        // } else {
+        //     server_args.push("false");
+        // }
 
         if args.log_options.file_color_log {
             server_args.push("--log-filecolor");
@@ -133,11 +151,17 @@ async fn actual_main() -> Result<()> {
     }
 
     info!("Waiting until connected");
-    let client = wait_till_connected(
-        SocketAddr::new(config.player_interface, config.player_port),
-        pid,
-    )
-    .await?;
+
+    let client = {
+        let addr = {
+            let config_read = config.tui.read();
+            SocketAddr::from(*config_read.settings.get_com().ok_or(anyhow::anyhow!(
+                "Expected tui-com settings to be resolved at this point"
+            ))?)
+        };
+
+        wait_till_connected(addr, pid).await?
+    };
     info!("Connected!");
 
     let mut ui = UI::new(config, client).await?;
@@ -211,28 +235,41 @@ fn find_source<E: Error + 'static>(err: &dyn Error) -> Option<&E> {
     None
 }
 
-fn get_config(args: &cli::Args) -> Result<Settings> {
-    let mut config = Settings::default();
-    config.load()?;
+fn get_config(args: &cli::Args) -> Result<CombinedSettings> {
+    let config_server = ServerConfigVersionedDefaulted::from_config_path()?.into_settings();
 
-    config.disable_album_art_from_cli = args.disable_cover;
-    config.disable_discord_rpc_from_cli = args.disable_discord;
+    let max_depth = args.max_depth.map(ScanDepth::Limited);
 
-    if let Some(dir) = &args.music_directory {
-        config.music_dir_from_cli = get_path(dir);
-    }
-
-    config.max_depth_cli = match args.max_depth {
-        Some(d) => d,
-        None => MAX_DEPTH,
+    let music_dir = if let Some(ref dir) = args.music_directory {
+        Some(get_path(dir).context("resolving cli music-dir")?)
+    } else {
+        None
     };
 
-    Ok(config)
+    let overlay_server = ServerOverlay {
+        settings: config_server,
+        music_dir_overwrite: music_dir,
+        disable_discord_status: args.disable_discord,
+        library_scan_depth: max_depth,
+    };
+
+    let config_tui = TuiConfigVersionedDefaulted::from_config_path()?.into_settings();
+
+    let coverart_hidden_overwrite = if args.disable_cover { Some(true) } else { None };
+
+    let overlay_tui = TuiOverlay {
+        settings: config_tui,
+        coverart_hidden_overwrite,
+    };
+
+    Ok(CombinedSettings {
+        server: new_shared_server_settings(overlay_server),
+        tui: new_shared_tui_settings(overlay_tui),
+    })
 }
 
-fn get_path(dir: &str) -> Option<String> {
-    let music_dir: Option<String>;
-    let mut path = Path::new(&dir).to_path_buf();
+fn get_path(dir: &Path) -> Result<PathBuf> {
+    let mut path = dir.to_path_buf();
 
     if path.exists() {
         if !path.has_root() {
@@ -245,53 +282,36 @@ fn get_path(dir: &str) -> Option<String> {
             path = p_canonical;
         }
 
-        music_dir = Some(path.to_string_lossy().to_string());
-    } else {
-        error!("Error: unknown directory '{dir}'");
-        process::exit(0);
+        return Ok(path);
     }
-    music_dir
+
+    bail!("Error: non-existing directory '{}'", dir.display());
 }
 
-fn execute_action(action: cli::Action, config: &Settings) {
+fn execute_action(action: cli::Action, config: &CombinedSettings) -> Result<()> {
     match action {
         cli::Action::Import { file } => {
-            println!("need to import from file {file}");
+            println!("need to import from file {}", file.display());
 
-            let path_str = get_path(&file);
-            let db_path = utils::get_app_config_path();
+            let path = get_path(&file).context("import cli file-path")?;
+            let config_dir_path =
+                utils::get_app_config_path().context("getting app-config-path")?;
 
-            if let (Some(path_str), Ok(db_path)) = (path_str, db_path) {
-                if let Err(e) = podcast::import_from_opml(db_path.as_path(), config, &path_str) {
-                    error!("Error when import file {file}: {e}");
-                }
-            }
+            podcast::import_from_opml(
+                &config_dir_path,
+                &config.server.read().settings.podcast,
+                &path,
+            )
+            .context("import opml")?;
         }
         cli::Action::Export { file } => {
-            println!("need to export to file {file}");
-            let path_string = get_path_export(&file);
-            if let Ok(db_path) = utils::get_app_config_path() {
-                println!("export to {path_string}");
-                if let Err(e) = podcast::export_to_opml(db_path.as_path(), &path_string) {
-                    error!("Error when export file {file}: {e}");
-                }
-            }
+            println!("need to export to file {}", file.display());
+            let path = utils::absolute_path(&file)?;
+            let config_dir_path =
+                utils::get_app_config_path().context("getting app-config-path")?;
+            podcast::export_to_opml(&config_dir_path, &path).context("export opml")?;
         }
     };
-}
 
-fn get_path_export(dir: &str) -> String {
-    let mut path = Path::new(&dir).to_path_buf();
-
-    if !path.has_root() {
-        if let Ok(p_base) = std::env::current_dir() {
-            path = p_base.join(path);
-        }
-    }
-
-    if let Ok(p_canonical) = path.canonicalize() {
-        path = p_canonical;
-    }
-
-    path.to_string_lossy().to_string()
+    Ok(())
 }
