@@ -1,13 +1,13 @@
 use crate::ui::Model;
 use std::path::Path;
-use termusiclib::sqlite::SearchCriteria;
+use termusiclib::config::SharedTuiSettings;
+use termusiclib::library_db::SearchCriteria;
 use termusiclib::types::{DBMsg, Id, Msg};
 use termusiclib::utils::{is_playlist, playlist_get_vec};
-use termusicplayback::SharedSettings;
 use tui_realm_stdlib::List;
 use tuirealm::command::{Cmd, CmdResult, Direction, Position};
+use tuirealm::props::Borders;
 use tuirealm::props::{Alignment, BorderType, TableBuilder, TextSpan};
-use tuirealm::props::{Borders, Color};
 use tuirealm::{
     event::{Key, KeyEvent, KeyModifiers, NoUserEvent},
     AttrValue, Attribute, Component, Event, MockComponent, State, StateValue,
@@ -18,43 +18,25 @@ pub struct DBListCriteria {
     component: List,
     on_key_tab: Msg,
     on_key_backtab: Msg,
-    config: SharedSettings,
+    config: SharedTuiSettings,
 }
 
 impl DBListCriteria {
-    pub fn new(config: SharedSettings, on_key_tab: Msg, on_key_backtab: Msg) -> Self {
+    pub fn new(config: SharedTuiSettings, on_key_tab: Msg, on_key_backtab: Msg) -> Self {
         let component = {
             let config = config.read();
             List::default()
                 .borders(
-                    Borders::default().modifiers(BorderType::Rounded).color(
-                        config
-                            .style_color_symbol
-                            .library_border()
-                            .unwrap_or(Color::Blue),
-                    ),
-                )
-                .background(
-                    config
-                        .style_color_symbol
-                        .library_background()
-                        .unwrap_or(Color::Reset),
-                )
-                .foreground(
-                    config
-                        .style_color_symbol
-                        .library_foreground()
-                        .unwrap_or(Color::Yellow),
+                    Borders::default()
+                        .modifiers(BorderType::Rounded)
+                        .color(config.settings.theme.library_border()),
                 )
+                .background(config.settings.theme.library_background())
+                .foreground(config.settings.theme.library_foreground())
                 .title(" DataBase ", Alignment::Left)
                 .scroll(true)
-                .highlighted_color(
-                    config
-                        .style_color_symbol
-                        .library_highlight()
-                        .unwrap_or(Color::LightBlue),
-                )
-                .highlighted_str(&config.style_color_symbol.library_highlight_symbol)
+                .highlighted_color(config.settings.theme.library_highlight())
+                .highlighted_str(&config.settings.theme.style.library.highlight_symbol)
                 .rewind(false)
                 .step(4)
                 .scroll(true)
@@ -85,7 +67,7 @@ impl DBListCriteria {
 impl Component<Msg, NoUserEvent> for DBListCriteria {
     fn on(&mut self, ev: Event<NoUserEvent>) -> Option<Msg> {
         let config = self.config.clone();
-        let keys = &config.read().keys;
+        let keys = &config.read().settings.keys;
         let _cmd_result = match ev {
             Event::Keyboard(KeyEvent {
                 code: Key::Down,
@@ -104,7 +86,7 @@ impl Component<Msg, NoUserEvent> for DBListCriteria {
                 code: Key::Up,
                 modifiers: KeyModifiers::NONE,
             }) => self.perform(Cmd::Move(Direction::Up)),
-            Event::Keyboard(key) if key == keys.global_down.key_event() => {
+            Event::Keyboard(key) if key == keys.navigation_keys.down.get() => {
                 if let Some(AttrValue::Table(t)) = self.query(Attribute::Content) {
                     if let State::One(StateValue::Usize(index)) = self.state() {
                         if index >= t.len() - 1 {
@@ -114,7 +96,7 @@ impl Component<Msg, NoUserEvent> for DBListCriteria {
                 }
                 self.perform(Cmd::Move(Direction::Down))
             }
-            Event::Keyboard(key) if key == keys.global_up.key_event() => {
+            Event::Keyboard(key) if key == keys.navigation_keys.up.get() => {
                 self.perform(Cmd::Move(Direction::Up))
             }
             Event::Keyboard(KeyEvent {
@@ -125,10 +107,10 @@ impl Component<Msg, NoUserEvent> for DBListCriteria {
                 code: Key::PageUp,
                 modifiers: KeyModifiers::NONE,
             }) => self.perform(Cmd::Scroll(Direction::Up)),
-            Event::Keyboard(key) if key == keys.global_goto_top.key_event() => {
+            Event::Keyboard(key) if key == keys.navigation_keys.goto_top.get() => {
                 self.perform(Cmd::GoTo(Position::Begin))
             }
-            Event::Keyboard(key) if key == keys.global_goto_bottom.key_event() => {
+            Event::Keyboard(key) if key == keys.navigation_keys.goto_bottom.get() => {
                 self.perform(Cmd::GoTo(Position::End))
             }
             Event::Keyboard(KeyEvent {
@@ -158,7 +140,7 @@ impl Component<Msg, NoUserEvent> for DBListCriteria {
                 modifiers: KeyModifiers::SHIFT,
             }) => return Some(self.on_key_backtab.clone()),
 
-            Event::Keyboard(keyevent) if keyevent == keys.library_search.key_event() => {
+            Event::Keyboard(keyevent) if keyevent == keys.library_keys.search.get() => {
                 return Some(Msg::GeneralSearch(crate::ui::GSMsg::PopupShowDatabase))
             }
             _ => CmdResult::None,
@@ -172,43 +154,25 @@ pub struct DBListSearchResult {
     component: List,
     on_key_tab: Msg,
     on_key_backtab: Msg,
-    config: SharedSettings,
+    config: SharedTuiSettings,
 }
 
 impl DBListSearchResult {
-    pub fn new(config: SharedSettings, on_key_tab: Msg, on_key_backtab: Msg) -> Self {
+    pub fn new(config: SharedTuiSettings, on_key_tab: Msg, on_key_backtab: Msg) -> Self {
         let component = {
             let config = config.read();
             List::default()
                 .borders(
-                    Borders::default().modifiers(BorderType::Rounded).color(
-                        config
-                            .style_color_symbol
-                            .library_border()
-                            .unwrap_or(Color::Blue),
-                    ),
-                )
-                .background(
-                    config
-                        .style_color_symbol
-                        .library_background()
-                        .unwrap_or(Color::Reset),
-                )
-                .foreground(
-                    config
-                        .style_color_symbol
-                        .library_foreground()
-                        .unwrap_or(Color::Yellow),
+                    Borders::default()
+                        .modifiers(BorderType::Rounded)
+                        .color(config.settings.theme.library_border()),
                 )
+                .background(config.settings.theme.library_background())
+                .foreground(config.settings.theme.library_foreground())
                 .title(" Result ", Alignment::Left)
                 .scroll(true)
-                .highlighted_color(
-                    config
-                        .style_color_symbol
-                        .library_highlight()
-                        .unwrap_or(Color::LightBlue),
-                )
-                .highlighted_str(&config.style_color_symbol.library_highlight_symbol)
+                .highlighted_color(config.settings.theme.library_highlight())
+                .highlighted_str(&config.settings.theme.style.library.highlight_symbol)
                 .rewind(false)
                 .step(4)
                 .scroll(true)
@@ -231,7 +195,7 @@ impl DBListSearchResult {
 impl Component<Msg, NoUserEvent> for DBListSearchResult {
     fn on(&mut self, ev: Event<NoUserEvent>) -> Option<Msg> {
         let config = self.config.clone();
-        let keys = &config.read().keys;
+        let keys = &config.read().settings.keys;
         let _cmd_result = match ev {
             Event::Keyboard(KeyEvent {
                 code: Key::Down,
@@ -257,7 +221,7 @@ impl Component<Msg, NoUserEvent> for DBListSearchResult {
                 }
                 self.perform(Cmd::Move(Direction::Up))
             }
-            Event::Keyboard(key) if key == keys.global_down.key_event() => {
+            Event::Keyboard(key) if key == keys.navigation_keys.down.get() => {
                 if let Some(AttrValue::Table(t)) = self.query(Attribute::Content) {
                     if let State::One(StateValue::Usize(index)) = self.state() {
                         if index >= t.len() - 1 {
@@ -267,7 +231,7 @@ impl Component<Msg, NoUserEvent> for DBListSearchResult {
                 }
                 self.perform(Cmd::Move(Direction::Down))
             }
-            Event::Keyboard(key) if key == keys.global_up.key_event() => {
+            Event::Keyboard(key) if key == keys.navigation_keys.up.get() => {
                 if let State::One(StateValue::Usize(index)) = self.state() {
                     if index == 0 {
                         return Some(self.on_key_backtab.clone());
@@ -283,10 +247,10 @@ impl Component<Msg, NoUserEvent> for DBListSearchResult {
                 code: Key::PageUp,
                 modifiers: KeyModifiers::NONE,
             }) => self.perform(Cmd::Scroll(Direction::Up)),
-            Event::Keyboard(key) if key == keys.global_goto_top.key_event() => {
+            Event::Keyboard(key) if key == keys.navigation_keys.goto_top.get() => {
                 self.perform(Cmd::GoTo(Position::Begin))
             }
-            Event::Keyboard(key) if key == keys.global_goto_bottom.key_event() => {
+            Event::Keyboard(key) if key == keys.navigation_keys.goto_bottom.get() => {
                 self.perform(Cmd::GoTo(Position::End))
             }
             Event::Keyboard(KeyEvent {
@@ -316,7 +280,7 @@ impl Component<Msg, NoUserEvent> for DBListSearchResult {
                 modifiers: KeyModifiers::SHIFT,
             }) => return Some(self.on_key_backtab.clone()),
 
-            Event::Keyboard(keyevent) if keyevent == keys.library_search.key_event() => {
+            Event::Keyboard(keyevent) if keyevent == keys.library_keys.search.get() => {
                 return Some(Msg::GeneralSearch(crate::ui::GSMsg::PopupShowDatabase))
             }
 
@@ -331,43 +295,25 @@ pub struct DBListSearchTracks {
     component: List,
     on_key_tab: Msg,
     on_key_backtab: Msg,
-    config: SharedSettings,
+    config: SharedTuiSettings,
 }
 
 impl DBListSearchTracks {
-    pub fn new(config: SharedSettings, on_key_tab: Msg, on_key_backtab: Msg) -> Self {
+    pub fn new(config: SharedTuiSettings, on_key_tab: Msg, on_key_backtab: Msg) -> Self {
         let component = {
             let config = config.read();
             List::default()
                 .borders(
-                    Borders::default().modifiers(BorderType::Rounded).color(
-                        config
-                            .style_color_symbol
-                            .library_border()
-                            .unwrap_or(Color::Blue),
-                    ),
-                )
-                .background(
-                    config
-                        .style_color_symbol
-                        .library_background()
-                        .unwrap_or(Color::Reset),
-                )
-                .foreground(
-                    config
-                        .style_color_symbol
-                        .library_foreground()
-                        .unwrap_or(Color::Yellow),
+                    Borders::default()
+                        .modifiers(BorderType::Rounded)
+                        .color(config.settings.theme.library_border()),
                 )
+                .background(config.settings.theme.library_background())
+                .foreground(config.settings.theme.library_foreground())
                 .title(" Tracks ", Alignment::Left)
                 .scroll(true)
-                .highlighted_color(
-                    config
-                        .style_color_symbol
-                        .library_highlight()
-                        .unwrap_or(Color::LightBlue),
-                )
-                .highlighted_str(&config.style_color_symbol.library_highlight_symbol)
+                .highlighted_color(config.settings.theme.library_highlight())
+                .highlighted_str(&config.settings.theme.style.library.highlight_symbol)
                 .rewind(false)
                 .step(4)
                 .scroll(true)
@@ -390,7 +336,7 @@ impl DBListSearchTracks {
 impl Component<Msg, NoUserEvent> for DBListSearchTracks {
     fn on(&mut self, ev: Event<NoUserEvent>) -> Option<Msg> {
         let config = self.config.clone();
-        let keys = &config.read().keys;
+        let keys = &config.read().settings.keys;
         let _cmd_result = match ev {
             Event::Keyboard(KeyEvent {
                 code: Key::Down,
@@ -407,10 +353,10 @@ impl Component<Msg, NoUserEvent> for DBListSearchTracks {
                 }
                 self.perform(Cmd::Move(Direction::Up))
             }
-            Event::Keyboard(key) if key == keys.global_down.key_event() => {
+            Event::Keyboard(key) if key == keys.navigation_keys.down.get() => {
                 self.perform(Cmd::Move(Direction::Down))
             }
-            Event::Keyboard(key) if key == keys.global_up.key_event() => {
+            Event::Keyboard(key) if key == keys.navigation_keys.up.get() => {
                 if let State::One(StateValue::Usize(index)) = self.state() {
                     if index == 0 {
                         return Some(self.on_key_backtab.clone());
@@ -426,10 +372,10 @@ impl Component<Msg, NoUserEvent> for DBListSearchTracks {
                 code: Key::PageUp,
                 modifiers: KeyModifiers::NONE,
             }) => self.perform(Cmd::Scroll(Direction::Up)),
-            Event::Keyboard(key) if key == keys.global_goto_top.key_event() => {
+            Event::Keyboard(key) if key == keys.navigation_keys.goto_top.get() => {
                 self.perform(Cmd::GoTo(Position::Begin))
             }
-            Event::Keyboard(key) if key == keys.global_goto_bottom.key_event() => {
+            Event::Keyboard(key) if key == keys.navigation_keys.goto_bottom.get() => {
                 self.perform(Cmd::GoTo(Position::End))
             }
             Event::Keyboard(KeyEvent {
@@ -450,17 +396,17 @@ impl Component<Msg, NoUserEvent> for DBListSearchTracks {
                 modifiers: KeyModifiers::SHIFT,
             }) => return Some(self.on_key_backtab.clone()),
 
-            Event::Keyboard(keyevent) if keyevent == keys.global_right.key_event() => {
+            Event::Keyboard(keyevent) if keyevent == keys.database_keys.add_selected.get() => {
                 if let State::One(StateValue::Usize(index)) = self.state() {
                     return Some(Msg::DataBase(DBMsg::AddPlaylist(index)));
                 }
                 CmdResult::None
             }
-            Event::Keyboard(keyevent) if keyevent == keys.database_add_all.key_event() => {
+            Event::Keyboard(keyevent) if keyevent == keys.database_keys.add_all.get() => {
                 return Some(Msg::DataBase(DBMsg::AddAllToPlaylist))
             }
 
-            Event::Keyboard(keyevent) if keyevent == keys.library_search.key_event() => {
+            Event::Keyboard(keyevent) if keyevent == keys.library_keys.search.get() => {
                 return Some(Msg::GeneralSearch(crate::ui::GSMsg::PopupShowDatabase))
             }
 
@@ -474,7 +420,7 @@ impl Model {
     pub fn database_sync_tracks(&mut self) {
         let mut table: TableBuilder = TableBuilder::default();
 
-        for (idx, record) in self.db_search_tracks.iter().enumerate() {
+        for (idx, record) in self.dw.search_tracks.iter().enumerate() {
             if idx > 0 {
                 table.add_row();
             }
@@ -494,7 +440,7 @@ impl Model {
                 .add_col(TextSpan::from(" "))
                 .add_col(TextSpan::from(name));
         }
-        if self.db_search_results.is_empty() {
+        if self.dw.search_results.is_empty() {
             table.add_col(TextSpan::from("empty results"));
         }
 
@@ -512,9 +458,9 @@ impl Model {
     pub fn database_sync_results(&mut self) {
         let mut table: TableBuilder = TableBuilder::default();
         let mut index = 0;
-        for (idx, record) in self.db_search_results.iter().enumerate() {
+        for (idx, record) in self.dw.search_results.iter().enumerate() {
             let mut display_name = String::new();
-            match self.db_criteria {
+            match self.dw.criteria {
                 SearchCriteria::Playlist => {
                     let path = Path::new(record);
                     let path_string = path.to_string_lossy().to_string();
@@ -536,7 +482,7 @@ impl Model {
                     }
                 }
                 _ => {
-                    display_name = record.clone();
+                    display_name.clone_from(record);
                 }
             };
             if !display_name.is_empty() {
@@ -550,7 +496,7 @@ impl Model {
                     .add_col(TextSpan::from(display_name));
             }
         }
-        if self.db_search_results.is_empty() {
+        if self.dw.search_results.is_empty() {
             table.add_col(TextSpan::from("empty results"));
         }
 
@@ -567,13 +513,13 @@ impl Model {
     }
 
     pub fn database_update_search_results(&mut self) {
-        match self.db_criteria {
+        match self.dw.criteria {
             SearchCriteria::Playlist => {
-                self.db_search_results = self.database_get_playlist();
+                self.dw.search_results = self.database_get_playlist();
             }
             _ => {
-                if let Ok(results) = self.db.get_criterias(&self.db_criteria) {
-                    self.db_search_results = results;
+                if let Ok(results) = self.db.get_criterias(&self.dw.criteria) {
+                    self.dw.search_results = results;
                 }
             }
         }
@@ -584,7 +530,7 @@ impl Model {
     fn database_get_playlist(&self) -> Vec<String> {
         let mut vec = Vec::new();
 
-        let root = self.tree.root();
+        let root = self.library.tree.root();
         let p: &Path = Path::new(root.id());
         let all_items = walkdir::WalkDir::new(p).follow_links(true);
         for record in all_items
@@ -599,9 +545,9 @@ impl Model {
     }
 
     pub fn database_update_search_tracks(&mut self, index: usize) {
-        match self.db_criteria {
+        match self.dw.criteria {
             SearchCriteria::Playlist => {
-                if let Some(result) = self.db_search_results.get(index) {
+                if let Some(result) = self.dw.search_results.get(index) {
                     if let Ok(vec) = playlist_get_vec(result) {
                         let mut vec_db = Vec::new();
                         for item in vec {
@@ -609,16 +555,16 @@ impl Model {
                                 vec_db.push(i);
                             }
                         }
-                        self.db_search_tracks = vec_db;
+                        self.dw.search_tracks = vec_db;
                     }
                 }
             }
             _ => {
                 if let Ok(vec) = self
                     .db
-                    .get_record_by_criteria(&self.db_search_results[index], &self.db_criteria)
+                    .get_record_by_criteria(&self.dw.search_results[index], &self.dw.criteria)
                 {
-                    self.db_search_tracks = vec;
+                    self.dw.search_tracks = vec;
                 };
             }
         }
@@ -634,7 +580,7 @@ impl Model {
             .remount(
                 Id::DBListCriteria,
                 Box::new(DBListCriteria::new(
-                    self.config.clone(),
+                    self.config_tui.clone(),
                     Msg::DataBase(DBMsg::CriteriaBlurDown),
                     Msg::DataBase(DBMsg::CriteriaBlurUp)
                 )),
@@ -647,7 +593,7 @@ impl Model {
             .remount(
                 Id::DBListSearchResult,
                 Box::new(DBListSearchResult::new(
-                    self.config.clone(),
+                    self.config_tui.clone(),
                     Msg::DataBase(DBMsg::SearchResultBlurDown),
                     Msg::DataBase(DBMsg::SearchResultBlurUp)
                 )),
@@ -659,7 +605,7 @@ impl Model {
             .remount(
                 Id::DBListSearchTracks,
                 Box::new(DBListSearchTracks::new(
-                    self.config.clone(),
+                    self.config_tui.clone(),
                     Msg::DataBase(DBMsg::SearchTracksBlurDown),
                     Msg::DataBase(DBMsg::SearchTracksBlurUp)
                 )),
@@ -667,8 +613,7 @@ impl Model {
             )
             .is_ok());
 
-        self.db_search_results = vec![];
-        self.db_search_tracks = vec![];
+        self.dw.reset_search_results();
         self.database_sync_tracks();
         self.database_sync_results();
     }
@@ -679,7 +624,7 @@ impl Model {
         let search = format!("*{}*", input.to_lowercase());
         let mut db_tracks = vec![];
         if let Ok(tracks) = self.db.get_all_records() {
-            db_tracks = tracks.clone();
+            db_tracks.clone_from(&tracks);
             for record in tracks {
                 if wildmatch::WildMatch::new(&search).matches(&record.artist.to_lowercase())
                     | wildmatch::WildMatch::new(&search).matches(&record.title.to_lowercase())
