@@ -1,18 +1,17 @@
 use crate::ui::{model::TermusicLayout, Model};
-use termusiclib::podcast::Episode;
+use termusiclib::podcast::episode::Episode;
 use termusiclib::track::MediaType;
 use termusiclib::types::{Id, LyricMsg, Msg};
 
 use anyhow::{anyhow, Result};
 use lazy_static::lazy_static;
 use regex::Regex;
-use termusicplayback::SharedSettings;
+use termusiclib::config::SharedTuiSettings;
 use tui_realm_stdlib::Textarea;
-// use tui_realm_textarea::TextArea;
 use tuirealm::command::{Cmd, CmdResult, Direction, Position};
 use tuirealm::event::{Key, KeyEvent, KeyModifiers, NoUserEvent};
 use tuirealm::props::{
-    Alignment, AttrValue, Attribute, BorderType, Borders, Color, PropPayload, PropValue, TextSpan,
+    Alignment, AttrValue, Attribute, BorderType, Borders, PropPayload, PropValue, TextSpan,
 };
 use tuirealm::{Component, Event, MockComponent, State, StateValue};
 
@@ -31,40 +30,25 @@ lazy_static! {
 #[derive(MockComponent)]
 pub struct Lyric {
     component: Textarea,
-    config: SharedSettings,
+    config: SharedTuiSettings,
 }
 
 impl Lyric {
-    pub fn new(config: SharedSettings) -> Self {
+    pub fn new(config: SharedTuiSettings) -> Self {
         let component = {
             let config = config.read();
             Textarea::default()
                 .borders(
                     Borders::default()
-                        .color(
-                            config
-                                .style_color_symbol
-                                .lyric_border()
-                                .unwrap_or(Color::Green),
-                        )
+                        .color(config.settings.theme.lyric_border())
                         .modifiers(BorderType::Rounded),
                 )
-                .background(
-                    config
-                        .style_color_symbol
-                        .lyric_background()
-                        .unwrap_or(Color::Reset),
-                )
-                .foreground(
-                    config
-                        .style_color_symbol
-                        .lyric_foreground()
-                        .unwrap_or(Color::Cyan),
-                )
+                .background(config.settings.theme.lyric_background())
+                .foreground(config.settings.theme.lyric_foreground())
                 .title(" Lyrics ", Alignment::Left)
                 // .wrap(true)
                 .step(4)
-                .highlighted_str(&config.style_color_symbol.playlist_highlight_symbol)
+                .highlighted_str(&config.settings.theme.style.playlist.highlight_symbol)
                 .text_rows(&[TextSpan::new(format!(
                     "{}.",
                     termusicplayback::Status::Stopped
@@ -78,7 +62,7 @@ impl Lyric {
 impl Component<Msg, NoUserEvent> for Lyric {
     fn on(&mut self, ev: Event<NoUserEvent>) -> Option<Msg> {
         let config = self.config.clone();
-        let keys = &config.read().keys;
+        let keys = &config.read().settings.keys;
         let _drop = match ev {
             Event::Keyboard(KeyEvent {
                 code: Key::Down,
@@ -113,17 +97,17 @@ impl Component<Msg, NoUserEvent> for Lyric {
                 modifiers: KeyModifiers::SHIFT,
             }) => return Some(Msg::LyricMessage(LyricMsg::LyricTextAreaBlurUp)),
 
-            Event::Keyboard(key) if key == keys.global_down.key_event() => {
+            Event::Keyboard(key) if key == keys.navigation_keys.down.get() => {
                 self.perform(Cmd::Move(Direction::Down))
             }
-            Event::Keyboard(key) if key == keys.global_up.key_event() => {
+            Event::Keyboard(key) if key == keys.navigation_keys.up.get() => {
                 self.perform(Cmd::Move(Direction::Up))
             }
 
-            Event::Keyboard(key) if key == keys.global_goto_top.key_event() => {
+            Event::Keyboard(key) if key == keys.navigation_keys.goto_top.get() => {
                 self.perform(Cmd::GoTo(Position::Begin))
             }
-            Event::Keyboard(key) if key == keys.global_goto_bottom.key_event() => {
+            Event::Keyboard(key) if key == keys.navigation_keys.goto_bottom.get() => {
                 self.perform(Cmd::GoTo(Position::End))
             }
             _ => CmdResult::None,
@@ -138,7 +122,7 @@ impl Model {
             .app
             .remount(
                 Id::Lyric,
-                Box::new(Lyric::new(self.config.clone())),
+                Box::new(Lyric::new(self.config_tui.clone())),
                 Vec::new()
             )
             .is_ok());
@@ -154,10 +138,10 @@ impl Model {
         if let Some(track) = self.playlist.current_track().cloned() {
             if MediaType::Podcast == track.media_type {
                 if let Some(file) = track.file() {
-                    'outer: for pod in &self.podcasts {
+                    'outer: for pod in &self.podcast.podcasts {
                         for ep in &pod.episodes {
                             if ep.url == file {
-                                pod_title = pod.title.clone();
+                                pod_title.clone_from(&pod.title);
                                 ep_for_lyric = ep.clone();
                                 need_update = true;
                                 break 'outer;
@@ -176,13 +160,14 @@ impl Model {
     }
 
     pub fn lyric_update_for_podcast(&mut self) -> Result<()> {
-        if self.podcasts.is_empty() {
+        if self.podcast.podcasts.is_empty() {
             return Ok(());
         }
         if let Ok(State::One(StateValue::Usize(episode_index))) = self.app.state(&Id::Episode) {
             let podcast_selected = self
+                .podcast
                 .podcasts
-                .get(self.podcasts_index)
+                .get(self.podcast.podcasts_index)
                 .ok_or_else(|| anyhow!("get podcast selected failed."))?
                 .clone();
             let episode_selected = podcast_selected
