@@ -24,7 +24,7 @@
 // -- modules
 mod config_editor;
 mod database;
-mod general_search;
+mod footer;
 mod labels;
 mod lyric;
 mod music_library;
@@ -41,30 +41,23 @@ mod progress;
 /// Tag Editor Controls
 mod tag_editor;
 mod xywh;
-mod youtube_search;
 
 // -- export
 pub use config_editor::*;
 pub use database::{DBListCriteria, DBListSearchResult, DBListSearchTracks};
-pub use general_search::{GSInputPopup, GSTablePopup, Source};
+pub use footer::Footer;
 pub use labels::{DownloadSpinner, LabelGeneric, LabelSpan};
 pub use lyric::Lyric;
 pub use music_library::MusicLibrary;
 pub use playlist::Playlist;
 pub use podcast::{EpisodeList, FeedsList};
-pub use popups::{
-    ErrorPopup, HelpPopup, MessagePopup, PodcastAddPopup, QuitPopup, SavePlaylistConfirm,
-    SavePlaylistPopup,
-};
+pub use popups::general_search::{GSInputPopup, GSTablePopup, Source};
 pub use progress::Progress;
 pub use tag_editor::*;
-use termusicplayback::SharedSettings;
-pub use youtube_search::{YSInputPopup, YSTablePopup};
+use termusiclib::config::v2::tui::keys::Keys;
+use termusiclib::config::SharedTuiSettings;
 
-use crate::config::Keys;
-use crate::ui::{
-    ConfigEditorMsg, GSMsg, Id, IdConfigEditor, IdTagEditor, Model, Msg, PLMsg, XYWHMsg, YSMsg,
-};
+use crate::ui::{ConfigEditorMsg, Id, IdConfigEditor, IdTagEditor, Model, Msg, PLMsg, XYWHMsg};
 use tui_realm_stdlib::Phantom;
 use tuirealm::event::NoUserEvent;
 use tuirealm::{Component, Event, MockComponent, Sub, SubClause, SubEventClause};
@@ -72,11 +65,11 @@ use tuirealm::{Component, Event, MockComponent, Sub, SubClause, SubEventClause};
 #[derive(MockComponent)]
 pub struct GlobalListener {
     component: Phantom,
-    config: SharedSettings,
+    config: SharedTuiSettings,
 }
 
 impl GlobalListener {
-    pub fn new(config: SharedSettings) -> Self {
+    pub fn new(config: SharedTuiSettings) -> Self {
         Self {
             component: Phantom::default(),
             config,
@@ -87,124 +80,112 @@ impl GlobalListener {
 impl Component<Msg, NoUserEvent> for GlobalListener {
     #[allow(clippy::too_many_lines)]
     fn on(&mut self, ev: Event<NoUserEvent>) -> Option<Msg> {
-        let keys = &self.config.read().keys;
+        let keys = &self.config.read().settings.keys;
         match ev {
             Event::WindowResize(..) => Some(Msg::UpdatePhoto),
-            Event::Keyboard(keyevent) if keyevent == keys.global_esc.key_event() => {
-                Some(Msg::QuitPopupShow)
-            }
-            Event::Keyboard(keyevent) if keyevent == keys.global_quit.key_event() => {
-                Some(Msg::QuitPopupShow)
-            }
-            Event::Keyboard(keyevent)
-                if keyevent == keys.global_player_toggle_pause.key_event() =>
-            {
+            Event::Keyboard(keyevent) if keyevent == keys.escape.get() => Some(Msg::QuitPopupShow),
+            Event::Keyboard(keyevent) if keyevent == keys.quit.get() => Some(Msg::QuitPopupShow),
+            Event::Keyboard(keyevent) if keyevent == keys.player_keys.toggle_pause.get() => {
                 Some(Msg::PlayerTogglePause)
             }
-            Event::Keyboard(keyevent) if keyevent == keys.global_player_next.key_event() => {
+            Event::Keyboard(keyevent) if keyevent == keys.player_keys.next_track.get() => {
                 Some(Msg::Playlist(PLMsg::NextSong))
             }
-            Event::Keyboard(keyevent) if keyevent == keys.global_player_previous.key_event() => {
+            Event::Keyboard(keyevent) if keyevent == keys.player_keys.previous_track.get() => {
                 Some(Msg::Playlist(PLMsg::PrevSong))
             }
-            Event::Keyboard(keyevent)
-                if keyevent == keys.global_player_volume_minus_1.key_event() =>
-            {
+            Event::Keyboard(keyevent) if keyevent == keys.player_keys.volume_down.get() => {
                 Some(Msg::PlayerVolumeDown)
             }
-            Event::Keyboard(keyevent)
-                if keyevent == keys.global_player_volume_minus_2.key_event() =>
-            {
-                Some(Msg::PlayerVolumeDown)
-            }
-            Event::Keyboard(keyevent)
-                if keyevent == keys.global_player_volume_plus_1.key_event() =>
-            {
+            // Event::Keyboard(keyevent)
+            //     if keyevent == keys.player_keys.volume_minus_2.get() =>
+            // {
+            //     Some(Msg::PlayerVolumeDown)
+            // }
+            Event::Keyboard(keyevent) if keyevent == keys.player_keys.volume_up.get() => {
                 Some(Msg::PlayerVolumeUp)
             }
-            Event::Keyboard(keyevent)
-                if keyevent == keys.global_player_volume_plus_2.key_event() =>
-            {
-                Some(Msg::PlayerVolumeUp)
-            }
-            Event::Keyboard(keyevent) if keyevent == keys.global_help.key_event() => {
+            // Event::Keyboard(keyevent)
+            //     if keyevent == keys.player_keys.volume_plus_2.get() =>
+            // {
+            //     Some(Msg::PlayerVolumeUp)
+            // }
+            Event::Keyboard(keyevent) if keyevent == keys.select_view_keys.open_help.get() => {
                 Some(Msg::HelpPopupShow)
             }
-            Event::Keyboard(keyevent)
-                if keyevent == keys.global_player_seek_forward.key_event() =>
-            {
+            Event::Keyboard(keyevent) if keyevent == keys.player_keys.seek_forward.get() => {
                 Some(Msg::PlayerSeekForward)
             }
-            Event::Keyboard(keyevent)
-                if keyevent == keys.global_player_seek_backward.key_event() =>
-            {
+            Event::Keyboard(keyevent) if keyevent == keys.player_keys.seek_backward.get() => {
                 Some(Msg::PlayerSeekBackward)
             }
-            Event::Keyboard(keyevent) if keyevent == keys.global_player_speed_up.key_event() => {
+            Event::Keyboard(keyevent) if keyevent == keys.player_keys.speed_up.get() => {
                 Some(Msg::PlayerSpeedUp)
             }
-            Event::Keyboard(keyevent) if keyevent == keys.global_player_speed_down.key_event() => {
+            Event::Keyboard(keyevent) if keyevent == keys.player_keys.speed_down.get() => {
                 Some(Msg::PlayerSpeedDown)
             }
 
             Event::Keyboard(keyevent)
-                if keyevent == keys.global_lyric_adjust_forward.key_event() =>
+                if keyevent == keys.lyric_keys.adjust_offset_forwards.get() =>
             {
                 Some(Msg::LyricAdjustDelay(1000))
             }
             Event::Keyboard(keyevent)
-                if keyevent == keys.global_lyric_adjust_backward.key_event() =>
+                if keyevent == keys.lyric_keys.adjust_offset_backwards.get() =>
             {
                 Some(Msg::LyricAdjustDelay(-1000))
             }
-            Event::Keyboard(keyevent) if keyevent == keys.global_lyric_cycle.key_event() => {
+            Event::Keyboard(keyevent) if keyevent == keys.lyric_keys.cycle_frames.get() => {
                 Some(Msg::LyricCycle)
             }
 
-            Event::Keyboard(keyevent) if keyevent == keys.global_layout_treeview.key_event() => {
+            Event::Keyboard(keyevent) if keyevent == keys.select_view_keys.view_library.get() => {
                 Some(Msg::LayoutTreeView)
             }
 
-            Event::Keyboard(keyevent) if keyevent == keys.global_layout_database.key_event() => {
+            Event::Keyboard(keyevent) if keyevent == keys.select_view_keys.view_database.get() => {
                 Some(Msg::LayoutDataBase)
             }
 
-            Event::Keyboard(keyevent) if keyevent == keys.global_layout_podcast.key_event() => {
+            Event::Keyboard(keyevent) if keyevent == keys.select_view_keys.view_podcasts.get() => {
                 Some(Msg::LayoutPodCast)
             }
 
-            Event::Keyboard(keyevent)
-                if keyevent == keys.global_player_toggle_gapless.key_event() =>
-            {
+            Event::Keyboard(keyevent) if keyevent == keys.player_keys.toggle_prefetch.get() => {
                 Some(Msg::PlayerToggleGapless)
             }
 
-            Event::Keyboard(keyevent) if keyevent == keys.global_config_open.key_event() => {
+            Event::Keyboard(keyevent) if keyevent == keys.select_view_keys.open_config.get() => {
                 Some(Msg::ConfigEditor(ConfigEditorMsg::Open))
             }
 
-            Event::Keyboard(keyevent) if keyevent == keys.global_save_playlist.key_event() => {
+            Event::Keyboard(keyevent) if keyevent == keys.player_keys.save_playlist.get() => {
                 Some(Msg::SavePlaylistPopupShow)
             }
-            Event::Keyboard(keyevent) if keyevent == keys.global_xywh_move_left.key_event() => {
+            Event::Keyboard(keyevent) if keyevent == keys.move_cover_art_keys.move_left.get() => {
                 Some(Msg::Xywh(XYWHMsg::MoveLeft))
             }
-            Event::Keyboard(keyevent) if keyevent == keys.global_xywh_move_right.key_event() => {
+            Event::Keyboard(keyevent) if keyevent == keys.move_cover_art_keys.move_right.get() => {
                 Some(Msg::Xywh(XYWHMsg::MoveRight))
             }
-            Event::Keyboard(keyevent) if keyevent == keys.global_xywh_move_up.key_event() => {
+            Event::Keyboard(keyevent) if keyevent == keys.move_cover_art_keys.move_up.get() => {
                 Some(Msg::Xywh(XYWHMsg::MoveUp))
             }
-            Event::Keyboard(keyevent) if keyevent == keys.global_xywh_move_down.key_event() => {
+            Event::Keyboard(keyevent) if keyevent == keys.move_cover_art_keys.move_down.get() => {
                 Some(Msg::Xywh(XYWHMsg::MoveDown))
             }
-            Event::Keyboard(keyevent) if keyevent == keys.global_xywh_zoom_in.key_event() => {
+            Event::Keyboard(keyevent)
+                if keyevent == keys.move_cover_art_keys.increase_size.get() =>
+            {
                 Some(Msg::Xywh(XYWHMsg::ZoomIn))
             }
-            Event::Keyboard(keyevent) if keyevent == keys.global_xywh_zoom_out.key_event() => {
+            Event::Keyboard(keyevent)
+                if keyevent == keys.move_cover_art_keys.decrease_size.get() =>
+            {
                 Some(Msg::Xywh(XYWHMsg::ZoomOut))
             }
-            Event::Keyboard(keyevent) if keyevent == keys.global_xywh_hide.key_event() => {
+            Event::Keyboard(keyevent) if keyevent == keys.move_cover_art_keys.toggle_hide.get() => {
                 Some(Msg::Xywh(XYWHMsg::Hide))
             }
             _ => None,
@@ -218,123 +199,123 @@ impl Model {
     pub fn subscribe(keys: &Keys) -> Vec<Sub<Id, NoUserEvent>> {
         vec![
             Sub::new(
-                SubEventClause::Keyboard(keys.global_esc.key_event()),
+                SubEventClause::Keyboard(keys.escape.get()),
                 Self::no_popup_mounted_clause(),
             ),
             Sub::new(
-                SubEventClause::Keyboard(keys.global_quit.key_event()),
+                SubEventClause::Keyboard(keys.quit.get()),
                 Self::no_popup_mounted_clause(),
             ),
             Sub::new(
-                SubEventClause::Keyboard(keys.global_player_toggle_pause.key_event()),
+                SubEventClause::Keyboard(keys.player_keys.toggle_pause.get()),
                 Self::no_popup_mounted_clause(),
             ),
             Sub::new(
-                SubEventClause::Keyboard(keys.global_player_next.key_event()),
+                SubEventClause::Keyboard(keys.player_keys.next_track.get()),
                 Self::no_popup_mounted_clause(),
             ),
             Sub::new(
-                SubEventClause::Keyboard(keys.global_player_previous.key_event()),
+                SubEventClause::Keyboard(keys.player_keys.previous_track.get()),
                 Self::no_popup_mounted_clause(),
             ),
             Sub::new(
-                SubEventClause::Keyboard(keys.global_player_speed_up.key_event()),
+                SubEventClause::Keyboard(keys.player_keys.speed_up.get()),
                 Self::no_popup_mounted_clause(),
             ),
             Sub::new(
-                SubEventClause::Keyboard(keys.global_player_speed_down.key_event()),
+                SubEventClause::Keyboard(keys.player_keys.speed_down.get()),
                 Self::no_popup_mounted_clause(),
             ),
             Sub::new(
-                SubEventClause::Keyboard(keys.global_player_volume_minus_1.key_event()),
+                SubEventClause::Keyboard(keys.player_keys.volume_down.get()),
+                Self::no_popup_mounted_clause(),
+            ),
+            // Sub::new(
+            //     SubEventClause::Keyboard(keys.player_keys.volume_minus_2.get()),
+            //     Self::no_popup_mounted_clause(),
+            // ),
+            Sub::new(
+                SubEventClause::Keyboard(keys.player_keys.volume_up.get()),
+                Self::no_popup_mounted_clause(),
+            ),
+            // Sub::new(
+            //     SubEventClause::Keyboard(keys.player_keys.volume_plus_2.get()),
+            //     Self::no_popup_mounted_clause(),
+            // ),
+            Sub::new(
+                SubEventClause::Keyboard(keys.select_view_keys.open_help.get()),
                 Self::no_popup_mounted_clause(),
             ),
             Sub::new(
-                SubEventClause::Keyboard(keys.global_player_volume_minus_2.key_event()),
+                SubEventClause::Keyboard(keys.player_keys.seek_forward.get()),
                 Self::no_popup_mounted_clause(),
             ),
             Sub::new(
-                SubEventClause::Keyboard(keys.global_player_volume_plus_1.key_event()),
+                SubEventClause::Keyboard(keys.player_keys.seek_backward.get()),
                 Self::no_popup_mounted_clause(),
             ),
             Sub::new(
-                SubEventClause::Keyboard(keys.global_player_volume_plus_2.key_event()),
+                SubEventClause::Keyboard(keys.lyric_keys.adjust_offset_forwards.get()),
                 Self::no_popup_mounted_clause(),
             ),
             Sub::new(
-                SubEventClause::Keyboard(keys.global_help.key_event()),
+                SubEventClause::Keyboard(keys.lyric_keys.adjust_offset_backwards.get()),
                 Self::no_popup_mounted_clause(),
             ),
             Sub::new(
-                SubEventClause::Keyboard(keys.global_player_seek_forward.key_event()),
+                SubEventClause::Keyboard(keys.lyric_keys.cycle_frames.get()),
                 Self::no_popup_mounted_clause(),
             ),
             Sub::new(
-                SubEventClause::Keyboard(keys.global_player_seek_backward.key_event()),
+                SubEventClause::Keyboard(keys.select_view_keys.view_library.get()),
                 Self::no_popup_mounted_clause(),
             ),
             Sub::new(
-                SubEventClause::Keyboard(keys.global_lyric_adjust_forward.key_event()),
+                SubEventClause::Keyboard(keys.select_view_keys.view_database.get()),
                 Self::no_popup_mounted_clause(),
             ),
             Sub::new(
-                SubEventClause::Keyboard(keys.global_lyric_adjust_backward.key_event()),
+                SubEventClause::Keyboard(keys.player_keys.toggle_prefetch.get()),
                 Self::no_popup_mounted_clause(),
             ),
             Sub::new(
-                SubEventClause::Keyboard(keys.global_lyric_cycle.key_event()),
+                SubEventClause::Keyboard(keys.select_view_keys.open_config.get()),
                 Self::no_popup_mounted_clause(),
             ),
             Sub::new(
-                SubEventClause::Keyboard(keys.global_layout_treeview.key_event()),
+                SubEventClause::Keyboard(keys.player_keys.save_playlist.get()),
                 Self::no_popup_mounted_clause(),
             ),
             Sub::new(
-                SubEventClause::Keyboard(keys.global_layout_database.key_event()),
+                SubEventClause::Keyboard(keys.select_view_keys.view_podcasts.get()),
                 Self::no_popup_mounted_clause(),
             ),
             Sub::new(
-                SubEventClause::Keyboard(keys.global_player_toggle_gapless.key_event()),
-                Self::no_popup_mounted_clause(),
-            ),
-            Sub::new(
-                SubEventClause::Keyboard(keys.global_config_open.key_event()),
-                Self::no_popup_mounted_clause(),
-            ),
-            Sub::new(
-                SubEventClause::Keyboard(keys.global_save_playlist.key_event()),
-                Self::no_popup_mounted_clause(),
-            ),
-            Sub::new(
-                SubEventClause::Keyboard(keys.global_layout_podcast.key_event()),
-                Self::no_popup_mounted_clause(),
-            ),
-            Sub::new(
-                SubEventClause::Keyboard(keys.global_xywh_move_left.key_event()),
+                SubEventClause::Keyboard(keys.move_cover_art_keys.move_left.get()),
                 SubClause::Always,
             ),
             Sub::new(
-                SubEventClause::Keyboard(keys.global_xywh_move_right.key_event()),
+                SubEventClause::Keyboard(keys.move_cover_art_keys.move_right.get()),
                 SubClause::Always,
             ),
             Sub::new(
-                SubEventClause::Keyboard(keys.global_xywh_move_up.key_event()),
+                SubEventClause::Keyboard(keys.move_cover_art_keys.move_up.get()),
                 SubClause::Always,
             ),
             Sub::new(
-                SubEventClause::Keyboard(keys.global_xywh_move_down.key_event()),
+                SubEventClause::Keyboard(keys.move_cover_art_keys.move_down.get()),
                 SubClause::Always,
             ),
             Sub::new(
-                SubEventClause::Keyboard(keys.global_xywh_zoom_in.key_event()),
+                SubEventClause::Keyboard(keys.move_cover_art_keys.increase_size.get()),
                 SubClause::Always,
             ),
             Sub::new(
-                SubEventClause::Keyboard(keys.global_xywh_zoom_out.key_event()),
+                SubEventClause::Keyboard(keys.move_cover_art_keys.decrease_size.get()),
                 SubClause::Always,
             ),
             Sub::new(
-                SubEventClause::Keyboard(keys.global_xywh_hide.key_event()),
+                SubEventClause::Keyboard(keys.move_cover_art_keys.toggle_hide.get()),
                 SubClause::Always,
             ),
             Sub::new(SubEventClause::WindowResize, SubClause::Always),
