@@ -3,23 +3,25 @@ use crate::utils::get_pin_yin;
 use anyhow::{bail, Context, Result};
 use std::fs::{remove_dir_all, remove_file, rename};
 use std::path::{Path, PathBuf};
-use termusicplayback::{PlayerCmd, SharedSettings};
+use termusiclib::config::v2::server::config_extra::ServerConfigVersionedDefaulted;
+use termusiclib::config::v2::server::ScanDepth;
+use termusiclib::config::SharedTuiSettings;
+use termusicplayback::PlayerCmd;
 use tui_realm_treeview::{Node, Tree, TreeView, TREE_CMD_CLOSE, TREE_CMD_OPEN, TREE_INITIAL_NODE};
 use tuirealm::command::{Cmd, CmdResult, Direction, Position};
 use tuirealm::event::{Key, KeyEvent, KeyModifiers, NoUserEvent};
 use tuirealm::props::{Alignment, BorderType, Borders, TableBuilder, TextSpan};
-use tuirealm::tui::style::Color;
 use tuirealm::{AttrValue, Attribute, Component, Event, MockComponent, State, StateValue};
 
 #[derive(MockComponent)]
 pub struct MusicLibrary {
     component: TreeView,
-    config: SharedSettings,
+    config: SharedTuiSettings,
     pub init: bool,
 }
 
 impl MusicLibrary {
-    pub fn new(tree: &Tree, initial_node: Option<String>, config: SharedSettings) -> Self {
+    pub fn new(tree: &Tree, initial_node: Option<String>, config: SharedTuiSettings) -> Self {
         // Preserve initial node if exists
         let initial_node = match initial_node {
             Some(id) if tree.root().query(&id).is_some() => id,
@@ -28,39 +30,19 @@ impl MusicLibrary {
         let component = {
             let config = config.read();
             TreeView::default()
-                .background(
-                    config
-                        .style_color_symbol
-                        .library_background()
-                        .unwrap_or(Color::Reset),
-                )
-                .foreground(
-                    config
-                        .style_color_symbol
-                        .library_foreground()
-                        .unwrap_or(Color::Magenta),
-                )
+                .background(config.settings.theme.library_background())
+                .foreground(config.settings.theme.library_foreground())
                 .borders(
                     Borders::default()
-                        .color(
-                            config
-                                .style_color_symbol
-                                .library_border()
-                                .unwrap_or(Color::Magenta),
-                        )
+                        .color(config.settings.theme.library_border())
                         .modifiers(BorderType::Rounded),
                 )
                 // .inactive(Style::default().fg(Color::Gray))
                 .indent_size(2)
                 .scroll_step(6)
                 .title(" Library ", Alignment::Left)
-                .highlighted_color(
-                    config
-                        .style_color_symbol
-                        .library_highlight()
-                        .unwrap_or(Color::Yellow),
-                )
-                .highlight_symbol(&config.style_color_symbol.library_highlight_symbol)
+                .highlighted_color(config.settings.theme.library_highlight())
+                .highlight_symbol(&config.settings.theme.style.library.highlight_symbol)
                 .preserve_state(true)
                 // .highlight_symbol("🦄")
                 .with_tree(tree.clone())
@@ -74,6 +56,7 @@ impl MusicLibrary {
         }
     }
 
+    /// Also known as going up in the tree
     fn handle_left_key(&mut self) -> CmdResult {
         if let State::One(StateValue::String(node_id)) = self.state() {
             if let Some(node) = self.component.tree().root().query(&node_id) {
@@ -94,6 +77,22 @@ impl MusicLibrary {
         }
         CmdResult::None
     }
+
+    /// Also known as going down the tree / adding file to playlist
+    fn handle_right_key(&mut self) -> (CmdResult, Option<Msg>) {
+        let current_node = self.component.tree_state().selected().unwrap();
+        let p: &Path = Path::new(current_node);
+        if p.is_dir() {
+            (self.perform(Cmd::Custom(TREE_CMD_OPEN)), None)
+        } else {
+            (
+                CmdResult::None,
+                Some(Msg::Playlist(crate::ui::PLMsg::Add(
+                    current_node.to_string(),
+                ))),
+            )
+        }
+    }
 }
 
 impl Component<Msg, NoUserEvent> for MusicLibrary {
@@ -108,9 +107,9 @@ impl Component<Msg, NoUserEvent> for MusicLibrary {
             }
         }
         let config = self.config.clone();
-        let keys = &config.read().keys;
+        let keys = &config.read().settings.keys;
         let result = match ev {
-            Event::Keyboard(keyevent) if keyevent == keys.global_left.key_event() => {
+            Event::Keyboard(keyevent) if keyevent == keys.navigation_keys.left.get() => {
                 self.handle_left_key()
             }
             Event::Keyboard(KeyEvent {
@@ -120,32 +119,20 @@ impl Component<Msg, NoUserEvent> for MusicLibrary {
             Event::Keyboard(KeyEvent {
                 code: Key::Right,
                 modifiers: KeyModifiers::NONE,
-            }) => {
-                let current_node = self.component.tree_state().selected().unwrap();
-                let p: &Path = Path::new(current_node);
-                if p.is_dir() {
-                    self.perform(Cmd::Custom(TREE_CMD_OPEN))
-                } else {
-                    return Some(Msg::Playlist(crate::ui::PLMsg::Add(
-                        current_node.to_string(),
-                    )));
+            }) => match self.handle_right_key() {
+                (_, Some(msg)) => return Some(msg),
+                (cmdresult, None) => cmdresult,
+            },
+            Event::Keyboard(keyevent) if keyevent == keys.navigation_keys.right.get() => {
+                match self.handle_right_key() {
+                    (_, Some(msg)) => return Some(msg),
+                    (cmdresult, None) => cmdresult,
                 }
             }
-            Event::Keyboard(keyevent) if keyevent == keys.global_right.key_event() => {
-                let current_node = self.component.tree_state().selected().unwrap();
-                let p: &Path = Path::new(current_node);
-                if p.is_dir() {
-                    self.perform(Cmd::Custom(TREE_CMD_OPEN))
-                } else {
-                    return Some(Msg::Playlist(crate::ui::PLMsg::Add(
-                        current_node.to_string(),
-                    )));
-                }
-            }
-            Event::Keyboard(keyevent) if keyevent == keys.global_down.key_event() => {
+            Event::Keyboard(keyevent) if keyevent == keys.navigation_keys.down.get() => {
                 self.perform(Cmd::Move(Direction::Down))
             }
-            Event::Keyboard(keyevent) if keyevent == keys.global_up.key_event() => {
+            Event::Keyboard(keyevent) if keyevent == keys.navigation_keys.up.get() => {
                 self.perform(Cmd::Move(Direction::Up))
             }
             Event::Keyboard(KeyEvent {
@@ -157,7 +144,7 @@ impl Component<Msg, NoUserEvent> for MusicLibrary {
                 modifiers: KeyModifiers::NONE,
             }) => self.perform(Cmd::Move(Direction::Up)),
 
-            Event::Keyboard(keyevent) if keyevent == keys.library_load_dir.key_event() => {
+            Event::Keyboard(keyevent) if keyevent == keys.library_keys.load_dir.get() => {
                 let current_node = self.component.tree_state().selected().unwrap();
                 let p: &Path = Path::new(current_node);
                 if p.is_dir() {
@@ -175,10 +162,10 @@ impl Component<Msg, NoUserEvent> for MusicLibrary {
                 code: Key::PageUp,
                 modifiers: KeyModifiers::NONE,
             }) => self.perform(Cmd::Scroll(Direction::Up)),
-            Event::Keyboard(keyevent) if keyevent == keys.global_goto_top.key_event() => {
+            Event::Keyboard(keyevent) if keyevent == keys.navigation_keys.goto_top.get() => {
                 self.perform(Cmd::GoTo(Position::Begin))
             }
-            Event::Keyboard(keyevent) if keyevent == keys.global_goto_bottom.key_event() => {
+            Event::Keyboard(keyevent) if keyevent == keys.navigation_keys.goto_bottom.get() => {
                 self.perform(Cmd::GoTo(Position::End))
             }
             Event::Keyboard(KeyEvent {
@@ -188,7 +175,7 @@ impl Component<Msg, NoUserEvent> for MusicLibrary {
             Event::Keyboard(KeyEvent {
                 code: Key::Backspace,
                 modifiers: KeyModifiers::NONE,
-            }) => return Some(Msg::Library(LIMsg::TreeGoToUpperDir)),
+            }) => return Some(Msg::Library(LIMsg::TreeStepOut)),
             Event::Keyboard(
                 KeyEvent {
                     code: Key::Tab,
@@ -199,33 +186,33 @@ impl Component<Msg, NoUserEvent> for MusicLibrary {
                     modifiers: KeyModifiers::SHIFT,
                 },
             ) => return Some(Msg::Library(LIMsg::TreeBlur)),
-            Event::Keyboard(keyevent) if keyevent == keys.library_delete.key_event() => {
+            Event::Keyboard(keyevent) if keyevent == keys.library_keys.delete.get() => {
                 return Some(Msg::DeleteConfirmShow)
             }
-            Event::Keyboard(keyevent) if keyevent == keys.library_yank.key_event() => {
+            Event::Keyboard(keyevent) if keyevent == keys.library_keys.yank.get() => {
                 return Some(Msg::Library(LIMsg::Yank))
             }
-            Event::Keyboard(keyevent) if keyevent == keys.library_paste.key_event() => {
+            Event::Keyboard(keyevent) if keyevent == keys.library_keys.paste.get() => {
                 return Some(Msg::Library(LIMsg::Paste))
             }
-            Event::Keyboard(keyevent) if keyevent == keys.library_switch_root.key_event() => {
+            Event::Keyboard(keyevent) if keyevent == keys.library_keys.cycle_root.get() => {
                 return Some(Msg::Library(LIMsg::SwitchRoot))
             }
 
-            Event::Keyboard(keyevent) if keyevent == keys.library_add_root.key_event() => {
+            Event::Keyboard(keyevent) if keyevent == keys.library_keys.add_root.get() => {
                 return Some(Msg::Library(LIMsg::AddRoot))
             }
-            Event::Keyboard(keyevent) if keyevent == keys.library_remove_root.key_event() => {
+            Event::Keyboard(keyevent) if keyevent == keys.library_keys.remove_root.get() => {
                 return Some(Msg::Library(LIMsg::RemoveRoot))
             }
-            Event::Keyboard(keyevent) if keyevent == keys.library_search.key_event() => {
+            Event::Keyboard(keyevent) if keyevent == keys.library_keys.search.get() => {
                 return Some(Msg::GeneralSearch(crate::ui::GSMsg::PopupShowLibrary))
             }
 
-            Event::Keyboard(keyevent) if keyevent == keys.library_search_youtube.key_event() => {
+            Event::Keyboard(keyevent) if keyevent == keys.library_keys.youtube_search.get() => {
                 return Some(Msg::YoutubeSearch(YSMsg::InputPopupShow))
             }
-            Event::Keyboard(keyevent) if keyevent == keys.library_tag_editor_open.key_event() => {
+            Event::Keyboard(keyevent) if keyevent == keys.library_keys.open_tag_editor.get() => {
                 let current_node = self.component.tree_state().selected().unwrap();
                 return Some(Msg::TagEditor(TEMsg::TagEditorRun(
                     current_node.to_string(),
@@ -236,7 +223,7 @@ impl Component<Msg, NoUserEvent> for MusicLibrary {
         };
         match result {
             CmdResult::Submit(State::One(StateValue::String(node))) => {
-                Some(Msg::Library(LIMsg::TreeExtendDir(node)))
+                Some(Msg::Library(LIMsg::TreeStepInto(node)))
             }
             _ => Some(Msg::None),
         }
@@ -244,31 +231,47 @@ impl Component<Msg, NoUserEvent> for MusicLibrary {
 }
 
 impl Model {
-    pub fn library_scan_dir(&mut self, p: &Path) {
-        self.path = p.to_path_buf();
-        self.tree = Tree::new(Self::library_dir_tree(p, self.config.read().max_depth_cli));
+    pub fn library_scan_dir<P: Into<PathBuf>>(&mut self, p: P) {
+        self.library.tree_path = p.into();
+        self.library.tree = Tree::new(Self::library_dir_tree(
+            &self.library.tree_path,
+            self.config_server.read().get_library_scan_depth(),
+        ));
     }
 
     pub fn library_upper_dir(&self) -> Option<PathBuf> {
-        self.path.parent().map(std::path::Path::to_path_buf)
+        self.library
+            .tree_path
+            .parent()
+            .map(std::path::Path::to_path_buf)
     }
 
-    pub fn library_dir_tree(p: &Path, depth: usize) -> Node {
+    pub fn library_dir_tree(p: &Path, depth: ScanDepth) -> Node {
         let name: String = match p.file_name() {
             None => "/".to_string(),
             Some(n) => n.to_string_lossy().into_owned(),
         };
         let mut node: Node = Node::new(p.to_string_lossy().into_owned(), name);
+
+        let depth = match depth {
+            ScanDepth::Limited(v) => v,
+            // put some kind of limit on it, thought the stack will likely overflow before this
+            ScanDepth::Unlimited => u32::MAX,
+        };
+
         if depth > 0 && p.is_dir() {
             if let Ok(paths) = std::fs::read_dir(p) {
                 let mut paths: Vec<_> = paths
                     .filter_map(std::result::Result::ok)
-                    .filter(|p| !p.file_name().to_string_lossy().to_string().starts_with('.'))
+                    .filter(|p| !p.file_name().to_string_lossy().starts_with('.'))
                     .collect();
 
                 paths.sort_by_cached_key(|k| get_pin_yin(&k.file_name().to_string_lossy()));
                 for p in paths {
-                    node.add_child(Self::library_dir_tree(p.path().as_path(), depth - 1));
+                    node.add_child(Self::library_dir_tree(
+                        p.path().as_path(),
+                        ScanDepth::Limited(depth - 1),
+                    ));
                 }
             }
         }
@@ -292,7 +295,7 @@ impl Model {
     }
 
     pub fn library_reload_with_node_focus(&mut self, node: Option<&str>) {
-        self.db.sync_database(self.path.as_path());
+        self.db.sync_database(self.library.tree_path.as_path());
         self.database_reload();
         self.library_reload_tree();
         if let Some(n) = node {
@@ -308,9 +311,9 @@ impl Model {
     }
 
     pub fn library_reload_tree(&mut self) {
-        self.tree = Tree::new(Self::library_dir_tree(
-            self.path.as_ref(),
-            self.config.read().max_depth_cli,
+        self.library.tree = Tree::new(Self::library_dir_tree(
+            self.library.tree_path.as_ref(),
+            self.config_server.read().get_library_scan_depth(),
         ));
         let current_node = match self.app.state(&Id::Library).ok().unwrap() {
             State::One(StateValue::String(id)) => Some(id),
@@ -331,9 +334,9 @@ impl Model {
                 .mount(
                     Id::Library,
                     Box::new(MusicLibrary::new(
-                        &self.tree.clone(),
+                        &self.library.tree.clone(),
                         current_node,
-                        self.config.clone(),
+                        self.config_tui.clone(),
                     ),),
                     Vec::new()
                 )
@@ -347,9 +350,9 @@ impl Model {
             .remount(
                 Id::Library,
                 Box::new(MusicLibrary::new(
-                    &self.tree.clone(),
+                    &self.library.tree.clone(),
                     current_node,
-                    self.config.clone(),
+                    self.config_tui.clone(),
                 ),),
                 Vec::new()
             )
@@ -365,13 +368,13 @@ impl Model {
     //     }
     // }
     pub fn library_stepinto(&mut self, node_id: &str) {
-        self.library_scan_dir(PathBuf::from(node_id).as_path());
+        self.library_scan_dir(PathBuf::from(node_id));
         self.library_reload_tree();
     }
 
     pub fn library_stepout(&mut self) {
         if let Some(p) = self.library_upper_dir() {
-            self.library_scan_dir(p.as_path());
+            self.library_scan_dir(p);
             self.library_reload_tree();
         }
     }
@@ -387,9 +390,9 @@ impl Model {
         }
     }
 
-    pub fn library_delete_song(&mut self) -> Result<()> {
+    pub fn library_delete_node(&mut self) -> Result<()> {
         if let Ok(State::One(StateValue::String(node_id))) = self.app.state(&Id::Library) {
-            if let Some(mut route) = self.tree.root().route_by_node(&node_id) {
+            if let Some(mut route) = self.library.tree.root().route_by_node(&node_id) {
                 let p: &Path = Path::new(node_id.as_str());
                 if p.is_file() {
                     remove_file(p)?;
@@ -398,9 +401,9 @@ impl Model {
                     remove_dir_all(p)?;
                 }
 
-                // // this is to keep the state of playlist
+                // this is to keep the state of playlist
                 self.library_reload_tree();
-                let tree = self.tree.clone();
+                let tree = self.library.tree.clone();
                 if let Some(new_node) = tree.root().node_by_route(&route) {
                     self.library_reload_with_node_focus(Some(new_node.id()));
                 } else {
@@ -429,13 +432,17 @@ impl Model {
 
     pub fn library_yank(&mut self) {
         if let Ok(State::One(StateValue::String(node_id))) = self.app.state(&Id::Library) {
-            self.yanked_node_id = Some(node_id);
+            self.library.yanked_node_id = Some(node_id);
         }
     }
 
     pub fn library_paste(&mut self) -> Result<()> {
         if let Ok(State::One(StateValue::String(new_id))) = self.app.state(&Id::Library) {
-            let old_id = self.yanked_node_id.as_ref().context("no id yanked")?;
+            let old_id = self
+                .library
+                .yanked_node_id
+                .as_ref()
+                .context("no id yanked")?;
             let p: &Path = Path::new(new_id.as_str());
             let pold: &Path = Path::new(old_id.as_str());
             let p_parent = p.parent().context("no parent folder found")?;
@@ -448,23 +455,22 @@ impl Model {
             rename(pold, new_node_id.as_path())?;
             self.library_reload_with_node_focus(new_node_id.to_str());
         }
-        self.yanked_node_id = None;
+        self.library.yanked_node_id = None;
         self.playlist_update_library_delete();
         Ok(())
     }
 
     pub fn library_update_search(&mut self, input: &str) {
         let mut table: TableBuilder = TableBuilder::default();
-        let root = self.tree.root();
+        let root = self.library.tree.root();
         let p: &Path = Path::new(root.id());
         let all_items = walkdir::WalkDir::new(p).follow_links(true);
         let mut idx = 0;
         let search = format!("*{}*", input.to_lowercase());
+        let search = wildmatch::WildMatch::new(&search);
         for record in all_items.into_iter().filter_map(std::result::Result::ok) {
             let file_name = record.path();
-            if wildmatch::WildMatch::new(&search)
-                .matches(&file_name.to_string_lossy().to_lowercase())
-            {
+            if search.matches(&file_name.to_string_lossy().to_lowercase()) {
                 if idx > 0 {
                     table.add_row();
                 }
@@ -481,24 +487,24 @@ impl Model {
 
     pub fn library_switch_root(&mut self) {
         let mut vec = Vec::new();
-        let config = self.config.read();
-        for dir in &config.music_dir {
-            let absolute_dir = shellexpand::tilde(dir).to_string();
+        let config_server = self.config_server.read();
+        for dir in &config_server.settings.player.music_dirs {
+            let absolute_dir = shellexpand::path::tilde(dir).into_owned();
             vec.push(absolute_dir);
         }
-        if let Some(dir) = &config.music_dir_from_cli {
-            let absolute_dir = shellexpand::tilde(&dir).to_string();
+        if let Some(dir) = &config_server.music_dir_overwrite {
+            let absolute_dir = shellexpand::path::tilde(dir).into_owned();
             vec.push(absolute_dir);
         }
         if vec.is_empty() {
             return;
         }
-        drop(config);
+        drop(config_server);
 
         let mut index = 0;
-        let current_path_str = self.path.to_string_lossy().to_string();
+        let current_path = &self.library.tree_path;
         for (idx, dir) in vec.iter().enumerate() {
-            if current_path_str == *dir {
+            if current_path == dir {
                 index = idx + 1;
                 break;
             }
@@ -508,44 +514,43 @@ impl Model {
         }
         if let Some(dir) = vec.get(index) {
             let pathbuf = PathBuf::from(dir);
-            self.library_scan_dir(pathbuf.as_path());
+            self.library_scan_dir(pathbuf);
             self.library_reload_with_node_focus(None);
         }
     }
 
     pub fn library_add_root(&mut self) -> Result<()> {
-        let current_path_string = self.path.to_string_lossy().to_string();
+        let current_path = &self.library.tree_path;
 
-        let mut config = self.config.write();
+        let mut config_server = self.config_server.write();
 
-        for dir in &config.music_dir {
-            let absolute_dir = shellexpand::tilde(dir).to_string();
-            if absolute_dir == current_path_string {
+        for dir in &config_server.settings.player.music_dirs {
+            let absolute_dir = shellexpand::path::tilde(dir);
+            if &absolute_dir == current_path {
                 bail!("Add root failed, same root already exists");
             }
         }
-        config.music_dir.push(current_path_string);
-        let res = config.save();
-        drop(config);
-        match res {
-            Ok(()) => {
-                self.command(&PlayerCmd::ReloadConfig);
-            }
-            Err(e) => {
-                bail!("error when saving config: {e}");
-            }
-        }
+        config_server
+            .settings
+            .player
+            .music_dirs
+            .push(current_path.clone());
+        let res = ServerConfigVersionedDefaulted::save_config_path(&config_server.settings);
+        drop(config_server);
+
+        res.context("Error while saving config")?;
+        self.command(&PlayerCmd::ReloadConfig);
         Ok(())
     }
 
     pub fn library_remove_root(&mut self) -> Result<()> {
-        let current_path_string = self.path.to_string_lossy().to_string();
-        let mut config = self.config.write();
+        let current_path = &self.library.tree_path;
+        let mut config_server = self.config_server.write();
 
         let mut vec = Vec::new();
-        for dir in &config.music_dir {
-            let absolute_dir = shellexpand::tilde(dir).to_string();
-            if absolute_dir == current_path_string {
+        for dir in &config_server.settings.player.music_dirs {
+            let absolute_dir = shellexpand::path::tilde(dir);
+            if &absolute_dir == current_path {
                 continue;
             }
             vec.push(dir.clone());
@@ -554,9 +559,14 @@ impl Model {
             bail!("At least 1 root music directory should be kept");
         }
 
-        config.music_dir = vec;
-        drop(config);
+        config_server.settings.player.music_dirs = vec;
+        let res = ServerConfigVersionedDefaulted::save_config_path(&config_server.settings);
+        drop(config_server);
+
         self.library_switch_root();
+
+        res.context("Error while saving config")?;
+
         Ok(())
     }
 }
