@@ -4,17 +4,17 @@ use anyhow::{anyhow, bail, Result};
 use rand::seq::SliceRandom;
 use std::borrow::Cow;
 use std::path::Path;
-use termusiclib::sqlite::SearchCriteria;
-use termusiclib::sqlite::TrackForDB;
+use termusiclib::config::SharedTuiSettings;
+use termusiclib::library_db::SearchCriteria;
+use termusiclib::library_db::TrackDB;
 use termusiclib::track::Track;
 use termusiclib::types::{GSMsg, Id, Msg, PLMsg};
 use termusiclib::utils::{filetype_supported, get_parent_folder, is_playlist, playlist_get_vec};
 use termusicplayback::PlayerCmd;
-use termusicplayback::SharedSettings;
 
 use tui_realm_stdlib::Table;
+use tuirealm::props::Borders;
 use tuirealm::props::{Alignment, BorderType, PropPayload, PropValue, TableBuilder, TextSpan};
-use tuirealm::props::{Borders, Color};
 use tuirealm::{
     command::{Cmd, CmdResult, Direction, Position},
     event::KeyModifiers,
@@ -27,43 +27,25 @@ use tuirealm::{
 #[derive(MockComponent)]
 pub struct Playlist {
     component: Table,
-    config: SharedSettings,
+    config: SharedTuiSettings,
 }
 
 impl Playlist {
-    pub fn new(config: SharedSettings) -> Self {
+    pub fn new(config: SharedTuiSettings) -> Self {
         let component = {
             let config = config.read();
             Table::default()
                 .borders(
-                    Borders::default().modifiers(BorderType::Rounded).color(
-                        config
-                            .style_color_symbol
-                            .playlist_border()
-                            .unwrap_or(Color::Blue),
-                    ),
-                )
-                .background(
-                    config
-                        .style_color_symbol
-                        .playlist_background()
-                        .unwrap_or(Color::Reset),
-                )
-                .foreground(
-                    config
-                        .style_color_symbol
-                        .playlist_foreground()
-                        .unwrap_or(Color::Yellow),
+                    Borders::default()
+                        .modifiers(BorderType::Rounded)
+                        .color(config.settings.theme.playlist_border()),
                 )
+                .background(config.settings.theme.playlist_background())
+                .foreground(config.settings.theme.playlist_foreground())
                 .title(" Playlist ", Alignment::Left)
                 .scroll(true)
-                .highlighted_color(
-                    config
-                        .style_color_symbol
-                        .playlist_highlight()
-                        .unwrap_or(Color::LightBlue),
-                )
-                .highlighted_str(&config.style_color_symbol.playlist_highlight_symbol)
+                .highlighted_color(config.settings.theme.playlist_highlight())
+                .highlighted_str(&config.settings.theme.style.playlist.highlight_symbol)
                 .rewind(false)
                 .step(4)
                 .row_height(1)
@@ -87,7 +69,7 @@ impl Component<Msg, NoUserEvent> for Playlist {
     #[allow(clippy::too_many_lines)]
     fn on(&mut self, ev: Event<NoUserEvent>) -> Option<Msg> {
         let config = self.config.clone();
-        let keys = &config.read().keys;
+        let keys = &config.read().settings.keys;
         let _cmd_result = match ev {
             Event::Keyboard(KeyEvent {
                 code: Key::Down,
@@ -97,10 +79,10 @@ impl Component<Msg, NoUserEvent> for Playlist {
                 code: Key::Up,
                 modifiers: KeyModifiers::NONE,
             }) => self.perform(Cmd::Move(Direction::Up)),
-            Event::Keyboard(key) if key == keys.global_down.key_event() => {
+            Event::Keyboard(key) if key == keys.navigation_keys.down.get() => {
                 self.perform(Cmd::Move(Direction::Down))
             }
-            Event::Keyboard(key) if key == keys.global_up.key_event() => {
+            Event::Keyboard(key) if key == keys.navigation_keys.up.get() => {
                 self.perform(Cmd::Move(Direction::Up))
             }
             Event::Keyboard(KeyEvent {
@@ -111,10 +93,10 @@ impl Component<Msg, NoUserEvent> for Playlist {
                 code: Key::PageUp,
                 modifiers: KeyModifiers::NONE,
             }) => self.perform(Cmd::Scroll(Direction::Up)),
-            Event::Keyboard(key) if key == keys.global_goto_top.key_event() => {
+            Event::Keyboard(key) if key == keys.navigation_keys.goto_top.get() => {
                 self.perform(Cmd::GoTo(Position::Begin))
             }
-            Event::Keyboard(key) if key == keys.global_goto_bottom.key_event() => {
+            Event::Keyboard(key) if key == keys.navigation_keys.goto_bottom.get() => {
                 self.perform(Cmd::GoTo(Position::End))
             }
             Event::Keyboard(KeyEvent {
@@ -133,7 +115,7 @@ impl Component<Msg, NoUserEvent> for Playlist {
                 code: Key::BackTab,
                 modifiers: KeyModifiers::SHIFT,
             }) => return Some(Msg::Playlist(PLMsg::PlaylistTableBlurUp)),
-            Event::Keyboard(key) if key == keys.playlist_delete.key_event() => {
+            Event::Keyboard(key) if key == keys.playlist_keys.delete.get() => {
                 match self.component.state() {
                     State::One(StateValue::Usize(index_selected)) => {
                         return Some(Msg::Playlist(PLMsg::Delete(index_selected)))
@@ -141,16 +123,16 @@ impl Component<Msg, NoUserEvent> for Playlist {
                     _ => return Some(Msg::None),
                 }
             }
-            Event::Keyboard(key) if key == keys.playlist_delete_all.key_event() => {
+            Event::Keyboard(key) if key == keys.playlist_keys.delete_all.get() => {
                 return Some(Msg::Playlist(PLMsg::DeleteAll))
             }
-            Event::Keyboard(key) if key == keys.playlist_shuffle.key_event() => {
+            Event::Keyboard(key) if key == keys.playlist_keys.shuffle.get() => {
                 return Some(Msg::Playlist(PLMsg::Shuffle))
             }
-            Event::Keyboard(key) if key == keys.playlist_mode_cycle.key_event() => {
+            Event::Keyboard(key) if key == keys.playlist_keys.cycle_loop_mode.get() => {
                 return Some(Msg::Playlist(PLMsg::LoopModeCycle))
             }
-            Event::Keyboard(key) if key == keys.playlist_play_selected.key_event() => {
+            Event::Keyboard(key) if key == keys.playlist_keys.play_selected.get() => {
                 if let State::One(StateValue::Usize(index)) = self.state() {
                     return Some(Msg::Playlist(PLMsg::PlaySelected(index)));
                 }
@@ -166,10 +148,10 @@ impl Component<Msg, NoUserEvent> for Playlist {
                 }
                 CmdResult::None
             }
-            Event::Keyboard(key) if key == keys.playlist_search.key_event() => {
+            Event::Keyboard(key) if key == keys.playlist_keys.search.get() => {
                 return Some(Msg::GeneralSearch(GSMsg::PopupShowPlaylist))
             }
-            Event::Keyboard(key) if key == keys.playlist_swap_down.key_event() => {
+            Event::Keyboard(key) if key == keys.playlist_keys.swap_down.get() => {
                 match self.component.state() {
                     State::One(StateValue::Usize(index_selected)) => {
                         self.perform(Cmd::Move(Direction::Down));
@@ -178,7 +160,7 @@ impl Component<Msg, NoUserEvent> for Playlist {
                     _ => return Some(Msg::None),
                 }
             }
-            Event::Keyboard(key) if key == keys.playlist_swap_up.key_event() => {
+            Event::Keyboard(key) if key == keys.playlist_keys.swap_up.get() => {
                 match self.component.state() {
                     State::One(StateValue::Usize(index_selected)) => {
                         self.perform(Cmd::Move(Direction::Up));
@@ -187,11 +169,11 @@ impl Component<Msg, NoUserEvent> for Playlist {
                     _ => return Some(Msg::None),
                 }
             }
-            Event::Keyboard(key) if key == keys.playlist_cmus_lqueue.key_event() => {
-                return Some(Msg::Playlist(PLMsg::CmusLQueue));
+            Event::Keyboard(key) if key == keys.playlist_keys.add_random_album.get() => {
+                return Some(Msg::Playlist(PLMsg::AddRandomAlbum));
             }
-            Event::Keyboard(key) if key == keys.playlist_cmus_tqueue.key_event() => {
-                return Some(Msg::Playlist(PLMsg::CmusTQueue));
+            Event::Keyboard(key) if key == keys.playlist_keys.add_random_songs.get() => {
+                return Some(Msg::Playlist(PLMsg::AddRandomTracks));
             }
             _ => CmdResult::None,
         };
@@ -205,7 +187,7 @@ impl Model {
             .app
             .remount(
                 Id::Playlist,
-                Box::new(Playlist::new(self.config.clone())),
+                Box::new(Playlist::new(self.config_tui.clone())),
                 Vec::new()
             )
             .is_ok());
@@ -274,20 +256,20 @@ impl Model {
 
     fn playlist_add_playlist(&mut self, current_node: &str) -> Result<()> {
         let vec = playlist_get_vec(current_node)?;
-        let vec_str: Vec<&str> = vec.iter().map(std::convert::AsRef::as_ref).collect();
-        self.playlist.add_playlist(vec_str)?;
+        self.playlist.add_playlist(&vec)?;
         self.player_sync_playlist()?;
         self.playlist_sync();
         Ok(())
     }
 
     pub fn playlist_add_episode(&mut self, episode_index: usize) -> Result<()> {
-        if self.podcasts.is_empty() {
+        if self.podcast.podcasts.is_empty() {
             return Ok(());
         }
         let podcast_selected = self
+            .podcast
             .podcasts
-            .get(self.podcasts_index)
+            .get(self.podcast.podcasts_index)
             .ok_or_else(|| anyhow!("get podcast selected failed."))?;
         let episode_selected = podcast_selected
             .episodes
@@ -306,11 +288,7 @@ impl Model {
         }
         if p.is_dir() {
             let new_items_vec = Self::library_dir_children(p);
-            let new_items_str_vec: Vec<&str> = new_items_vec
-                .iter()
-                .map(std::convert::AsRef::as_ref)
-                .collect();
-            self.playlist.add_playlist(new_items_str_vec)?;
+            self.playlist.add_playlist(&new_items_vec)?;
             self.player_sync_playlist()?;
             self.playlist_sync();
             return Ok(());
@@ -325,16 +303,14 @@ impl Model {
             self.playlist_add_playlist(current_node)?;
             return Ok(());
         }
-        let vec = vec![current_node];
-        self.playlist.add_playlist(vec)?;
+        self.playlist.add_playlist(&[current_node])?;
         self.player_sync_playlist()?;
         Ok(())
     }
 
-    pub fn playlist_add_all_from_db(&mut self, vec: &[TrackForDB]) {
-        let vec2: Vec<String> = vec.iter().map(|f| f.file.clone()).collect();
-        let vec3: Vec<&str> = vec2.iter().map(std::convert::AsRef::as_ref).collect();
-        if let Err(e) = self.playlist.add_playlist(vec3) {
+    pub fn playlist_add_all_from_db(&mut self, vec: &[TrackDB]) {
+        let vec2: Vec<&str> = vec.iter().map(|f| f.file.as_str()).collect();
+        if let Err(e) = self.playlist.add_playlist(&vec2) {
             self.mount_error_popup(e.context("add all to playlist from database"));
         }
         if let Err(e) = self.player_sync_playlist() {
@@ -343,17 +319,27 @@ impl Model {
         self.playlist_sync();
     }
 
-    pub fn playlist_add_cmus_lqueue(&mut self) {
-        let playlist_select_random_album_quantity =
-            self.config.read().playlist_select_random_album_quantity;
-        let vec = self.playlist_get_records_for_cmus_lqueue(playlist_select_random_album_quantity);
+    pub fn playlist_add_random_album(&mut self) {
+        let playlist_select_random_album_quantity = self
+            .config_server
+            .read()
+            .settings
+            .player
+            .random_album_min_quantity
+            .get();
+        let vec = self.playlist_get_random_album_tracks(playlist_select_random_album_quantity);
         self.playlist_add_all_from_db(&vec);
     }
 
-    pub fn playlist_add_cmus_tqueue(&mut self) {
-        let playlist_select_random_album_quantity =
-            self.config.read().playlist_select_random_album_quantity;
-        let vec = self.playlist_get_records_for_cmus_tqueue(playlist_select_random_album_quantity);
+    pub fn playlist_add_random_tracks(&mut self) {
+        let playlist_select_random_track_quantity = self
+            .config_server
+            .read()
+            .settings
+            .player
+            .random_track_quantity
+            .get();
+        let vec = self.playlist_get_random_tracks(playlist_select_random_track_quantity);
         self.playlist_add_all_from_db(&vec);
     }
 
@@ -375,7 +361,13 @@ impl Model {
             if idx == self.playlist.get_current_track_index() {
                 title = format!(
                     "{}{title}",
-                    self.ce_style_color_symbol.currently_playing_track_symbol
+                    self.config_tui
+                        .read()
+                        .settings
+                        .theme
+                        .style
+                        .playlist
+                        .current_track_symbol
                 );
             };
             table
@@ -425,7 +417,13 @@ impl Model {
             if idx == self.playlist.get_current_track_index() {
                 title = format!(
                     "{}{title}",
-                    self.ce_style_color_symbol.currently_playing_track_symbol
+                    self.config_tui
+                        .read()
+                        .settings
+                        .theme
+                        .style
+                        .playlist
+                        .current_track_symbol
                 )
                 .into();
             };
@@ -492,12 +490,20 @@ impl Model {
 
     pub fn playlist_update_title(&mut self) {
         let duration = self.playlist.tracks().iter().map(Track::duration).sum();
-        let config = self.config.read();
+        let display_symbol = self
+            .config_tui
+            .read()
+            .settings
+            .theme
+            .style
+            .playlist
+            .use_loop_mode_symbol;
+        let loop_mode = self.config_server.read().settings.player.loop_mode;
         let title = format!(
             "\u{2500} Playlist \u{2500}\u{2500}\u{2524} Total {} tracks | {} | Mode: {} \u{251c}\u{2500}",
             self.playlist.len(),
             Track::duration_formatted_short(&duration),
-            config.player_loop_mode.display(config.playlist_display_symbol),
+            loop_mode.display(display_symbol),
         );
         self.app
             .attr(
@@ -568,7 +574,7 @@ impl Model {
             .is_ok());
     }
 
-    pub fn playlist_get_records_for_cmus_tqueue(&mut self, quantity: u32) -> Vec<TrackForDB> {
+    pub fn playlist_get_random_tracks(&mut self, quantity: u32) -> Vec<TrackDB> {
         let mut result = vec![];
         if let Ok(vec) = self.db.get_all_records() {
             let mut i = 0;
@@ -590,7 +596,7 @@ impl Model {
         result
     }
 
-    pub fn playlist_get_records_for_cmus_lqueue(&mut self, quantity: u32) -> Vec<TrackForDB> {
+    pub fn playlist_get_random_album_tracks(&mut self, quantity: u32) -> Vec<TrackDB> {
         let mut result = vec![];
         if let Ok(vec) = self.db.get_all_records() {
             loop {
