@@ -1,17 +1,19 @@
 use crate::ui::Model;
 use anyhow::{anyhow, bail, Result};
 use percent_encoding::{utf8_percent_encode, NON_ALPHANUMERIC};
+use reqwest::ClientBuilder;
 use sanitize_filename::{sanitize_with_options, Options};
 use serde_json::Value;
 use std::time::Duration;
+use termusiclib::config::SharedTuiSettings;
 use termusiclib::podcast::{download_list, EpData, PodcastFeed, PodcastNoId};
 use termusiclib::track::MediaType;
 use termusiclib::types::{Id, Msg, PCMsg};
-use termusicplayback::SharedSettings;
+use tokio::runtime::Handle;
 use tui_realm_stdlib::List;
 use tuirealm::command::{Cmd, CmdResult, Direction, Position};
 use tuirealm::props::{Alignment, BorderType, TableBuilder, TextSpan};
-use tuirealm::props::{Borders, Color, PropPayload, PropValue};
+use tuirealm::props::{Borders, PropPayload, PropValue};
 use tuirealm::{
     event::{Key, KeyEvent, KeyModifiers, NoUserEvent},
     AttrValue, Attribute, Component, Event, MockComponent, State, StateValue,
@@ -24,43 +26,25 @@ pub struct FeedsList {
     component: List,
     on_key_tab: Msg,
     on_key_backtab: Msg,
-    config: SharedSettings,
+    config: SharedTuiSettings,
 }
 
 impl FeedsList {
-    pub fn new(config: SharedSettings, on_key_tab: Msg, on_key_backtab: Msg) -> Self {
+    pub fn new(config: SharedTuiSettings, on_key_tab: Msg, on_key_backtab: Msg) -> Self {
         let component = {
             let config = config.read();
             List::default()
                 .borders(
-                    Borders::default().modifiers(BorderType::Rounded).color(
-                        config
-                            .style_color_symbol
-                            .library_border()
-                            .unwrap_or(Color::Blue),
-                    ),
-                )
-                .background(
-                    config
-                        .style_color_symbol
-                        .library_background()
-                        .unwrap_or(Color::Reset),
-                )
-                .foreground(
-                    config
-                        .style_color_symbol
-                        .library_foreground()
-                        .unwrap_or(Color::Yellow),
+                    Borders::default()
+                        .modifiers(BorderType::Rounded)
+                        .color(config.settings.theme.library_border()),
                 )
+                .background(config.settings.theme.library_background())
+                .foreground(config.settings.theme.library_foreground())
                 .title(" Podcast Feeds: ", Alignment::Left)
                 .scroll(true)
-                .highlighted_color(
-                    config
-                        .style_color_symbol
-                        .library_highlight()
-                        .unwrap_or(Color::LightBlue),
-                )
-                .highlighted_str(&config.style_color_symbol.library_highlight_symbol)
+                .highlighted_color(config.settings.theme.library_highlight())
+                .highlighted_str(&config.settings.theme.style.library.highlight_symbol)
                 .rewind(false)
                 .step(4)
                 .scroll(true)
@@ -84,7 +68,7 @@ impl Component<Msg, NoUserEvent> for FeedsList {
     #[allow(clippy::too_many_lines)]
     fn on(&mut self, ev: Event<NoUserEvent>) -> Option<Msg> {
         let config = self.config.clone();
-        let keys = &config.read().keys;
+        let keys = &config.read().settings.keys;
         let _cmd_result = match ev {
             Event::Keyboard(KeyEvent {
                 code: Key::Down,
@@ -103,7 +87,7 @@ impl Component<Msg, NoUserEvent> for FeedsList {
                 code: Key::Up,
                 modifiers: KeyModifiers::NONE,
             }) => self.perform(Cmd::Move(Direction::Up)),
-            Event::Keyboard(key) if key == keys.global_down.key_event() => {
+            Event::Keyboard(key) if key == keys.navigation_keys.down.get() => {
                 if let Some(AttrValue::Table(t)) = self.query(Attribute::Content) {
                     if let State::One(StateValue::Usize(index)) = self.state() {
                         if index >= t.len() - 1 {
@@ -113,7 +97,7 @@ impl Component<Msg, NoUserEvent> for FeedsList {
                 }
                 self.perform(Cmd::Move(Direction::Down))
             }
-            Event::Keyboard(key) if key == keys.global_up.key_event() => {
+            Event::Keyboard(key) if key == keys.navigation_keys.up.get() => {
                 self.perform(Cmd::Move(Direction::Up))
             }
             Event::Keyboard(KeyEvent {
@@ -124,10 +108,10 @@ impl Component<Msg, NoUserEvent> for FeedsList {
                 code: Key::PageUp,
                 modifiers: KeyModifiers::NONE,
             }) => self.perform(Cmd::Scroll(Direction::Up)),
-            Event::Keyboard(key) if key == keys.global_goto_top.key_event() => {
+            Event::Keyboard(key) if key == keys.navigation_keys.goto_top.get() => {
                 self.perform(Cmd::GoTo(Position::Begin))
             }
-            Event::Keyboard(key) if key == keys.global_goto_bottom.key_event() => {
+            Event::Keyboard(key) if key == keys.navigation_keys.goto_bottom.get() => {
                 self.perform(Cmd::GoTo(Position::End))
             }
             Event::Keyboard(KeyEvent {
@@ -145,7 +129,7 @@ impl Component<Msg, NoUserEvent> for FeedsList {
                 CmdResult::None
             }
 
-            Event::Keyboard(key) if key == keys.global_right.key_event() => {
+            Event::Keyboard(key) if key == keys.navigation_keys.right.get() => {
                 if let State::One(StateValue::Usize(index)) = self.state() {
                     return Some(Msg::Podcast(PCMsg::PodcastSelected(index)));
                 }
@@ -166,29 +150,29 @@ impl Component<Msg, NoUserEvent> for FeedsList {
                 modifiers: KeyModifiers::SHIFT,
             }) => return Some(self.on_key_backtab.clone()),
 
-            Event::Keyboard(keyevent) if keyevent == keys.podcast_search_add_feed.key_event() => {
+            Event::Keyboard(keyevent) if keyevent == keys.podcast_keys.search.get() => {
                 return Some(Msg::Podcast(PCMsg::PodcastAddPopupShow));
             }
 
-            Event::Keyboard(keyevent) if keyevent == keys.podcast_refresh_feed.key_event() => {
+            Event::Keyboard(keyevent) if keyevent == keys.podcast_keys.refresh_feed.get() => {
                 if let State::One(StateValue::Usize(index)) = self.state() {
                     return Some(Msg::Podcast(PCMsg::PodcastRefreshOne(index)));
                 }
                 CmdResult::None
             }
 
-            Event::Keyboard(keyevent) if keyevent == keys.podcast_refresh_all_feeds.key_event() => {
+            Event::Keyboard(keyevent) if keyevent == keys.podcast_keys.refresh_all_feeds.get() => {
                 return Some(Msg::Podcast(PCMsg::PodcastRefreshAll));
             }
 
-            Event::Keyboard(keyevent) if keyevent == keys.podcast_delete_feed.key_event() => {
+            Event::Keyboard(keyevent) if keyevent == keys.podcast_keys.delete_feed.get() => {
                 return Some(Msg::Podcast(PCMsg::FeedDeleteShow));
             }
-            Event::Keyboard(keyevent) if keyevent == keys.podcast_delete_all_feeds.key_event() => {
+            Event::Keyboard(keyevent) if keyevent == keys.podcast_keys.delete_all_feeds.get() => {
                 return Some(Msg::Podcast(PCMsg::FeedsDeleteShow));
             }
 
-            Event::Keyboard(keyevent) if keyevent == keys.library_search.key_event() => {
+            Event::Keyboard(keyevent) if keyevent == keys.library_keys.search.get() => {
                 return Some(Msg::GeneralSearch(crate::ui::GSMsg::PopupShowPodcast))
             }
             _ => CmdResult::None,
@@ -202,43 +186,25 @@ pub struct EpisodeList {
     component: List,
     on_key_tab: Msg,
     on_key_backtab: Msg,
-    config: SharedSettings,
+    config: SharedTuiSettings,
 }
 
 impl EpisodeList {
-    pub fn new(config: SharedSettings, on_key_tab: Msg, on_key_backtab: Msg) -> Self {
+    pub fn new(config: SharedTuiSettings, on_key_tab: Msg, on_key_backtab: Msg) -> Self {
         let component = {
             let config = config.read();
             List::default()
                 .borders(
-                    Borders::default().modifiers(BorderType::Rounded).color(
-                        config
-                            .style_color_symbol
-                            .library_border()
-                            .unwrap_or(Color::Blue),
-                    ),
-                )
-                .background(
-                    config
-                        .style_color_symbol
-                        .library_background()
-                        .unwrap_or(Color::Reset),
-                )
-                .foreground(
-                    config
-                        .style_color_symbol
-                        .library_foreground()
-                        .unwrap_or(Color::Yellow),
+                    Borders::default()
+                        .modifiers(BorderType::Rounded)
+                        .color(config.settings.theme.library_border()),
                 )
+                .background(config.settings.theme.library_background())
+                .foreground(config.settings.theme.library_foreground())
                 .title(" Episodes: ", Alignment::Left)
                 .scroll(true)
-                .highlighted_color(
-                    config
-                        .style_color_symbol
-                        .library_highlight()
-                        .unwrap_or(Color::LightBlue),
-                )
-                .highlighted_str(&config.style_color_symbol.library_highlight_symbol)
+                .highlighted_color(config.settings.theme.library_highlight())
+                .highlighted_str(&config.settings.theme.style.library.highlight_symbol)
                 .rewind(false)
                 .step(4)
                 .scroll(true)
@@ -262,7 +228,7 @@ impl Component<Msg, NoUserEvent> for EpisodeList {
     #[allow(clippy::too_many_lines)]
     fn on(&mut self, ev: Event<NoUserEvent>) -> Option<Msg> {
         let config = self.config.clone();
-        let keys = &config.read().keys;
+        let keys = &config.read().settings.keys;
         let _cmd_result = match ev {
             Event::Keyboard(KeyEvent {
                 code: Key::Down,
@@ -283,11 +249,11 @@ impl Component<Msg, NoUserEvent> for EpisodeList {
                 self.perform(Cmd::Move(Direction::Up));
                 return Some(Msg::Podcast(PCMsg::DescriptionUpdate));
             }
-            Event::Keyboard(key) if key == keys.global_down.key_event() => {
+            Event::Keyboard(key) if key == keys.navigation_keys.down.get() => {
                 self.perform(Cmd::Move(Direction::Down));
                 return Some(Msg::Podcast(PCMsg::DescriptionUpdate));
             }
-            Event::Keyboard(key) if key == keys.global_up.key_event() => {
+            Event::Keyboard(key) if key == keys.navigation_keys.up.get() => {
                 if let State::One(StateValue::Usize(index)) = self.state() {
                     if index == 0 {
                         return Some(self.on_key_backtab.clone());
@@ -304,10 +270,10 @@ impl Component<Msg, NoUserEvent> for EpisodeList {
                 code: Key::PageUp,
                 modifiers: KeyModifiers::NONE,
             }) => self.perform(Cmd::Scroll(Direction::Up)),
-            Event::Keyboard(key) if key == keys.global_goto_top.key_event() => {
+            Event::Keyboard(key) if key == keys.navigation_keys.goto_top.get() => {
                 self.perform(Cmd::GoTo(Position::Begin))
             }
-            Event::Keyboard(key) if key == keys.global_goto_bottom.key_event() => {
+            Event::Keyboard(key) if key == keys.navigation_keys.goto_bottom.get() => {
                 self.perform(Cmd::GoTo(Position::End))
             }
             Event::Keyboard(KeyEvent {
@@ -338,25 +304,25 @@ impl Component<Msg, NoUserEvent> for EpisodeList {
                 CmdResult::None
             }
 
-            Event::Keyboard(keyevent) if keyevent == keys.global_right.key_event() => {
+            Event::Keyboard(keyevent) if keyevent == keys.navigation_keys.right.get() => {
                 if let State::One(StateValue::Usize(index)) = self.state() {
                     return Some(Msg::Podcast(PCMsg::EpisodeAdd(index)));
                 }
                 CmdResult::None
             }
 
-            Event::Keyboard(keyevent) if keyevent == keys.podcast_mark_played.key_event() => {
+            Event::Keyboard(keyevent) if keyevent == keys.podcast_keys.mark_played.get() => {
                 if let State::One(StateValue::Usize(index)) = self.state() {
                     return Some(Msg::Podcast(PCMsg::EpisodeMarkPlayed(index)));
                 }
                 CmdResult::None
             }
 
-            Event::Keyboard(keyevent) if keyevent == keys.podcast_mark_all_played.key_event() => {
+            Event::Keyboard(keyevent) if keyevent == keys.podcast_keys.mark_all_played.get() => {
                 return Some(Msg::Podcast(PCMsg::EpisodeMarkAllPlayed));
             }
 
-            Event::Keyboard(keyevent) if keyevent == keys.podcast_episode_download.key_event() => {
+            Event::Keyboard(keyevent) if keyevent == keys.podcast_keys.download_episode.get() => {
                 if let State::One(StateValue::Usize(index)) = self.state() {
                     return Some(Msg::Podcast(PCMsg::EpisodeDownload(index)));
                 }
@@ -364,14 +330,14 @@ impl Component<Msg, NoUserEvent> for EpisodeList {
             }
 
             Event::Keyboard(keyevent)
-                if keyevent == keys.podcast_episode_delete_file.key_event() =>
+                if keyevent == keys.podcast_keys.delete_local_episode.get() =>
             {
                 if let State::One(StateValue::Usize(index)) = self.state() {
                     return Some(Msg::Podcast(PCMsg::EpisodeDeleteFile(index)));
                 }
                 CmdResult::None
             }
-            Event::Keyboard(keyevent) if keyevent == keys.library_search.key_event() => {
+            Event::Keyboard(keyevent) if keyevent == keys.library_keys.search.get() => {
                 return Some(Msg::GeneralSearch(crate::ui::GSMsg::PopupShowEpisode))
             }
             _ => CmdResult::None,
@@ -381,23 +347,33 @@ impl Component<Msg, NoUserEvent> for EpisodeList {
 }
 
 impl Model {
+    #[allow(clippy::doc_markdown)]
+    /// Search ITunes for podcasts and send it to `Model::tx_to_main` as [`Msg::Podcast`] and [`PCMsg::Search*`](PCMsg).
+    ///
+    /// Requires that the current thread has a entered runtime
     pub fn podcast_search_itunes(&self, search_str: &str) {
         let encoded: String = utf8_percent_encode(search_str, NON_ALPHANUMERIC).to_string();
         let url =
             format!("https://itunes.apple.com/search?media=podcast&entity=podcast&term={encoded}",);
-        let agent = reqwest::blocking::ClientBuilder::new()
+        let agent = ClientBuilder::new()
             .connect_timeout(Duration::from_secs(5))
             .build()
             .expect("error build client");
         // let result = agent.get(&url).call()?;
 
-        let mut max_retries = self.config.read().podcast_max_retries;
+        let mut max_retries = self
+            .config_server
+            .read()
+            .settings
+            .podcast
+            .max_download_retries;
 
         let tx = self.tx_to_main.clone();
 
-        std::thread::spawn(move || {
-            let request: Result<reqwest::blocking::Response> = loop {
-                let response = agent.get(&url).send();
+        // this will work for now as the tui loop is a async function, and this function is called on the same thread
+        Handle::current().spawn(async move {
+            let request: Result<reqwest::Response> = loop {
+                let response = agent.get(&url).send().await;
                 if let Ok(resp) = response {
                     break Ok(resp);
                 }
@@ -412,7 +388,7 @@ impl Model {
             //     .expect("write failed");
             match request {
                 Ok(result) => match result.status() {
-                    reqwest::StatusCode::OK => match result.text() {
+                    reqwest::StatusCode::OK => match result.text().await {
                         Ok(text) => {
                             if let Some(vec) = parse_itunes_results(&text) {
                                 tx.send(Msg::Podcast(PCMsg::SearchSuccess(vec))).ok();
@@ -430,10 +406,10 @@ impl Model {
                             .ok();
                         }
                     },
-                    _ => {
-                        tx.send(Msg::Podcast(PCMsg::SearchError(
-                            "Error result status code".to_string(),
-                        )))
+                    code => {
+                        tx.send(Msg::Podcast(PCMsg::SearchError(format!(
+                            "Error result status code: {code}"
+                        ))))
                         .ok();
                     }
                 },
@@ -450,15 +426,21 @@ impl Model {
 
         crate::podcast::check_feed(
             feed,
-            self.config.read().podcast_max_retries,
-            &self.threadpool,
+            usize::from(
+                self.config_server
+                    .read()
+                    .settings
+                    .podcast
+                    .max_download_retries,
+            ),
+            &self.taskpool,
             self.tx_to_main.clone(),
         );
     }
     pub fn podcast_sync_feeds_and_episodes(&mut self) {
         let mut table: TableBuilder = TableBuilder::default();
 
-        for (idx, record) in self.podcasts.iter().enumerate() {
+        for (idx, record) in self.podcast.podcasts.iter().enumerate() {
             if idx > 0 {
                 table.add_row();
             }
@@ -471,7 +453,7 @@ impl Model {
 
             table.add_col(TextSpan::new(format!("{} ({new}/{total})", record.title)));
         }
-        if self.podcasts.is_empty() {
+        if self.podcast.podcasts.is_empty() {
             table.add_col(TextSpan::from("empty feeds list"));
         }
 
@@ -489,7 +471,7 @@ impl Model {
     }
 
     pub fn podcast_sync_episodes(&mut self) -> Result<()> {
-        if self.podcasts.is_empty() {
+        if self.podcast.podcasts.is_empty() {
             let mut table: TableBuilder = TableBuilder::default();
             table.add_col(TextSpan::from("empty episodes list"));
 
@@ -507,8 +489,9 @@ impl Model {
         }
 
         let podcast_selected = self
+            .podcast
             .podcasts
-            .get(self.podcasts_index)
+            .get(self.podcast.podcasts_index)
             .ok_or_else(|| anyhow!("get podcast selected failed."))?;
         // let episodes = self.db_podcast.get_episodes(podcast_selected.id, true)?;
         let mut table: TableBuilder = TableBuilder::default();
@@ -546,26 +529,29 @@ impl Model {
         Ok(())
     }
     pub fn episode_mark_played(&mut self, index: usize) -> Result<()> {
-        if self.podcasts.is_empty() {
+        if self.podcast.podcasts.is_empty() {
             return Ok(());
         }
         let podcast_selected = self
+            .podcast
             .podcasts
-            .get_mut(self.podcasts_index)
+            .get_mut(self.podcast.podcasts_index)
             .ok_or_else(|| anyhow!("get podcast selected failed."))?;
         let ep = podcast_selected
             .episodes
             .get_mut(index)
             .ok_or_else(|| anyhow!("get episode selected failed"))?;
         ep.played = !ep.played;
-        self.db_podcast.set_played_status(ep.id, ep.played)?;
+        self.podcast
+            .db_podcast
+            .set_played_status(ep.id, ep.played)?;
         self.podcast_sync_feeds_and_episodes();
 
         Ok(())
     }
 
     pub fn episode_mark_all_played(&mut self) -> Result<()> {
-        if self.podcasts.is_empty() {
+        if self.podcast.podcasts.is_empty() {
             return Ok(());
         }
 
@@ -574,8 +560,9 @@ impl Model {
             ep_index = idx;
         }
         let podcast_selected = self
+            .podcast
             .podcasts
-            .get_mut(self.podcasts_index)
+            .get_mut(self.podcast.podcasts_index)
             .ok_or_else(|| anyhow!("get podcast selected failed."))?;
         let played = podcast_selected
             .episodes
@@ -587,7 +574,9 @@ impl Model {
             epid_vec.push(ep.id);
             ep.played = !played;
         }
-        self.db_podcast.set_all_played_status(&epid_vec, !played)?;
+        self.podcast
+            .db_podcast
+            .set_all_played_status(&epid_vec, !played)?;
         self.podcast_sync_feeds_and_episodes();
 
         Ok(())
@@ -598,77 +587,16 @@ impl Model {
     /// `pod_id` will be None if a new podcast is being added (i.e.,
     /// the database has not given it an id yet).
     pub fn add_or_sync_data(&mut self, pod: &PodcastNoId, pod_id: Option<i64>) -> Result<()> {
-        // let title = pod.title.clone();
-        let db_result;
-
         if let Some(id) = pod_id {
-            db_result = self.db_podcast.update_podcast(id, pod);
+            self.podcast.db_podcast.update_podcast(id, pod)?;
         } else {
-            db_result = self.db_podcast.insert_podcast(pod);
+            self.podcast.db_podcast.insert_podcast(pod)?;
         }
-        match db_result {
-            Ok(_result) => {
-                {
-                    self.podcasts = self.db_podcast.get_podcasts()?;
-                    self.podcast_sync_feeds_and_episodes();
-                    Ok(())
-                }
-                // self.update_filters(self.filters, true);
 
-                // if pod_id.is_some() {
-                //     self.sync_tracker.push(result);
-                //     self.sync_counter -= 1;
-                //     self.update_tracker_notif();
+        self.podcast.podcasts = self.podcast.db_podcast.get_podcasts()?;
+        self.podcast_sync_feeds_and_episodes();
 
-                //     if self.sync_counter == 0 {
-                //         // count up total new episodes and updated
-                //         // episodes when sync process is finished
-                //         let mut added = 0;
-                //         let mut updated = 0;
-                //         let mut new_eps = Vec::new();
-                //         for res in self.sync_tracker.iter() {
-                //             added += res.added.len();
-                //             updated += res.updated.len();
-                //             new_eps.extend(res.added.clone());
-                //         }
-                //         self.sync_tracker = Vec::new();
-                //         self.notif_to_ui(
-                //             format!("Sync complete: Added {added}, updated {updated} episodes."),
-                //             false,
-                //         );
-
-                //         // deal with new episodes once syncing is
-                //         // complete, based on user preferences
-                //         if !new_eps.is_empty() {
-                //             match self.config.download_new_episodes {
-                //                 DownloadNewEpisodes::Always => {
-                //                     for ep in new_eps.into_iter() {
-                //                         self.download(ep.pod_id, Some(ep.id));
-                //                     }
-                //                 }
-                //                 DownloadNewEpisodes::AskSelected => {
-                //                     self.tx_to_ui
-                //                         .send(MainMessage::UiSpawnDownloadPopup(new_eps, true))
-                //                         .expect("Thread messaging error");
-                //                 }
-                //                 DownloadNewEpisodes::AskUnselected => {
-                //                     self.tx_to_ui
-                //                         .send(MainMessage::UiSpawnDownloadPopup(new_eps, false))
-                //                         .expect("Thread messaging error");
-                //                 }
-                //                 _ => (),
-                //             }
-                //         }
-                //     }
-                // } else {
-                //     self.notif_to_ui(
-                //         format!("Successfully added {} episodes.", result.added.len()),
-                //         false,
-                //     );
-                // }
-            }
-            Err(e) => Err(e),
-        }
+        Ok(())
     }
 
     /// Synchronize RSS feed data for one or more podcasts.
@@ -683,10 +611,11 @@ impl Model {
         match index {
             // just grab one podcast
             Some(i) => {
-                if self.podcasts.is_empty() {
+                if self.podcast.podcasts.is_empty() {
                     return Ok(());
                 }
                 let pod_selected = self
+                    .podcast
                     .podcasts
                     .get(i)
                     .ok_or_else(|| anyhow!("get podcast selected failed."))?;
@@ -701,6 +630,7 @@ impl Model {
             // get all of 'em!
             None => {
                 pod_data = self
+                    .podcast
                     .podcasts
                     .iter()
                     .map(|pod| {
@@ -712,8 +642,14 @@ impl Model {
         for feed in pod_data {
             crate::podcast::check_feed(
                 feed,
-                self.config.read().podcast_max_retries,
-                &self.threadpool,
+                usize::from(
+                    self.config_server
+                        .read()
+                        .settings
+                        .podcast
+                        .max_download_retries,
+                ),
+                &self.taskpool,
                 self.tx_to_main.clone(),
             );
         }
@@ -723,12 +659,13 @@ impl Model {
     }
 
     pub fn episode_download(&mut self, index: Option<usize>) -> Result<()> {
-        if self.podcasts.is_empty() {
+        if self.podcast.podcasts.is_empty() {
             return Ok(());
         }
         let podcast_selected = self
+            .podcast
             .podcasts
-            .get_mut(self.podcasts_index)
+            .get_mut(self.podcast.podcasts_index)
             .ok_or_else(|| anyhow!("get podcast selected failed."))?;
 
         let pod_title;
@@ -796,7 +733,7 @@ impl Model {
                     replacement: "",
                 },
             );
-            match crate::utils::create_podcast_dir(&self.config.read(), dir_name) {
+            match crate::utils::create_podcast_dir(&self.config_server.read(), dir_name) {
                 Ok(path) => {
                     // for ep in ep_data.iter() {
                     //     self.download_tracker.insert(ep.id);
@@ -804,8 +741,14 @@ impl Model {
                     download_list(
                         ep_data,
                         &path,
-                        self.config.read().podcast_max_retries,
-                        &self.threadpool,
+                        usize::from(
+                            self.config_server
+                                .read()
+                                .settings
+                                .podcast
+                                .max_download_retries,
+                        ),
+                        &self.taskpool,
                         &self.tx_to_main,
                     );
                 }
@@ -819,7 +762,7 @@ impl Model {
 
     pub fn episode_download_complete(&mut self, ep_data: EpData) -> Result<()> {
         let file_path = ep_data.file_path.unwrap();
-        let res = self.db_podcast.insert_file(ep_data.id, &file_path);
+        let res = self.podcast.db_podcast.insert_file(ep_data.id, &file_path);
         if res.is_err() {
             bail!(
                 "Could not add episode file to database: {}",
@@ -827,8 +770,8 @@ impl Model {
             );
         }
 
-        let podcasts = self.db_podcast.get_podcasts()?;
-        self.podcasts = podcasts;
+        let podcasts = self.podcast.db_podcast.get_podcasts()?;
+        self.podcast.podcasts = podcasts;
 
         self.podcast_sync_feeds_and_episodes();
         self.episode_update_playlist();
@@ -838,12 +781,13 @@ impl Model {
     /// Deletes a downloaded file for an episode from the user's local
     /// system.
     pub fn episode_delete_file(&mut self, ep_index: usize) -> Result<()> {
-        if self.podcasts.is_empty() {
+        if self.podcast.podcasts.is_empty() {
             return Ok(());
         }
         let podcast_selected = self
+            .podcast
             .podcasts
-            .get_mut(self.podcasts_index)
+            .get_mut(self.podcast.podcasts_index)
             .ok_or_else(|| anyhow!("get podcast selected failed."))?;
 
         let ep = podcast_selected
@@ -856,12 +800,19 @@ impl Model {
             let path = ep.path.clone().unwrap();
             match std::fs::remove_file(path) {
                 Ok(()) => {
-                    self.db_podcast.remove_file(ep.id).map_err(|e| {
+                    self.podcast.db_podcast.remove_file(ep.id).map_err(|e| {
                         anyhow!(format!("Could not remove file from db: {title} {e}"))
                     })?;
                     ep.path = None;
                 }
-                Err(e) => bail!(format!("Error deleting \"{title}\": {e}")),
+                Err(e) => {
+                    // Repeat the same thing in case the local file is missing. Update db
+                    self.podcast.db_podcast.remove_file(ep.id).map_err(|e| {
+                        anyhow!(format!("Could not remove file from db: {title} {e}"))
+                    })?;
+                    ep.path = None;
+                    bail!(format!("Error deleting \"{title}\": {e}"));
+                }
             }
         }
         self.podcast_sync_feeds_and_episodes();
@@ -879,9 +830,10 @@ impl Model {
         let mut success = true;
         {
             let podcast_selected = self
+                .podcast
                 .podcasts
                 .get_mut(pod_index)
-                .ok_or_else(|| anyhow!("get podcast selected failed."))?;
+                .ok_or_else(|| anyhow!("failed to find the podcast selected for deletion."))?;
 
             for ep in &mut podcast_selected.episodes {
                 if ep.path.is_some() {
@@ -896,29 +848,29 @@ impl Model {
             }
         }
 
-        self.db_podcast.remove_files(&eps_to_remove)?;
+        self.podcast.db_podcast.remove_files(&eps_to_remove)?;
         if !success {
-            bail!("Error happend when removing local file. Please check.");
+            bail!("Error happened when deleting a file. Please verify that it is accessible.");
         }
 
         Ok(())
     }
 
     pub fn podcast_remove_all_feeds(&mut self) -> Result<()> {
-        if self.podcasts.is_empty() {
+        if self.podcast.podcasts.is_empty() {
             return Ok(());
         }
 
-        let len = self.podcasts.len();
+        let len = self.podcast.podcasts.len();
 
         for index in 0..len {
             self.podcast_delete_files(index).ok();
         }
 
-        self.db_podcast.clear_db()?;
+        self.podcast.db_podcast.clear_db()?;
 
-        self.podcasts = Vec::new();
-        self.podcasts_index = 0;
+        self.podcast.podcasts = Vec::new();
+        self.podcast.podcasts_index = 0;
 
         self.podcast_sync_feeds_and_episodes();
         self.episode_update_playlist();
@@ -926,17 +878,19 @@ impl Model {
     }
 
     pub fn podcast_remove_feed(&mut self) -> Result<()> {
-        if self.podcasts.is_empty() {
+        if self.podcast.podcasts.is_empty() {
             return Ok(());
         }
 
         if let Ok(feed_index) = self.podcast_get_feed_index() {
             self.podcast_delete_files(feed_index)?;
-            let podcast_selected = self.podcasts.remove(feed_index);
-            self.db_podcast.remove_podcast(podcast_selected.id)?;
+            let podcast_selected = self.podcast.podcasts.remove(feed_index);
+            self.podcast
+                .db_podcast
+                .remove_podcast(podcast_selected.id)?;
         }
 
-        self.podcasts_index = self.podcasts_index.saturating_sub(1);
+        self.podcast.podcasts_index = self.podcast.podcasts_index.saturating_sub(1);
         self.podcast_sync_feeds_and_episodes();
         self.episode_update_playlist();
         Ok(())
@@ -957,18 +911,20 @@ impl Model {
     }
 
     pub fn podcast_mark_current_track_played(&mut self) -> Result<()> {
-        if self.podcasts.is_empty() {
+        if self.podcast.podcasts.is_empty() {
             return Ok(());
         }
         if let Some(track) = self.playlist.current_track() {
             if MediaType::Podcast == track.media_type {
                 if let Some(url) = track.file() {
-                    'outer: for pod in &mut self.podcasts {
+                    'outer: for pod in &mut self.podcast.podcasts {
                         for ep in &mut pod.episodes {
                             if ep.url == url {
                                 if !ep.played {
                                     ep.played = true;
-                                    self.db_podcast.set_played_status(ep.id, ep.played)?;
+                                    self.podcast
+                                        .db_podcast
+                                        .set_played_status(ep.id, ep.played)?;
                                 }
                                 break 'outer;
                             }
@@ -984,10 +940,10 @@ impl Model {
     }
 
     pub fn podcast_get_album_photo_by_url(&self, url: &str) -> Option<String> {
-        if self.podcasts.is_empty() {
+        if self.podcast.podcasts.is_empty() {
             return None;
         }
-        for pod in &self.podcasts {
+        for pod in &self.podcast.podcasts {
             for ep in &pod.episodes {
                 if ep.url == url {
                     return pod.image_url.clone();
@@ -1019,16 +975,16 @@ impl Model {
         let search = format!("*{}*", input.to_lowercase());
         let mut db_tracks = vec![];
         // Get all episodes
-        let podcasts = self.podcasts.clone();
+        let podcasts = self.podcast.podcasts.clone();
         for podcast in podcasts {
-            if let Ok(episodes) = self.db_podcast.get_episodes(podcast.id, true) {
+            if let Ok(episodes) = self.podcast.db_podcast.get_episodes(podcast.id, true) {
                 db_tracks.extend(episodes);
             }
         }
 
         if db_tracks.is_empty() {
             table.add_col(TextSpan::from("0"));
-            table.add_col(TextSpan::from("empty tracks from podcasts db"));
+            table.add_col(TextSpan::from("empty tracks in the podcasts db"));
             table.add_col(TextSpan::from(""));
         } else {
             for record in db_tracks {
@@ -1054,11 +1010,11 @@ impl Model {
         let mut idx = 0;
         let search = format!("*{}*", input.to_lowercase());
         // Get all episodes
-        let db_tracks = self.podcasts.clone();
+        let db_tracks = self.podcast.podcasts.clone();
 
         if db_tracks.is_empty() {
             table.add_col(TextSpan::from("0"));
-            table.add_col(TextSpan::from("empty tracks from podcasts db"));
+            table.add_col(TextSpan::from("empty tracks in the podcasts db"));
             table.add_col(TextSpan::from(""));
         } else {
             for record in db_tracks {
@@ -1103,11 +1059,11 @@ impl Model {
 
     #[allow(clippy::cast_possible_wrap)]
     pub fn podcast_find_by_ep_id(&mut self, ep_id: usize) -> Result<(usize, usize)> {
-        for (podcast_index, podcast) in self.podcasts.iter().enumerate() {
+        for (podcast_index, podcast) in self.podcast.podcasts.iter().enumerate() {
             for (episode_index, episode) in podcast.episodes.iter().enumerate() {
                 if episode.id == ep_id as i64 {
                     // Need to set podcast index here, otherwise the wrong episodes will be added
-                    self.podcasts_index = podcast_index;
+                    self.podcast.podcasts_index = podcast_index;
                     return Ok((podcast_index, episode_index));
                 }
             }
@@ -1117,10 +1073,10 @@ impl Model {
 
     #[allow(clippy::cast_possible_wrap)]
     pub fn podcast_find_by_pod_id(&mut self, pod_id: usize) -> Result<usize> {
-        for (podcast_index, podcast) in self.podcasts.iter().enumerate() {
+        for (podcast_index, podcast) in self.podcast.podcasts.iter().enumerate() {
             if podcast.id == pod_id as i64 {
                 // Need to set podcast index here
-                self.podcasts_index = podcast_index;
+                self.podcast.podcasts_index = podcast_index;
                 return Ok(podcast_index);
             }
         }
