@@ -1,6 +1,7 @@
 use crate::ui::Model;
 use anyhow::{anyhow, Context, Result};
 use std::path::Path;
+use termusiclib::config::SharedTuiSettings;
 /**
  * MIT License
  *
@@ -26,52 +27,34 @@ use std::path::Path;
  */
 use termusiclib::songtag::{search, SongTag};
 use termusiclib::types::{Id, IdTagEditor, Msg, SearchLyricState, TEMsg, TFMsg};
-use termusicplayback::SharedSettings;
+use tokio::runtime::Handle;
 use tui_realm_stdlib::Table;
 use tuirealm::command::{Cmd, CmdResult, Direction, Position};
 use tuirealm::event::{Key, KeyEvent, KeyModifiers, NoUserEvent};
-use tuirealm::props::{Alignment, BorderType, Borders, Color, TableBuilder, TextSpan};
+use tuirealm::props::{Alignment, BorderType, Borders, TableBuilder, TextSpan};
 use tuirealm::{Component, Event, MockComponent, State, StateValue};
 
 #[derive(MockComponent)]
 pub struct TETableLyricOptions {
     component: Table,
-    config: SharedSettings,
+    config: SharedTuiSettings,
 }
 
 impl TETableLyricOptions {
-    pub fn new(config: SharedSettings) -> Self {
+    pub fn new(config: SharedTuiSettings) -> Self {
         let component = {
             let config = config.read();
             Table::default()
                 .borders(
-                    Borders::default().modifiers(BorderType::Rounded).color(
-                        config
-                            .style_color_symbol
-                            .library_border()
-                            .unwrap_or(Color::Blue),
-                    ),
-                )
-                .foreground(
-                    config
-                        .style_color_symbol
-                        .library_foreground()
-                        .unwrap_or(Color::Yellow),
-                )
-                .background(
-                    config
-                        .style_color_symbol
-                        .library_background()
-                        .unwrap_or(Color::Reset),
+                    Borders::default()
+                        .modifiers(BorderType::Rounded)
+                        .color(config.settings.theme.library_border()),
                 )
+                .foreground(config.settings.theme.library_foreground())
+                .background(config.settings.theme.library_background())
                 .title(" Search Results ", Alignment::Left)
                 .scroll(true)
-                .highlighted_color(
-                    config
-                        .style_color_symbol
-                        .library_highlight()
-                        .unwrap_or(Color::LightBlue),
-                )
+                .highlighted_color(config.settings.theme.library_highlight())
                 .highlighted_str("\u{1f680}")
                 // .highlighted_str("🚀")
                 .rewind(false)
@@ -96,7 +79,7 @@ impl TETableLyricOptions {
 impl Component<Msg, NoUserEvent> for TETableLyricOptions {
     fn on(&mut self, ev: Event<NoUserEvent>) -> Option<Msg> {
         let config = self.config.clone();
-        let keys = &config.read().keys;
+        let keys = &config.read().settings.keys;
         let _cmd_result = match ev {
             Event::Keyboard(KeyEvent { code: Key::Tab, .. }) => {
                 return Some(Msg::TagEditor(TEMsg::TEFocus(
@@ -112,14 +95,14 @@ impl Component<Msg, NoUserEvent> for TETableLyricOptions {
                 )))
             }
 
-            Event::Keyboard(keyevent) if keyevent == keys.config_save.key_event() => {
+            Event::Keyboard(keyevent) if keyevent == keys.config_keys.save.get() => {
                 return Some(Msg::TagEditor(TEMsg::TERename))
             }
 
-            Event::Keyboard(k) if k == keys.global_quit.key_event() => {
+            Event::Keyboard(k) if k == keys.quit.get() => {
                 return Some(Msg::TagEditor(TEMsg::TagEditorClose(None)))
             }
-            Event::Keyboard(k) if k == keys.global_esc.key_event() => {
+            Event::Keyboard(k) if k == keys.escape.get() => {
                 return Some(Msg::TagEditor(TEMsg::TagEditorClose(None)))
             }
             Event::Keyboard(KeyEvent {
@@ -128,10 +111,10 @@ impl Component<Msg, NoUserEvent> for TETableLyricOptions {
             Event::Keyboard(KeyEvent { code: Key::Up, .. }) => {
                 self.perform(Cmd::Move(Direction::Up))
             }
-            Event::Keyboard(k) if k == keys.global_down.key_event() => {
+            Event::Keyboard(k) if k == keys.navigation_keys.down.get() => {
                 self.perform(Cmd::Move(Direction::Down))
             }
-            Event::Keyboard(k) if k == keys.global_up.key_event() => {
+            Event::Keyboard(k) if k == keys.navigation_keys.up.get() => {
                 self.perform(Cmd::Move(Direction::Up))
             }
             Event::Keyboard(KeyEvent {
@@ -148,14 +131,14 @@ impl Component<Msg, NoUserEvent> for TETableLyricOptions {
                 self.perform(Cmd::GoTo(Position::End))
             }
 
-            Event::Keyboard(k) if k == keys.global_goto_top.key_event() => {
+            Event::Keyboard(k) if k == keys.navigation_keys.goto_top.get() => {
                 self.perform(Cmd::GoTo(Position::Begin))
             }
 
-            Event::Keyboard(k) if k == keys.global_goto_bottom.key_event() => {
+            Event::Keyboard(k) if k == keys.navigation_keys.goto_bottom.get() => {
                 self.perform(Cmd::GoTo(Position::End))
             }
-            Event::Keyboard(k) if k == keys.library_search_youtube.key_event() => {
+            Event::Keyboard(k) if k == keys.library_keys.youtube_search.get() => {
                 if let State::One(StateValue::Usize(index)) = self.state() {
                     return Some(Msg::TagEditor(TEMsg::TEDownload(index)));
                 }
@@ -248,7 +231,12 @@ impl Model {
                 }
             }
         }
-        search(&search_str, self.sender_songtag.clone());
+        // this needs to be wrapped as this is not running another thread but some main-runtime thread and so needs to inform the runtime to hand-off other tasks
+        // though i am not fully sure if that is 100% the case, this avoid the panic though
+        tokio::task::block_in_place(move || {
+            // TODO: consider changing this to be a spawn, but will require "search" param changes
+            Handle::current().block_on(search(&search_str, self.sender_songtag.clone()));
+        });
     }
     pub fn te_update_lyric_options(&mut self) {
         if self
@@ -269,7 +257,12 @@ impl Model {
             .with_context(|| format!("no song_tag with index {index} found"))?;
         if let Some(song) = &self.tageditor_song {
             let file = song.file().context("no file path found")?;
-            song_tag.download(file, &self.tx_to_main)?;
+            // this needs to be wrapped as this is not running another thread but some main-runtime thread and so needs to inform the runtime to hand-off other tasks
+            // though i am not fully sure if that is 100% the case, this avoid the panic though
+            let tx_to_main = &self.tx_to_main;
+            tokio::task::block_in_place(move || {
+                Handle::current().block_on(song_tag.download(file, tx_to_main))
+            })?;
         }
         Ok(())
     }
@@ -323,10 +316,18 @@ impl Model {
                 song.set_album(album);
             }
 
-            if let Ok(lyric_string) = song_tag.fetch_lyric() {
+            // this needs to be wrapped as this is not running another thread but some main-runtime thread and so needs to inform the runtime to hand-off other tasks
+            // though i am not fully sure if that is 100% the case, this avoid the panic though
+            let (lyric_string, artwork) = tokio::task::block_in_place(move || {
+                Handle::current().block_on(async {
+                    tokio::join!(song_tag.fetch_lyric(), song_tag.fetch_photo())
+                })
+            });
+
+            if let Ok(lyric_string) = lyric_string {
                 song.set_lyric(&lyric_string, lang_ext);
             }
-            if let Ok(artwork) = song_tag.fetch_photo() {
+            if let Ok(artwork) = artwork {
                 song.set_photo(artwork);
             }
 
