@@ -1,3 +1,4 @@
+mod download_tracker;
 /**
  * MIT License
  *
@@ -24,40 +25,44 @@
 mod update;
 mod view;
 mod youtube_options;
-use crate::ui::Application;
-use termusiclib::sqlite::{DataBase, SearchCriteria};
-use termusiclib::types::{Id, Msg, SearchLyricState, YoutubeOptions};
 
-#[cfg(all(feature = "cover", not(target_os = "windows")))]
+use crate::ui::Application;
+use crate::CombinedSettings;
+use download_tracker::DownloadTracker;
+use termusiclib::config::v2::tui::keys::Keys;
+use termusiclib::config::v2::tui::theme::ThemeWrap;
+use termusiclib::library_db::{DataBase, SearchCriteria};
+use termusiclib::types::{Id, Msg, SearchLyricState, YoutubeOptions};
+use termusiclib::xywh;
+
+use termusiclib::track::{MediaType, Track};
+#[cfg(all(feature = "cover-ueberzug", not(target_os = "windows")))]
 use termusiclib::ueberzug::UeInstance;
-use termusiclib::{
-    config::Settings,
-    track::{MediaType, Track},
-};
 
 use anyhow::{anyhow, Result};
 use std::path::PathBuf;
 use std::sync::mpsc::{self, Receiver, Sender};
 use std::time::{Duration, Instant};
-use termusiclib::config::{Keys, StyleColorSymbol};
-use termusiclib::podcast::{db::Database as DBPod, Podcast, PodcastFeed, Threadpool};
+use termusiclib::config::{ServerOverlay, SharedServerSettings, SharedTuiSettings};
+use termusiclib::library_db::TrackDB;
+use termusiclib::podcast::{db::Database as DBPod, Podcast, PodcastFeed};
 use termusiclib::songtag::SongTag;
-use termusiclib::sqlite::TrackForDB;
-use termusiclib::utils::{get_app_config_path, DownloadTracker};
-use termusicplayback::{PlayerCmd, Playlist, SharedSettings};
+use termusiclib::taskpool::TaskPool;
+use termusiclib::utils::get_app_config_path;
+use termusicplayback::{PlayerCmd, Playlist};
 use tokio::sync::mpsc::UnboundedSender;
 use tui_realm_treeview::Tree;
 use tuirealm::event::NoUserEvent;
 use tuirealm::terminal::TerminalBridge;
 
-#[derive(PartialEq, Eq)]
+#[derive(Debug, PartialEq, Eq, Clone, Copy)]
 pub enum TermusicLayout {
     TreeView,
     DataBase,
     Podcast,
 }
 
-#[derive(PartialEq, Clone, Eq)]
+#[derive(Debug, PartialEq, Clone, Copy, Eq)]
 pub enum ConfigEditorLayout {
     General,
     Color,
@@ -65,6 +70,67 @@ pub enum ConfigEditorLayout {
     Key2,
 }
 
+/// All data specific to the Music Library Widget / View
+#[derive(Debug)]
+pub struct MusicLibraryData {
+    /// Current Path that the library-tree is in
+    pub tree_path: PathBuf,
+    /// Tree of the Music Library widget
+    pub tree: Tree,
+    /// The Node that a yank & paste was started on
+    pub yanked_node_id: Option<String>,
+}
+
+/// All data specific to the Database Widget / View
+#[derive(Debug)]
+pub struct DatabaseWidgetData {
+    /// Criteria to search for
+    pub criteria: SearchCriteria,
+    /// Criteria Search results `(criteria -> this)`
+    pub search_results: Vec<String>,
+    /// Results of the critea results search `(criteria -> search_results -> this)`
+    pub search_tracks: Vec<TrackDB>,
+}
+
+impl DatabaseWidgetData {
+    /// Reset all search Vectors to a new empty Vector
+    ///
+    /// (removing allocations)
+    pub fn reset_search_results(&mut self) {
+        // Reset instead of ".clear" as "clear" does not remove capacity and might not be used again and could potentially be large
+        self.search_results = Vec::new();
+        self.search_tracks = Vec::new();
+    }
+}
+
+/// All data specific to the Podcast Widget / View
+#[derive(Debug)]
+pub struct PodcastWidgetData {
+    /// Loaded and displayed Podcast list
+    pub podcasts: Vec<Podcast>,
+    /// Selected podcast index
+    pub podcasts_index: usize,
+    /// Podcast Database
+    pub db_podcast: DBPod,
+    /// Podcast search results
+    pub search_results: Option<Vec<PodcastFeed>>,
+}
+
+/// All data specific to the Config Editor Widget / View
+#[derive(Debug)]
+pub struct ConfigEditorData {
+    /// All possible themes that could be selected
+    pub themes: Vec<String>,
+    /// The Theme to edit to preview before saving
+    pub theme: ThemeWrap,
+    /// The Keybindings to preview before saving
+    pub key_config: Keys,
+    /// The current tab in the config editor
+    pub layout: ConfigEditorLayout,
+    /// Indicator to prompt a save on config editor exit
+    pub config_changed: bool,
+}
+
 pub struct Model {
     /// Indicates that the application must quit
     pub quit: bool,
@@ -74,149 +140,197 @@ pub struct Model {
     pub app: Application<Id, Msg, NoUserEvent>,
     /// Used to draw to terminal
     pub terminal: TerminalBridge,
-    pub path: PathBuf,
-    pub tree: Tree,
-    pub config: SharedSettings,
-    pub yanked_node_id: Option<String>,
+    pub tx_to_main: Sender<Msg>,
+    pub rx_to_main: Receiver<Msg>,
+    /// Sender for Player Commands
+    pub cmd_tx: UnboundedSender<PlayerCmd>,
+
+    pub config_tui: SharedTuiSettings,
+    pub config_server: SharedServerSettings,
+    pub db: DataBase,
+
+    pub layout: TermusicLayout,
+    pub library: MusicLibraryData,
+    pub dw: DatabaseWidgetData,
+    pub podcast: PodcastWidgetData,
+    pub config_editor: ConfigEditorData,
+
+    /// Clone of `playlist.current_track`, but kept around when playlist goes empty but song is still playing
     pub current_song: Option<Track>,
     pub tageditor_song: Option<Track>,
     pub time_pos: Duration,
     pub lyric_line: String,
-    youtube_options: YoutubeOptions,
-    #[cfg(all(feature = "cover", not(target_os = "windows")))]
+    pub playlist: Playlist,
+
+    #[cfg(all(feature = "cover-ueberzug", not(target_os = "windows")))]
     pub ueberzug_instance: UeInstance,
+    pub viuer_supported: ViuerSupported,
+    pub xywh: xywh::Xywh,
+
+    youtube_options: YoutubeOptions,
     pub songtag_options: Vec<SongTag>,
     pub sender_songtag: Sender<SearchLyricState>,
     pub receiver_songtag: Receiver<SearchLyricState>,
-    pub viuer_supported: ViuerSupported,
-    pub ce_themes: Vec<String>,
-    pub ce_style_color_symbol: StyleColorSymbol,
-    pub ke_key_config: Keys,
-    pub db: DataBase,
-    pub db_criteria: SearchCriteria,
-    pub db_search_results: Vec<String>,
-    pub db_search_tracks: Vec<TrackForDB>,
-    pub layout: TermusicLayout,
-    pub config_layout: ConfigEditorLayout,
-    pub config_changed: bool,
     pub download_tracker: DownloadTracker,
-    pub podcasts: Vec<Podcast>,
-    pub podcasts_index: usize,
-    pub db_podcast: DBPod,
-    pub threadpool: Threadpool,
-    pub tx_to_main: Sender<Msg>,
-    pub rx_to_main: Receiver<Msg>,
-    pub podcast_search_vec: Option<Vec<PodcastFeed>>,
-    pub playlist: Playlist,
-    pub cmd_tx: UnboundedSender<PlayerCmd>,
+    /// Taskpool to limit number of active network requests
+    ///
+    /// Currently only used for podcast sync & download
+    pub taskpool: TaskPool,
 }
 
-#[derive(Debug)]
+#[derive(Debug, PartialEq, Clone, Copy)]
 pub enum ViuerSupported {
+    #[cfg(feature = "cover-viuer-kitty")]
     Kitty,
+    #[cfg(feature = "cover-viuer-iterm")]
     ITerm,
-    // Sixel,
+    #[cfg(feature = "cover-viuer-sixel")]
+    Sixel,
     NotSupported,
 }
 
+fn get_viuer_support() -> ViuerSupported {
+    #[cfg(feature = "cover-viuer-kitty")]
+    if viuer::KittySupport::None != viuer::get_kitty_support() {
+        return ViuerSupported::Kitty;
+    }
+    #[cfg(feature = "cover-viuer-iterm")]
+    if viuer::is_iterm_supported() {
+        return ViuerSupported::ITerm;
+    }
+    #[cfg(feature = "cover-viuer-sixel")]
+    if viuer::is_sixel_supported() {
+        return ViuerSupported::Sixel;
+    }
+
+    ViuerSupported::NotSupported
+}
+
 impl Model {
-    pub async fn new(config: Settings, cmd_tx: UnboundedSender<PlayerCmd>) -> Self {
-        let path = Self::get_full_path_from_config(&config);
-        let tree = Tree::new(Self::library_dir_tree(&path, config.max_depth_cli));
+    pub async fn new(config: CombinedSettings, cmd_tx: UnboundedSender<PlayerCmd>) -> Self {
+        let CombinedSettings {
+            server: config_server,
+            tui: config_tui,
+        } = config;
+        let path = Self::get_full_path_from_config(&config_server.read());
+        let tree = Tree::new(Self::library_dir_tree(
+            &path,
+            config_server.read().get_library_scan_depth(),
+        ));
 
         let (tx3, rx3): (Sender<SearchLyricState>, Receiver<SearchLyricState>) = mpsc::channel();
 
-        let mut viuer_supported = ViuerSupported::NotSupported;
-        if viuer::KittySupport::None != viuer::get_kitty_support() {
-            viuer_supported = ViuerSupported::Kitty;
-        } else if viuer::is_iterm_supported() {
-            viuer_supported = ViuerSupported::ITerm;
-        }
-        let db = DataBase::new(&config);
+        let viuer_supported = get_viuer_support();
+        let db = DataBase::new(&config_server.read()).expect("Open Library Database");
         let db_criteria = SearchCriteria::Artist;
         let terminal = TerminalBridge::new().expect("Could not initialize terminal");
-        // let viuer_supported =
-        //     viuer::KittySupport::None != viuer::get_kitty_support() || viuer::is_iterm_supported();
 
-        #[cfg(all(feature = "cover", not(target_os = "windows")))]
+        #[cfg(all(feature = "cover-ueberzug", not(target_os = "windows")))]
         let ueberzug_instance = UeInstance::default();
         let db_path = get_app_config_path().expect("failed to get podcast db path.");
 
-        let db_podcast = DBPod::connect(&db_path).expect("error connecting to podcast db.");
+        let db_podcast = DBPod::new(&db_path).expect("error connecting to podcast db.");
 
         let podcasts = db_podcast
             .get_podcasts()
             .expect("failed to get podcasts from db.");
-        let threadpool = Threadpool::new(config.podcast_simultanious_download);
+        let taskpool = TaskPool::new(usize::from(
+            config_server
+                .read()
+                .settings
+                .podcast
+                .concurrent_downloads_max
+                .get(),
+        ));
         let (tx_to_main, rx_to_main) = mpsc::channel();
 
-        let config = std::sync::Arc::new(parking_lot::RwLock::new(config));
-
-        let playlist = Playlist::new(config.clone()).unwrap_or_default();
-        let app = Self::init_app(&tree, &config);
+        let playlist = Playlist::new(config_server.clone()).unwrap_or_default();
+        let app = Self::init_app(&tree, &config_tui);
 
         // This line is required, in order to show the playing message for the first track
         // playlist.set_current_track_index(0);
 
+        let ce_theme = config_tui.read().settings.theme.clone();
+        let xywh = xywh::Xywh::from(&config_tui.read().settings.coverart);
+
         Self {
             app,
             quit: false,
             redraw: true,
             last_redraw: Instant::now(),
-            tree,
-            path,
             terminal,
-            config,
-            yanked_node_id: None,
+            config_server,
+            config_tui,
             // current_song: None,
             tageditor_song: None,
             time_pos: Duration::default(),
             lyric_line: String::new(),
 
+            library: MusicLibraryData {
+                tree_path: path,
+                tree,
+                yanked_node_id: None,
+            },
             // TODO: Consider making YoutubeOptions async and use async reqwest in YoutubeOptions
             // and avoid this `spawn_blocking` call.
             youtube_options: tokio::task::spawn_blocking(YoutubeOptions::default)
                 .await
                 .expect("Failed to initialize YoutubeOptions in a blocking task due to a panic"),
-            #[cfg(all(feature = "cover", not(target_os = "windows")))]
+            #[cfg(all(feature = "cover-ueberzug", not(target_os = "windows")))]
             ueberzug_instance,
             songtag_options: vec![],
             sender_songtag: tx3,
             receiver_songtag: rx3,
             viuer_supported,
-            ce_themes: vec![],
-            ce_style_color_symbol: StyleColorSymbol::default(),
-            ke_key_config: Keys::default(),
             db,
             layout: TermusicLayout::TreeView,
-            config_layout: ConfigEditorLayout::General,
-            db_criteria,
-            db_search_results: Vec::new(),
-            db_search_tracks: Vec::new(),
-            config_changed: false,
-            podcasts,
-            podcasts_index: 0,
-            db_podcast,
-            threadpool,
+            dw: DatabaseWidgetData {
+                criteria: db_criteria,
+                search_results: Vec::new(),
+                search_tracks: Vec::new(),
+            },
+            podcast: PodcastWidgetData {
+                podcasts,
+                podcasts_index: 0,
+                db_podcast,
+                search_results: None,
+            },
+            config_editor: ConfigEditorData {
+                themes: Vec::new(),
+                theme: ce_theme,
+                key_config: Keys::default(),
+                layout: ConfigEditorLayout::General,
+                config_changed: false,
+            },
+            taskpool,
             tx_to_main,
             rx_to_main,
             download_tracker: DownloadTracker::default(),
-            podcast_search_vec: None,
             playlist,
             cmd_tx,
             current_song: None,
+            xywh,
         }
     }
 
-    pub fn get_full_path_from_config(config: &Settings) -> PathBuf {
+    #[inline]
+    pub fn get_combined_settings(&self) -> CombinedSettings {
+        CombinedSettings {
+            server: self.config_server.clone(),
+            tui: self.config_tui.clone(),
+        }
+    }
+
+    /// Get the first music directory or the cli provided music dir resolved
+    pub fn get_full_path_from_config(config: &ServerOverlay) -> PathBuf {
         let mut full_path = String::new();
-        if let Some(dir) = config.music_dir.first() {
-            full_path = shellexpand::tilde(dir).to_string();
+
+        if let Some(first_music_dir) = config.get_first_music_dir() {
+            full_path = shellexpand::path::tilde(first_music_dir)
+                .to_string_lossy()
+                .to_string();
         }
 
-        if let Some(music_dir) = &config.music_dir_from_cli {
-            full_path = shellexpand::tilde(music_dir).to_string();
-        };
         PathBuf::from(full_path)
     }
 
@@ -225,7 +339,7 @@ impl Model {
             self.mount_error_popup(e.context("theme save"));
         }
         self.mount_label_help();
-        self.db.sync_database(&self.path);
+        self.db.sync_database(&self.library.tree_path);
         self.playlist_sync();
     }
 
