@@ -2,7 +2,7 @@ use crate::ui::{model::TermusicLayout, Model};
 use anyhow::anyhow;
 use std::thread::{self, sleep};
 use std::time::Duration;
-use termusiclib::sqlite::SearchCriteria;
+use termusiclib::library_db::SearchCriteria;
 use termusiclib::track::MediaType;
 use termusiclib::types::{
     DBMsg, DLMsg, GSMsg, Id, IdTagEditor, LIMsg, LyricMsg, Msg, PCMsg, PLMsg, XYWHMsg, YSMsg,
@@ -44,7 +44,7 @@ impl Update<Msg> for Model {
             self.redraw = true;
             // Match message
             match msg {
-                Msg::ConfigEditor(m) => self.update_config_editor(&m),
+                Msg::ConfigEditor(m) => self.update_config_editor(m),
                 Msg::DataBase(m) => self.update_database_list(&m),
 
                 Msg::DeleteConfirmShow
@@ -58,7 +58,7 @@ impl Update<Msg> for Model {
                     None
                 }
                 Msg::QuitPopupShow => {
-                    if self.config.read().enable_exit_confirmation {
+                    if self.config_tui.read().settings.behavior.confirm_quit {
                         self.mount_quit_popup();
                     } else {
                         self.quit = true;
@@ -168,9 +168,9 @@ impl Update<Msg> for Model {
                     None
                 }
                 Msg::Podcast(m) => self.update_podcast(&m),
-                Msg::LyricMessage(m) => self.update_lyric_textarea(&m),
+                Msg::LyricMessage(m) => self.update_lyric_textarea(m),
                 Msg::Download(m) => self.update_download_msg(&m),
-                Msg::Xywh(m) => self.update_xywh_msg(&m),
+                Msg::Xywh(m) => self.update_xywh_msg(m),
             }
         } else {
             None
@@ -179,7 +179,16 @@ impl Update<Msg> for Model {
 }
 
 impl Model {
-    fn update_xywh_msg(&mut self, msg: &XYWHMsg) -> Option<Msg> {
+    /// Ensure the [`QuitPopup`](crate::ui::components::QuitPopup) always has the focus (top-most)
+    pub fn ensure_quit_popup_top_most_focus(&mut self) {
+        if self.app.mounted(&Id::QuitPopup)
+            && !self.app.focus().is_some_and(|v| *v == Id::QuitPopup)
+        {
+            self.app.active(&Id::QuitPopup).ok();
+        }
+    }
+
+    fn update_xywh_msg(&mut self, msg: XYWHMsg) -> Option<Msg> {
         match msg {
             XYWHMsg::MoveLeft => self.xywh_move_left(),
             XYWHMsg::MoveRight => self.xywh_move_right(),
@@ -194,7 +203,7 @@ impl Model {
         None
     }
 
-    fn update_lyric_textarea(&mut self, msg: &LyricMsg) -> Option<Msg> {
+    fn update_lyric_textarea(&mut self, msg: LyricMsg) -> Option<Msg> {
         match msg {
             LyricMsg::LyricTextAreaBlurUp => self.app.active(&Id::Playlist).ok(),
             LyricMsg::LyricTextAreaBlurDown => match self.layout {
@@ -268,7 +277,7 @@ impl Model {
                 );
             }
             PCMsg::PodcastSelected(index) => {
-                self.podcasts_index = *index;
+                self.podcast.podcasts_index = *index;
                 if let Err(e) = self.podcast_sync_episodes() {
                     self.mount_error_popup(e.context("podcast sync episodes"));
                 }
@@ -390,7 +399,7 @@ impl Model {
             PCMsg::FeedsDeleteCloseCancel => self.umount_feed_delete_confirm_input(),
             PCMsg::SearchItunesCloseCancel => self.umount_podcast_search_table(),
             PCMsg::SearchItunesCloseOk(index) => {
-                if let Some(vec) = &self.podcast_search_vec {
+                if let Some(vec) = &self.podcast.search_results {
                     if let Some(pod) = vec.get(*index) {
                         let url = pod.url.clone();
                         self.podcast_add(&url);
@@ -398,7 +407,7 @@ impl Model {
                 }
             }
             PCMsg::SearchSuccess(vec) => {
-                self.podcast_search_vec = Some(vec.clone());
+                self.podcast.search_results = Some(vec.clone());
                 self.update_podcast_search_table();
             }
             PCMsg::SearchError(e) => self.mount_error_popup(anyhow!(e.to_owned())),
@@ -562,15 +571,15 @@ impl Model {
                 self.app.active(&Id::DBListCriteria).ok();
             }
             DBMsg::SearchResult(index) => {
-                self.db_criteria = SearchCriteria::from(*index);
+                self.dw.criteria = SearchCriteria::from(*index);
                 self.database_update_search_results();
             }
             DBMsg::SearchTrack(index) => {
                 self.database_update_search_tracks(*index);
             }
             DBMsg::AddPlaylist(index) => {
-                if !self.db_search_tracks.is_empty() {
-                    if let Some(track) = self.db_search_tracks.get(*index) {
+                if !self.dw.search_tracks.is_empty() {
+                    if let Some(track) = self.dw.search_tracks.get(*index) {
                         let file = track.file.clone();
                         if let Err(e) = self.playlist_add(&file) {
                             self.mount_error_popup(e.context("playlist add"));
@@ -579,7 +588,7 @@ impl Model {
                 }
             }
             DBMsg::AddAllToPlaylist => {
-                let db_search_tracks = self.db_search_tracks.clone();
+                let db_search_tracks = self.dw.search_tracks.clone();
                 self.playlist_add_all_from_db(&db_search_tracks);
             }
         }
@@ -591,10 +600,10 @@ impl Model {
             LIMsg::TreeBlur => {
                 assert!(self.app.active(&Id::Playlist).is_ok());
             }
-            LIMsg::TreeExtendDir(path) => {
+            LIMsg::TreeStepInto(path) => {
                 self.library_stepinto(path);
             }
-            LIMsg::TreeGoToUpperDir => {
+            LIMsg::TreeStepOut => {
                 self.library_stepout();
             }
             LIMsg::Yank => {
@@ -800,7 +809,7 @@ impl Model {
                 if self.app.mounted(&Id::DeleteConfirmInputPopup) {
                     let _drop = self.app.umount(&Id::DeleteConfirmInputPopup);
                 }
-                if let Err(e) = self.library_delete_song() {
+                if let Err(e) = self.library_delete_node() {
                     self.mount_error_popup(e.context("library delete song"));
                 };
             }
@@ -829,7 +838,8 @@ impl Model {
             }
             PLMsg::LoopModeCycle => {
                 self.command(&PlayerCmd::CycleLoop);
-                self.config.write().player_loop_mode = self.playlist.cycle_loop_mode();
+                self.config_server.write().settings.player.loop_mode =
+                    self.playlist.cycle_loop_mode();
                 self.playlist_update_title();
             }
             PLMsg::PlaylistTableBlurDown => match self.layout {
@@ -858,11 +868,11 @@ impl Model {
                     self.mount_error_popup(e.context("playlist sync playlist"));
                 }
             }
-            PLMsg::CmusLQueue => {
-                self.playlist_add_cmus_lqueue();
+            PLMsg::AddRandomAlbum => {
+                self.playlist_add_random_album();
             }
-            PLMsg::CmusTQueue => {
-                self.playlist_add_cmus_tqueue();
+            PLMsg::AddRandomTracks => {
+                self.playlist_add_random_tracks();
             }
             PLMsg::PlaylistTableBlurUp => match self.layout {
                 TermusicLayout::TreeView => assert!(self.app.active(&Id::Library).is_ok()),
@@ -879,11 +889,11 @@ impl Model {
         if let Some(track) = self.playlist.current_track() {
             if self.layout == TermusicLayout::Podcast {
                 let title = track.title().unwrap_or("Unknown Episode");
-                self.update_show_message_timeout("Current Playing", title, None);
+                self.update_show_message_timeout("Currently Playing", title, None);
                 return;
             }
             let name = track.name().unwrap_or("Unknown Song");
-            self.update_show_message_timeout("Current Playing", name, None);
+            self.update_show_message_timeout("Currently Playing", name, None);
 
             // TODO: is there a better way to update only a single / 2 columns (prev/next) instead of re-doing the whole playist; OR a way to decide at draw-time?
             // sync playlist to update any dynamic parts added to the columns (like current playing symbol)
@@ -944,7 +954,7 @@ impl Model {
             DLMsg::DownloadRunning(url, title) => {
                 self.download_tracker.increase_one(url);
                 self.show_message_timeout_label_help(
-                    &self.download_tracker.message_download_start(title),
+                    self.download_tracker.message_download_start(title),
                     None,
                     None,
                     None,
