@@ -1,13 +1,16 @@
 use crate::ui::components::{
     DBListCriteria, DBListSearchResult, DBListSearchTracks, DownloadSpinner, EpisodeList,
-    ErrorPopup, FeedsList, GSInputPopup, GSTablePopup, GlobalListener, HelpPopup, LabelSpan, Lyric,
-    MessagePopup, MusicLibrary, Playlist, PodcastAddPopup, Progress, QuitPopup,
-    SavePlaylistConfirm, SavePlaylistPopup, Source, YSInputPopup, YSTablePopup,
+    FeedsList, Footer, GSInputPopup, GSTablePopup, GlobalListener, LabelSpan, Lyric, MusicLibrary,
+    Playlist, Progress, Source,
 };
 use crate::ui::model::{ConfigEditorLayout, Model, TermusicLayout};
+use crate::ui::utils::{
+    draw_area_in_absolute, draw_area_in_relative, draw_area_top_right_absolute,
+};
 use crate::ui::Application;
 use anyhow::{bail, Result};
 use std::time::{Duration, Instant};
+use termusiclib::config::SharedTuiSettings;
 /**
  * MIT License
  *
@@ -32,10 +35,7 @@ use std::time::{Duration, Instant};
  * SOFTWARE.
  */
 use termusiclib::types::{DBMsg, Id, IdConfigEditor, IdTagEditor, Msg, PCMsg};
-use termusiclib::utils::{
-    draw_area_in_absolute, draw_area_in_relative, draw_area_top_right_absolute, get_parent_folder,
-};
-use termusicplayback::SharedSettings;
+use termusiclib::utils::get_parent_folder;
 use tui_realm_treeview::Tree;
 use tuirealm::event::NoUserEvent;
 use tuirealm::props::{AttrValue, Attribute, Color, PropPayload, PropValue, TextSpan};
@@ -46,7 +46,7 @@ use tuirealm::{Frame, State, StateValue};
 
 impl Model {
     #[allow(clippy::too_many_lines)]
-    pub fn init_app(tree: &Tree, config: &SharedSettings) -> Application<Id, Msg, NoUserEvent> {
+    pub fn init_app(tree: &Tree, config: &SharedTuiSettings) -> Application<Id, Msg, NoUserEvent> {
         // Setup application
         // NOTE: NoUserEvent is a shorthand to tell tui-realm we're not going to use any custom user event
         // NOTE: the event listener is configured to use the default crossterm input listener and to raise a Tick event each second
@@ -153,7 +153,7 @@ impl Model {
             .mount(
                 Id::GlobalListener,
                 Box::new(GlobalListener::new(config.clone())),
-                Self::subscribe(&config.read().keys),
+                Self::subscribe(&config.read().settings.keys),
             )
             .is_ok());
         // Active library
@@ -172,7 +172,7 @@ impl Model {
                 self.view_tag_editor();
                 return;
             } else if self.app.mounted(&Id::ConfigEditor(IdConfigEditor::Header)) {
-                match self.config_layout {
+                match self.config_editor.layout {
                     ConfigEditorLayout::General => self.view_config_editor_general(),
                     ConfigEditorLayout::Color => self.view_config_editor_color(),
                     ConfigEditorLayout::Key1 => self.view_config_editor_key1(),
@@ -444,47 +444,13 @@ impl Model {
             app.view(&Id::ErrorPopup, f, popup);
         }
     }
-    /// Mount error and give focus to it
-    // This should likely be refactored to be "std::error::Error", but see https://github.com/dtolnay/anyhow/issues/63 on why it was easier this way
-    pub fn mount_error_popup<E: Into<anyhow::Error>>(&mut self, err: E) {
-        assert!(self
-            .app
-            .remount(Id::ErrorPopup, Box::new(ErrorPopup::new(err)), vec![])
-            .is_ok());
-        assert!(self.app.active(&Id::ErrorPopup).is_ok());
-    }
-    /// Mount quit popup
-    pub fn mount_quit_popup(&mut self) {
-        assert!(self
-            .app
-            .remount(
-                Id::QuitPopup,
-                Box::new(QuitPopup::new(self.config.clone())),
-                vec![]
-            )
-            .is_ok());
-        assert!(self.app.active(&Id::QuitPopup).is_ok());
-    }
-    /// Mount help popup
-    pub fn mount_help_popup(&mut self) {
-        assert!(self
-            .app
-            .remount(
-                Id::HelpPopup,
-                Box::new(HelpPopup::new(self.config.clone())),
-                vec![]
-            )
-            .is_ok());
-        self.update_photo().ok();
-        assert!(self.app.active(&Id::HelpPopup).is_ok());
-    }
 
     pub fn mount_search_library(&mut self) {
         assert!(self
             .app
             .remount(
                 Id::GeneralSearchInput,
-                Box::new(GSInputPopup::new(Source::Library, &self.config.read())),
+                Box::new(GSInputPopup::new(Source::Library, &self.config_tui.read())),
                 vec![]
             )
             .is_ok());
@@ -492,7 +458,7 @@ impl Model {
             .app
             .remount(
                 Id::GeneralSearchTable,
-                Box::new(GSTablePopup::new(Source::Library, self.config.clone())),
+                Box::new(GSTablePopup::new(Source::Library, self.config_tui.clone())),
                 vec![]
             )
             .is_ok());
@@ -508,7 +474,7 @@ impl Model {
             .app
             .remount(
                 Id::GeneralSearchInput,
-                Box::new(GSInputPopup::new(Source::Playlist, &self.config.read())),
+                Box::new(GSInputPopup::new(Source::Playlist, &self.config_tui.read())),
                 vec![]
             )
             .is_ok());
@@ -516,7 +482,7 @@ impl Model {
             .app
             .remount(
                 Id::GeneralSearchTable,
-                Box::new(GSTablePopup::new(Source::Playlist, self.config.clone())),
+                Box::new(GSTablePopup::new(Source::Playlist, self.config_tui.clone())),
                 vec![]
             )
             .is_ok());
@@ -531,7 +497,7 @@ impl Model {
             .app
             .remount(
                 Id::GeneralSearchInput,
-                Box::new(GSInputPopup::new(Source::Database, &self.config.read())),
+                Box::new(GSInputPopup::new(Source::Database, &self.config_tui.read())),
                 vec![]
             )
             .is_ok());
@@ -539,7 +505,7 @@ impl Model {
             .app
             .remount(
                 Id::GeneralSearchTable,
-                Box::new(GSTablePopup::new(Source::Database, self.config.clone())),
+                Box::new(GSTablePopup::new(Source::Database, self.config_tui.clone())),
                 vec![]
             )
             .is_ok());
@@ -554,7 +520,7 @@ impl Model {
             .app
             .remount(
                 Id::GeneralSearchInput,
-                Box::new(GSInputPopup::new(Source::Episode, &self.config.read())),
+                Box::new(GSInputPopup::new(Source::Episode, &self.config_tui.read())),
                 vec![]
             )
             .is_ok());
@@ -562,7 +528,7 @@ impl Model {
             .app
             .remount(
                 Id::GeneralSearchTable,
-                Box::new(GSTablePopup::new(Source::Episode, self.config.clone())),
+                Box::new(GSTablePopup::new(Source::Episode, self.config_tui.clone())),
                 vec![]
             )
             .is_ok());
@@ -578,7 +544,7 @@ impl Model {
             .app
             .remount(
                 Id::GeneralSearchInput,
-                Box::new(GSInputPopup::new(Source::Podcast, &self.config.read())),
+                Box::new(GSInputPopup::new(Source::Podcast, &self.config_tui.read())),
                 vec![]
             )
             .is_ok());
@@ -586,7 +552,7 @@ impl Model {
             .app
             .remount(
                 Id::GeneralSearchTable,
-                Box::new(GSTablePopup::new(Source::Podcast, self.config.clone())),
+                Box::new(GSTablePopup::new(Source::Podcast, self.config_tui.clone())),
                 vec![]
             )
             .is_ok());
@@ -597,184 +563,13 @@ impl Model {
         }
     }
 
-    pub fn mount_youtube_search_input(&mut self) {
-        assert!(self
-            .app
-            .remount(
-                Id::YoutubeSearchInputPopup,
-                Box::new(YSInputPopup::new(&self.config.read())),
-                vec![]
-            )
-            .is_ok());
-        assert!(self.app.active(&Id::YoutubeSearchInputPopup).is_ok());
-    }
-
-    pub fn mount_youtube_search_table(&mut self) {
-        assert!(self
-            .app
-            .remount(
-                Id::YoutubeSearchTablePopup,
-                Box::new(YSTablePopup::new(self.config.clone())),
-                vec![]
-            )
-            .is_ok());
-        assert!(self.app.active(&Id::YoutubeSearchTablePopup).is_ok());
-        if let Err(e) = self.update_photo() {
-            self.mount_error_popup(e.context("update_photo"));
-        }
-    }
-    pub fn mount_message(&mut self, title: &str, text: &str) {
-        assert!(self
-            .app
-            .remount(
-                Id::MessagePopup,
-                Box::new(MessagePopup::new(title, text)),
-                vec![]
-            )
-            .is_ok());
-    }
-
-    /// ### `umount_message`
-    ///
-    /// Umount error message
-    pub fn umount_message(&mut self, _title: &str, text: &str) {
-        if let Ok(Some(AttrValue::Payload(PropPayload::Vec(spans)))) =
-            self.app.query(&Id::MessagePopup, Attribute::Text)
-        {
-            if let Some(display_text) = spans.first() {
-                let d = display_text.clone().unwrap_text_span().content;
-                if text.eq(&d) {
-                    self.app.umount(&Id::MessagePopup).ok();
-                }
-            }
-        }
-    }
-
     pub fn mount_label_help(&mut self) {
-        let config = self.config.read();
+        let config = self.config_tui.read();
         self.app
-            .remount(
-                Id::Label,
-                Box::new(LabelSpan::new(
-                    &config,
-                    &[
-                        TextSpan::new(" Version: ")
-                            .fg(config
-                                .style_color_symbol
-                                .library_foreground()
-                                .unwrap_or(Color::Blue))
-                            .bold(),
-                        // maybe consider moving version into Help or Config or its own popup (like a About)
-                        TextSpan::new(env!("TERMUSIC_VERSION"))
-                            .fg(config
-                                .style_color_symbol
-                                .library_highlight()
-                                .unwrap_or(Color::Cyan))
-                            .bold(),
-                        TextSpan::new(" Help: ")
-                            .fg(config
-                                .style_color_symbol
-                                .library_foreground()
-                                .unwrap_or(Color::Blue))
-                            .bold(),
-                        TextSpan::new(format!("<{}>", config.keys.global_help))
-                            .fg(config
-                                .style_color_symbol
-                                .library_highlight()
-                                .unwrap_or(Color::Cyan))
-                            .bold(),
-                        TextSpan::new(" Config: ")
-                            .fg(config
-                                .style_color_symbol
-                                .library_foreground()
-                                .unwrap_or(Color::Blue))
-                            .bold(),
-                        TextSpan::new(format!("<{}>", config.keys.global_config_open))
-                            .fg(config
-                                .style_color_symbol
-                                .library_highlight()
-                                .unwrap_or(Color::Cyan))
-                            .bold(),
-                        TextSpan::new(" Library: ")
-                            .fg(config
-                                .style_color_symbol
-                                .library_foreground()
-                                .unwrap_or(Color::Blue))
-                            .bold(),
-                        TextSpan::new(format!("<{}>", config.keys.global_layout_treeview))
-                            .fg(config
-                                .style_color_symbol
-                                .library_highlight()
-                                .unwrap_or(Color::Cyan))
-                            .bold(),
-                        TextSpan::new(" Database: ")
-                            .fg(config
-                                .style_color_symbol
-                                .library_foreground()
-                                .unwrap_or(Color::Blue))
-                            .bold(),
-                        TextSpan::new(format!("<{}>", config.keys.global_layout_database))
-                            .fg(config
-                                .style_color_symbol
-                                .library_highlight()
-                                .unwrap_or(Color::Cyan))
-                            .bold(),
-                        TextSpan::new(" Podcasts: ")
-                            .fg(config
-                                .style_color_symbol
-                                .library_foreground()
-                                .unwrap_or(Color::Blue))
-                            .bold(),
-                        TextSpan::new(format!("<{}>", config.keys.global_layout_podcast))
-                            .fg(config
-                                .style_color_symbol
-                                .library_highlight()
-                                .unwrap_or(Color::Cyan))
-                            .bold(),
-                    ],
-                )),
-                Vec::default(),
-            )
+            .remount(Id::Label, Box::new(Footer::new(&config)), Vec::default())
             .expect("Expected to remount without error");
     }
 
-    pub fn umount_error_popup(&mut self) {
-        self.app.umount(&Id::ErrorPopup).ok();
-    }
-
-    pub fn umount_youtube_search_table_popup(&mut self) {
-        if self.app.mounted(&Id::YoutubeSearchTablePopup) {
-            assert!(self.app.umount(&Id::YoutubeSearchTablePopup).is_ok());
-        }
-        if let Err(e) = self.update_photo() {
-            self.mount_error_popup(e.context("update_photo"));
-        }
-    }
-
-    pub fn mount_save_playlist(&mut self) -> Result<()> {
-        assert!(self
-            .app
-            .remount(
-                Id::SavePlaylistPopup,
-                Box::new(SavePlaylistPopup::new(
-                    &self.config.read().style_color_symbol
-                )),
-                vec![]
-            )
-            .is_ok());
-
-        self.remount_save_playlist_label("")?;
-        assert!(self.app.active(&Id::SavePlaylistPopup).is_ok());
-        Ok(())
-    }
-
-    pub fn umount_save_playlist(&mut self) {
-        if self.app.mounted(&Id::SavePlaylistPopup) {
-            assert!(self.app.umount(&Id::SavePlaylistPopup).is_ok());
-            assert!(self.app.umount(&Id::SavePlaylistLabel).is_ok());
-        }
-    }
-
     pub fn remount_save_playlist_label(&mut self, filename: &str) -> Result<()> {
         let current_node: String = match self.app.state(&Id::Library).ok().unwrap() {
             State::One(StateValue::String(id)) => id,
@@ -784,7 +579,7 @@ impl Model {
         let mut path_string = get_parent_folder(&current_node);
         path_string.push('/');
 
-        let config = self.config.read();
+        let config = self.config_tui.read();
 
         self.app
             .remount(
@@ -793,23 +588,14 @@ impl Model {
                     &config,
                     &[
                         TextSpan::new("Full name: ")
-                            .fg(config
-                                .style_color_symbol
-                                .library_highlight()
-                                .unwrap_or(Color::Cyan))
+                            .fg(config.settings.theme.fallback_highlight())
                             .bold(),
                         TextSpan::new(path_string)
-                            .fg(config
-                                .style_color_symbol
-                                .library_foreground()
-                                .unwrap_or(Color::Red))
+                            .fg(config.settings.theme.fallback_foreground())
                             .bold(),
                         TextSpan::new(filename).fg(Color::Cyan).bold(),
                         TextSpan::new(".m3u")
-                            .fg(config
-                                .style_color_symbol
-                                .library_foreground()
-                                .unwrap_or(Color::Cyan))
+                            .fg(config.settings.theme.fallback_foreground())
                             .bold(),
                     ],
                 )),
@@ -819,43 +605,6 @@ impl Model {
         Ok(())
     }
 
-    pub fn mount_save_playlist_confirm(&mut self, filename: &str) {
-        assert!(self
-            .app
-            .remount(
-                Id::SavePlaylistConfirm,
-                Box::new(SavePlaylistConfirm::new(self.config.clone(), filename)),
-                vec![]
-            )
-            .is_ok());
-        assert!(self.app.active(&Id::SavePlaylistConfirm).is_ok());
-    }
-
-    pub fn umount_save_playlist_confirm(&mut self) {
-        if self.app.mounted(&Id::SavePlaylistConfirm) {
-            assert!(self.app.umount(&Id::SavePlaylistConfirm).is_ok());
-        }
-    }
-
-    pub fn mount_podcast_add_popup(&mut self) {
-        assert!(self
-            .app
-            .remount(
-                Id::PodcastAddPopup,
-                Box::new(PodcastAddPopup::new(&self.config.read().style_color_symbol)),
-                vec![]
-            )
-            .is_ok());
-
-        assert!(self.app.active(&Id::PodcastAddPopup).is_ok());
-    }
-
-    pub fn umount_podcast_add_popup(&mut self) {
-        if self.app.mounted(&Id::PodcastAddPopup) {
-            assert!(self.app.umount(&Id::PodcastAddPopup).is_ok());
-        }
-    }
-
     pub fn show_message_timeout_label_help<S: AsRef<str>>(
         &mut self,
         active_msg: S,
@@ -863,21 +612,11 @@ impl Model {
         background: Option<Color>,
         timeout: Option<isize>,
     ) {
-        let config = self.config.read();
+        let config = self.config_tui.read();
         let textspan = &[TextSpan::new(active_msg)
-            .fg(foreground.unwrap_or_else(|| {
-                config
-                    .style_color_symbol
-                    .library_highlight()
-                    .unwrap_or(Color::Cyan)
-            }))
+            .fg(foreground.unwrap_or_else(|| config.settings.theme.library_highlight()))
             .bold()
-            .bg(background.unwrap_or_else(|| {
-                config
-                    .style_color_symbol
-                    .library_background()
-                    .unwrap_or(Color::Reset)
-            }))];
+            .bg(background.unwrap_or_else(|| config.settings.theme.library_background()))];
         self.app
             .attr(
                 &Id::Label,
